@@ -9,10 +9,10 @@
    This file does some tests of complex user-defined types. It is
    primarily meant to be run with valgrind as a memory test.
 
-   $Id: tst_h_atts4.c 1177 2011-05-11 17:09:12Z ed $
+   $Id: tst_h_atts4.c 1644 2011-11-14 04:20:19Z dmh $
 */
 
-#include <err_macros.h>
+#include "h5_err_macros.h"
 #include <hdf5.h>
 
 #define MY_CHUNK_CACHE_SIZE 32000000
