@@ -8,7 +8,7 @@
 
    $Id: tst_h_dimscales3.c,v 1.2 2010/06/01 15:34:51 ed Exp $
 */
-#include <err_macros.h>
+#include "h5_err_macros.h"
 #include <hdf5.h>
 #include <H5DSpublic.h>
 #include <hdf5.h>
