@@ -14,6 +14,15 @@ for more info.
 #include <stddef.h> /* size_t, ptrdiff_t */
 #include <errno.h>  /* netcdf functions sometimes return system errors */
 
+/* Required for alloca on Windows */ 
+#if defined(_WIN32) || defined(_WIN64)
+#include <malloc.h>
+#endif
+
+#ifdef _WIN64
+#include <sys/stat.h>
+#endif
+
 /*! The nc_type type is just an int. */
 typedef int nc_type;
 
@@ -43,7 +52,7 @@ extern "C" {
 
 /* The following are use internally in support of user-defines
  * types. They are also the class returned by nc_inq_user_type. */
-#define	NC_VLEN 	13	/**< vlen types */
+#define	NC_VLEN 	13	/**< vlen (variable-length) types */
 #define	NC_OPAQUE 	14	/**< opaque types */
 #define	NC_ENUM 	15	/**< enum types */
 #define	NC_COMPOUND 	16	/**< compound types */
@@ -157,11 +166,10 @@ Use this in mode flags for both nc_create() and nc_open(). */
 #define NC_FORMAT_NETCDF4_CLASSIC  (4)
 /**@}*/
 
-/** Let nc__create() or nc__open() figure out as suitable chunk
- * size. */
+/** Let nc__create() or nc__open() figure out a suitable buffer size. */
 #define NC_SIZEHINT_DEFAULT 0
 
-/** In nc__enddef(), align to the chunk size. */
+/** In nc__enddef(), align to the buffer size. */
 #define NC_ALIGN_CHUNK ((size_t)(-1))
 
 /** Size argument to nc_def_dim() for an unlimited dimension. */
@@ -179,11 +187,11 @@ variable attributes (\ref NC_MAX_ATTRS), the number of variables in
 the file (\ref NC_MAX_VARS), and the length of a name (\ref
 NC_MAX_NAME).
 
-This maximum is enforced by the interface, to facilitate writing
+These maximums are enforced by the interface, to facilitate writing
 applications and utilities.  However, nothing is statically allocated
-to this size internally.
+to these sizes internally.
 
-This maximum is not used for netCDF-4/HDF5 files unless they were
+These maximums are not used for netCDF-4/HDF5 files unless they were
 created with the ::NC_CLASSIC_MODEL flag.
 
 As a rule, NC_MAX_VAR_DIMS <= NC_MAX_DIMS.
@@ -330,7 +338,7 @@ by the desired type. */
 #define NC_EDDS		(-72)      /**< Malformed or inaccessible DDS */
 #define NC_EDATADDS	(-73)      /**< Malformed or inaccessible DATADDS */
 #define NC_EDAPURL	(-74)      /**< Malformed DAP URL */
-#define NC_EDAPCONSTRAINT (-75)      /**< Malformed DAP Constraint*/
+#define NC_EDAPCONSTRAINT (-75)    /**< Malformed DAP Constraint*/
 #define NC_ETRANSLATION (-76)      /**< Untranslatable construct */
 
 /* The following was added in support of netcdf-4. Make all netcdf-4
@@ -392,7 +400,6 @@ by the desired type. */
  */
 
 /* Declaration modifiers for DLL support (MSC et al) */
-
 #if defined(DLL_NETCDF) /* define when library is a DLL */
 #  if defined(DLL_EXPORT) /* define when building the library */
 #   define MSC_EXTRA __declspec(dllexport)
@@ -400,14 +407,19 @@ by the desired type. */
 #   define MSC_EXTRA __declspec(dllimport)
 #  endif
 #include <io.h>
-/*#define lseek _lseeki64
-  #define off_t __int64*/
 #else
 #define MSC_EXTRA
 #endif	/* defined(DLL_NETCDF) */
 
 # define EXTERNL MSC_EXTRA extern
 
+/* Define for 64 bit windows. */
+#if defined (_WIN64)
+#define off_t __int64
+#define size_t unsigned __int64
+#define _OFF_T_DEFINED
+#endif
+
 #if defined(DLL_NETCDF) /* define when library is a DLL */
 EXTERNL int ncerr;
 EXTERNL int ncopts;
@@ -819,6 +831,7 @@ nc_get_var_chunk_cache(int ncid, int varid, size_t *sizep, size_t *nelemsp,
 EXTERNL int
 nc_redef(int ncid);
 
+/* Is this ever used? */
 EXTERNL int
 nc__enddef(int ncid, size_t h_minfree, size_t v_align,
 	size_t v_minfree, size_t r_align);
