@@ -10,5 +10,12 @@
 int
 ast_catch(int code)
 {
+    if(code != AST_NOERR && code != AST_EOF) 
+	ast_breakpoint(code);
     return code;
 }
+
+void
+ast_breakpoint(int code)
+{
+}
