@@ -8,6 +8,12 @@
 #ifndef CRUTIL_H
 #define CRUTIL_H
 
+/*Forward*/
+struct CRnode;
+
+
 extern int crstrindex(char* s, char* match);
 
+extern void crcollectnodepath(struct CRnode*, NClist* path);
+
 #endif /*CRUTIL_H*/
