@@ -1,6 +1,6 @@
-#ifdef HAVE_CONFIG_H
+//#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
+//#endif
 
 #include <stdlib.h>
 #include <stdio.h>
