@@ -8,47 +8,56 @@
 #include "daputil.h"
 #include "dapdump.h"
 
+#ifdef DAPDEBUG
+extern char* ocfqn(OCddsnode);
+#endif
+
 CDFnode* v4node = NULL;
 
 /* Forward*/
 static NCerror sequencecheck3r(CDFnode* node, NClist* vars, CDFnode* topseq);
-static NCerror regrid3r(CDFnode*, CDFnode*, NClist*);
-static NCerror testregrid3(CDFnode* node, CDFnode* template, NClist*);
-static CDFnode* makenewgrid3(CDFnode* node, CDFnode* template);
-static NCerror regridinsert(CDFnode* newgrid, CDFnode* node);
-static NCerror regridremove(CDFnode* newgrid, CDFnode* node);
+static NCerror restruct3r(CDFnode*, CDFnode*, NClist*);
+static NCerror repairgrids(NClist*);
+static NCerror structwrap3(CDFnode*, CDFnode*, int, CDFnode*, int);
+static int findin(CDFnode* parent, CDFnode* child);
+static CDFnode* makenewstruct3(CDFnode* node, CDFnode* template);
 static NCerror mapnodes3r(CDFnode*, CDFnode*, int depth);
 static NCerror mapfcn(CDFnode* dstnode, CDFnode* srcnode);
 static NCerror definedimsetplus3(NCDAPCOMMON* nccomm, CDFnode* node);
 static NCerror definedimsetall3(NCDAPCOMMON* nccomm, CDFnode* node);
+static NCerror definedimsetall3(NCDAPCOMMON* nccomm, CDFnode* node);
+static NCerror definedimsettrans3(NCDAPCOMMON* nccomm, CDFnode* node);
 
 /* Accumulate useful node sets  */
 NCerror
-computecdfnodesets3(NCDAPCOMMON* nccomm)
+computecdfnodesets3(NCDAPCOMMON* nccomm, CDFtree* tree)
 {
     unsigned int i;
-    NClist* varnodes = nclistnew(); 
-    NClist* allnodes = nccomm->cdf.ddsroot->tree->nodes;
+    NClist* varnodes;
+    NClist* allnodes;
 
-    if(nccomm->cdf.seqnodes == NULL) nccomm->cdf.seqnodes = nclistnew();
-    if(nccomm->cdf.gridnodes == NULL) nccomm->cdf.gridnodes = nclistnew();
-    nclistclear(nccomm->cdf.seqnodes);
-    nclistclear(nccomm->cdf.gridnodes);
+    allnodes = tree->nodes;
+    varnodes = nclistnew(); 
+
+    if(tree->seqnodes == NULL) tree->seqnodes = nclistnew();
+    if(tree->gridnodes == NULL) tree->gridnodes = nclistnew();
+    nclistclear(tree->seqnodes);
+    nclistclear(tree->gridnodes);
 
     computevarnodes3(nccomm,allnodes,varnodes);
-    nclistfree(nccomm->cdf.varnodes);
-    nccomm->cdf.varnodes = varnodes;
+    nclistfree(tree->varnodes);
+    tree->varnodes = varnodes;
+    varnodes = NULL;
 
     /* Now compute other sets of interest */
     for(i=0;i<nclistlength(allnodes);i++) {
 	CDFnode* node = (CDFnode*)nclistget(allnodes,i);
-	if(!node->visible) continue;
 	switch (node->nctype) {
 	case NC_Sequence:
-	    nclistpush(nccomm->cdf.seqnodes,(ncelem)node);
+	    nclistpush(tree->seqnodes,(void*)node);
 	    break;
 	case NC_Grid:
-	    nclistpush(nccomm->cdf.gridnodes,(ncelem)node);
+	    nclistpush(tree->gridnodes,(void*)node);
 	    break;
 	default: break;
 	}
@@ -63,15 +72,16 @@ computevarnodes3(NCDAPCOMMON* nccomm, NClist* allnodes, NClist* varnodes)
     NClist* allvarnodes = nclistnew();
     for(i=0;i<nclistlength(allnodes);i++) {
 	CDFnode* node = (CDFnode*)nclistget(allnodes,i);
+#if 0
 	/* If this node has a bad name, repair it */
 	if(dap_badname(node->ocname)) {
 	    char* newname = dap_repairname(node->ocname);
 	    nullfree(node->ocname);
 	    node->ocname = newname;
 	}
-	if(!node->visible) continue;
-	if(node->nctype == NC_Primitive)
-	    nclistpush(allvarnodes,(ncelem)node);
+#endif
+	if(node->nctype == NC_Atomic)
+	    nclistpush(allvarnodes,(void*)node);
     }
     /* Further process the variable nodes to get the final set */
     /* Use toplevel vars first */
@@ -80,8 +90,8 @@ computevarnodes3(NCDAPCOMMON* nccomm, NClist* allnodes, NClist* varnodes)
 	CDFnode* node = (CDFnode*)nclistget(allvarnodes,i);
 	if(node == NULL) continue;
         if(daptoplevel(node)) {
-	    nclistpush(varnodes,(ncelem)node);
-	    nclistset(allvarnodes,i,(ncelem)NULL);
+	    nclistpush(varnodes,(void*)node);
+	    nclistset(allvarnodes,i,(void*)NULL);
 	}
     }
     /*... then grid arrays and maps.
@@ -92,19 +102,19 @@ computevarnodes3(NCDAPCOMMON* nccomm, NClist* allnodes, NClist* varnodes)
 	CDFnode* node = (CDFnode*)nclistget(allvarnodes,i);
 	if(node == NULL) continue;
 	if(dapgridarray(node)) {
-	    nclistpush(varnodes,(ncelem)node);
-	    nclistset(allvarnodes,i,(ncelem)NULL);
+	    nclistpush(varnodes,(void*)node);
+	    nclistset(allvarnodes,i,(void*)NULL);
         } else if(dapgridmap(node)) {
 	    if(!FLAGSET(nccomm->controls,NCF_NCDAP))
-		nclistpush(varnodes,(ncelem)node);
-	    nclistset(allvarnodes,i,(ncelem)NULL);
+		nclistpush(varnodes,(void*)node);
+	    nclistset(allvarnodes,i,(void*)NULL);
 	}
     }
     /*... then all others */
     for(i=0;i<len;i++) {
 	CDFnode* node = (CDFnode*)nclistget(allvarnodes,i);
 	if(node == NULL) continue;
-        nclistpush(varnodes,(ncelem)node);
+        nclistpush(varnodes,(void*)node);
     }
     nclistfree(allvarnodes);
 #ifdef DEBUG2
@@ -121,7 +131,7 @@ NCerror
 fixgrids3(NCDAPCOMMON* nccomm)
 {
     unsigned int i;
-    NClist* gridnodes = nccomm->cdf.gridnodes;
+    NClist* gridnodes = nccomm->cdf.ddsroot->tree->gridnodes;
 
     for(i=0;i<nclistlength(gridnodes);i++) {
         CDFnode* grid = (CDFnode*)nclistget(gridnodes,i);
@@ -218,7 +228,8 @@ A sequence is usable iff:
 NCerror
 sequencecheck3(NCDAPCOMMON* nccomm)
 {
-    (void)sequencecheck3r(nccomm->cdf.ddsroot,nccomm->cdf.varnodes,NULL);    
+    (void)sequencecheck3r(nccomm->cdf.ddsroot,
+                          nccomm->cdf.ddsroot->tree->varnodes,NULL);    
     return NC_NOERR;
 }
 
@@ -252,7 +263,7 @@ sequencecheck3r(CDFnode* node, NClist* vars, CDFnode* topseq)
 	    node->usesequence = 0;
 	    err = NC_EINVAL;
 	}
-    } else if(nclistcontains(vars,(ncelem)node)) {
+    } else if(nclistcontains(vars,(void*)node)) {
 	/* If we reach a leaf, then topseq is usable, so save it */
 	node->array.sequence = topseq;
     } else { /* Some kind of non-sequence container node with no dimensions */
@@ -268,267 +279,231 @@ sequencecheck3r(CDFnode* node, NClist* vars, CDFnode* topseq)
 }
 
 /*
-OPeNDAP is in the process of changing servers so that
-partial grids are converted to structures.  However, not all
-servers do this: some elide the grid altogether, which can
-lead to ambiguities.  Handle this last case by attempting to
-convert the elided case to look like the newer structure
-case.  [for some reason, this code has been difficult to get
-right; I have rewritten 6 times and it probably is still not
-right.]
+Originally, if one did a constraint on a Grid such that only
+one array or map in the grid was returned, that element was
+returned as a top level variable.  This is incorrect because
+it loses the Grid scope information.
+
+Eventually, this behavior was changed so that such partial
+grids are converted to structures where the structure name
+is the grid name. This preserves the proper scoping.
+However, it is still the case that some servers do the old
+behavior.
+
+The rules that most old-style servers appear to adhere to are these.
+1. Asking for just a grid array or a single grid map
+   returns just the array not wrapped in a structure.
+2. Asking for a subset of the fields (array plus map) of a grid
+   returns those fields wrapped in a structure.
+3. However, there is an odd situation: asking for a grid array
+   plus any subset of maps that includes the last map in the grid
+   returns a malformed grid. This is clearly a bug.
+
+For case 1, we insert a structure node so that case 1 is consistent
+with case 2. Case 3 should cause an error with a malformed grid.
+
+[Note: for some reason, this code has been difficult to get right;
+I have rewritten 6 times and it probably is still not right.]
+[2/25/2013 Sigh! Previous fixes have introducted another bug,
+so now we fix the fix.]
 
 Input is
 (1) the root of the dds that needs to be re-gridded
 (2) the full datadds tree that defines where the grids are.
 (3) the projections that were used to produce (1) from (2).
+
 */
 
 NCerror
-regrid3(CDFnode* ddsroot, CDFnode* template, NClist* projections)
+restruct3(CDFnode* ddsroot, CDFnode* template, NClist* projections)
 {
     NCerror ncstat = NC_NOERR;
-    NClist* newgrids = nclistnew();
+    NClist* repairs = nclistnew();
 
-    /* The current regrid assumes that the ddsroot tree
+    /* The current restruct assumes that the ddsroot tree
        has missing grids compared to the template.
        It is also assumed that order of the nodes
        in the ddsroot is the same as in the template.
     */
-    if(ddsroot->tree->regridded) return NC_NOERR;
+    if(ddsroot->tree->restructed) return NC_NOERR;
 
 #ifdef DEBUG
-fprintf(stderr,"regrid: ddsroot=%s\n",dumptree(ddsroot));
-fprintf(stderr,"regrid: template=%s\n",dumptree(template));
+fprintf(stderr,"restruct: ddsroot=%s\n",dumptree(ddsroot));
+fprintf(stderr,"restruct: template=%s\n",dumptree(template));
 #endif
 
-
-#ifdef PROJECTED
-    /* turn off the projection tag for all nodes */
-    unprojected3(template->tree->nodes);
-    /* Set the projection flag for all paths of all nodes
-       that are referenced in the projections that produced ddsroot.
-       This includes containers and subnodes. If there are no
-       projections then mark all nodes 
-    */
-     projectall3(template->tree->nodes);
-#endif
-
-    if(simplenodematch34(ddsroot,template)) {
-        ncstat = regrid3r(ddsroot,template,newgrids);
-        ddsroot->tree->regridded = 1;
-    } else
-	ncstat = NC_EINVAL;
-    nclistfree(newgrids);
-    return ncstat;
-}
-
-#ifdef PROJECTED
-static void
-unprojected3(NClist* nodes)
-{
-    int i;
-    for(i=0;i<nclistlength(nodes);i++) {
-	CDFnode* node = (CDFnode*)nclistget(nodes,i);
-	node->projected = 0;
+    /* Match roots */
+    if(!simplenodematch34(ddsroot,template))
+	ncstat = NC_EDATADDS;
+    else if(!restruct3r(ddsroot,template,repairs))
+	ncstat = NC_EDATADDS;
+    else if(nclistlength(repairs) > 0) {
+	/* Do the repairs */
+	ncstat = repairgrids(repairs);
     }
-}
 
-static void
-projectall3(NClist* nodes)
-{
-    int i;
-    for(i=0;i<nclistlength(nodes);i++) {
-	CDFnode* node = (CDFnode*)nclistget(nodes,i);
-	node->projected = 1;
-    }
-}
+    if(repairs)
+      nclistfree(repairs);
 
-static void
-projection3r(CDFnode* node)
-{
-    int i;
-    NClist* path = nclistnew();
-    collectnodepath3(node,path,!WITHDATASET);
-    for(i=0;i<nclistlength(path);i++) {
-        CDFnode* pathnode = (CDFnode*)nclistget(path,i);
-#ifdef DEBUG
-if(pathnode->projected == 0)
-fprintf(stderr,"projection: %s\n",makesimplepathstring3(pathnode));
-#endif
-	pathnode->projected = 1;
-    }
-    /* Now tag everything below me */
-    for(i=0;i<nclistlength(node->subnodes);i++) {
-	CDFnode* subnode = (CDFnode*)nclistget(node->subnodes,i);
-	projection3r(subnode);
-    }
-    nclistfree(path);
-}
-#endif /*PROJECTED*/
-
-/*
-Add in virtual structure nodes so that
-old style constrainted DDS and DATADDS
-look like the new style with structures.
-*/
-static NCerror
-regrid3r(CDFnode* node, CDFnode* template, NClist* gridnodes)
-{
-    unsigned int inode, itemp;
-    NCerror ncstat = NC_NOERR;
-
-    /* Try to match node's subnodes to a subset of the
-       template subnodes
-    */
-#ifdef DEBUG
-fprintf(stderr,"regrid: matched: %s -> %s\n",
-node->ocname,template->ocname);
-#endif
-    for(inode=0;inode<nclistlength(node->subnodes);inode++) {
-        CDFnode* subnode = (CDFnode*)nclistget(node->subnodes,inode);
-	int match = 0;
-        for(itemp=0;itemp<nclistlength(template->subnodes);itemp++) {
-            CDFnode* subtemp = (CDFnode*)nclistget(template->subnodes,itemp);
-	    if(
-#ifdef PROJECTED
-		subtemp->projected &&
-#endif
-		simplenodematch34(subnode,subtemp)) {
-                ncstat = regrid3r(subnode,subtemp,gridnodes);
-                if(ncstat != NC_NOERR) return THROW(ncstat);
-		match = 1;
-#ifdef PROJECTED
-		subtemp->projected = 0; /*make sure we dont reuse this node*/
-#endif
-		break;
-	    }
-        }
-	if(!match) { /* subnode has no match */
-	    /* ok, see if we can regrid */
-            for(itemp=0;itemp<nclistlength(template->subnodes);itemp++) {
-                CDFnode* subtemp = (CDFnode*)nclistget(template->subnodes,itemp);
-#ifdef DEBUG
-fprintf(stderr,"regrid: inside: %s.%s :: %s.%s\n",
-node->ocname,subnode->ocname,
-template->ocname,subtemp->ocname);
-#endif
-		if(subtemp->nctype != NC_Grid)
-		    continue;
-#ifdef PROJECTED
-		if(!subtemp->projected) continue;
-#endif
-		ncstat = testregrid3(subnode,subtemp,gridnodes);
-                if(ncstat == NC_NOERR) {match=1; break;}
-	    }
-	    if(!match) {/* really no match */
-	        ncstat = THROW(NC_EDDS); /* no match */
-	    }
-	}
-    }
     return THROW(ncstat);
 }
 
-/* See if this node can match a subnode of the template
-   as a grid, and if so, then rebuild the node graph.
+/*
+Locate nodes in the tree rooted at node
+that correspond to a single grid field in the template
+when the template is a grid.
+Wrap that grid field in a synthesized structure.
+
+The key thing to look for is the case where
+we have an atomic variable that appear where
+we expected a grid.
+
 */
-static NCerror
-testregrid3(CDFnode* node, CDFnode* template, NClist* gridnodes)
+
+static int
+restruct3r(CDFnode* parentnode, CDFnode* templateparent, NClist* repairlist)
 {
-    int i,match;
-    NCerror ncstat = NC_NOERR;
-    ASSERT((template->nctype == NC_Grid));
-    { /* try to match inside the grid */
-        for(match=0,i=0;i<nclistlength(template->subnodes);i++) {
-            CDFnode* gridelem = (CDFnode*)nclistget(template->subnodes,i);
-            if(!simplenodematch34(gridelem,node))
-		continue;
-            ncstat = regrid3r(node,gridelem,gridnodes);
-            if(ncstat == NC_NOERR) {
-                /* create new grid node if not already created */
-                CDFnode* newgrid = NULL;
-	        match = 1;
-                for(i=0;i<nclistlength(gridnodes);i++) {
-                    newgrid = (CDFnode*)nclistget(gridnodes,i);
-                    if(newgrid->template == template) break;
-                    newgrid = NULL;
+    int index, i, j, match;
+
+#ifdef DEBUG
+fprintf(stderr,"restruct: matched: %s -> %s\n",
+ocfqn(parentnode->ocnode),ocfqn(templateparent->ocnode));
+#endif
+
+    /* walk each node child and locate its match
+       in the template's children; recurse on matches,
+       non-matches may be nodes needing wrapping.
+    */
+
+    for(index=0;index<nclistlength(parentnode->subnodes);index++) {
+        CDFnode* subnode = (CDFnode*)nclistget(parentnode->subnodes,index);
+	CDFnode* matchnode = NULL;
+
+	/* Look for a matching template node with same ocname */
+        for(i=0;i<nclistlength(templateparent->subnodes);i++) {
+            CDFnode* subtemp = (CDFnode*)nclistget(templateparent->subnodes,i);
+	    if(strcmp(subnode->ocname,subtemp->ocname) == 0) {
+		matchnode = subtemp;
+		break;
+	    }
+	}
+#ifdef DEBUG
+fprintf(stderr,"restruct: candidate: %s -> %s\n",
+ocfqn(subnode->ocnode),ocfqn(matchnode->ocnode));
+#endif
+	if(simplenodematch34(subnode,matchnode)) {
+	    /* this subnode of the node matches the corresponding
+               node of the template, so it is ok =>
+               recurse looking for nested mis-matches
+            */
+	    if(!restruct3r(subnode,matchnode,repairlist))
+		return 0;
+	} else {
+            /* If we do not have a direct match, then we need to look
+               at all the grids to see if this node matches a field
+               in one of the grids
+            */
+            for(match=0,i=0;!match && i<nclistlength(templateparent->subnodes);i++) {
+                CDFnode* subtemp = (CDFnode*)nclistget(templateparent->subnodes,i);
+                if(subtemp->nctype == NC_Grid) { /* look inside */
+                    for(j=0;j<nclistlength(templateparent->subnodes);j++) {
+                        CDFnode* gridfield = (CDFnode*)nclistget(subtemp->subnodes,j);
+                        if(simplenodematch34(subnode,gridfield)) {
+                            /* We need to do this repair */
+                            nclistpush(repairlist,(void*)subnode);
+                            nclistpush(repairlist,(void*)gridfield);
+                            match = 1;
+                            break;
+                        }
+                    }
                 }
-                if(newgrid == NULL) {
-                    newgrid = makenewgrid3(node,template);
-                    if(newgrid == NULL) {ncstat = NC_ENOMEM; goto done;}
-                    /* Insert the grid into node's parent */
-                    regridinsert(newgrid,node);
-                    nclistpush(gridnodes,(ncelem)newgrid);
-                    nclistpush(node->root->tree->nodes,(ncelem)newgrid);
-                } 
-                regridremove(newgrid, node);
-                node->container = newgrid;
-                nclistpush(newgrid->subnodes,(ncelem)node);
-                break; /* done with node */
             }
-        }
+	    if(!match) return 0; /* we failed */
+	}
+    }
+    return 1; /* we matched everything at this level */
+}
+
+/* Wrap the node wrt the template grid or template struct */
+
+static NCerror
+repairgrids(NClist* repairlist)
+{
+    NCerror ncstat = NC_NOERR;
+    int i;
+    assert(nclistlength(repairlist) % 2 == 0);
+    for(i=0;i<nclistlength(repairlist);i+=2) {
+	CDFnode* node = (CDFnode*)nclistget(repairlist,i);
+	CDFnode* template = (CDFnode*)nclistget(repairlist,i+1);
+	int index = findin(node->container,node);
+	int tindex = findin(template->container,template);
+	ncstat = structwrap3(node,node->container,index,
+                             template->container,tindex);
+#ifdef DEBUG
+fprintf(stderr,"repairgrids: %s -> %s\n",
+ocfqn(node->ocnode),ocfqn(template->ocnode));
+#endif
+
     }
-    if(!match) ncstat = NC_EDDS;
-done:
     return ncstat;
 }
 
+static NCerror
+structwrap3(CDFnode* node, CDFnode* parent, int parentindex,
+                           CDFnode* templategrid, int gridindex)
+{
+    CDFnode* newstruct;
 
+    ASSERT((templategrid->nctype == NC_Grid));
+
+    newstruct = makenewstruct3(node,templategrid);
+    if(newstruct == NULL) {return THROW(NC_ENOMEM);}
+
+    /* replace the node with the new structure
+       in the parent's list of children*/
+    nclistset(parent->subnodes,parentindex,(void*)newstruct);
+
+    /* Update the list of all nodes in the tree */
+    nclistpush(node->root->tree->nodes,(void*)newstruct);
+    return NC_NOERR;
+}
+
+static int
+findin(CDFnode* parent, CDFnode* child)
+{
+    int i;
+    NClist* subnodes = parent->subnodes;
+    for(i=0;i<nclistlength(subnodes);i++) {
+	if(nclistget(subnodes,i) == child)
+	    return i;
+    }
+    return -1;
+}
+
+/* Create a structure to surround projected grid array or map;
+   this occurs because some servers (that means you ferret and you thredds!)
+   do not adhere to the DAP2 protocol spec.
+*/
+  
 static CDFnode*
-makenewgrid3(CDFnode* node, CDFnode* template)
+makenewstruct3(CDFnode* node, CDFnode* templatenode)
 {
-    CDFnode* newgrid;
-    newgrid = (CDFnode*)calloc(1,sizeof(CDFnode));
-    if(newgrid == NULL) return NULL;
-    memset((void*)newgrid,0,sizeof(CDFnode));
-    newgrid->virtual = 1;
-    newgrid->ocname = nulldup(template->ocname);
-    newgrid->ncbasename = nulldup(template->ncbasename);
-    newgrid->nctype = NC_Grid;
-    newgrid->subnodes = nclistnew();
-    newgrid->container = node->container;
-    newgrid->template = template;
-    return newgrid;
+    CDFnode* newstruct = (CDFnode*)calloc(1,sizeof(CDFnode));
+    if(newstruct == NULL) return NULL;
+    newstruct->nctype = NC_Structure;
+    newstruct->nc_virtual = 1;
+    newstruct->ocname = nulldup(templatenode->ocname);
+    newstruct->ocnode = templatenode->ocnode;
+    newstruct->ncbasename = nulldup(templatenode->ncbasename);
+    newstruct->subnodes = nclistnew();
+    newstruct->container = node->container;
+    newstruct->template = templatenode;
+    node->container = newstruct;
+    nclistpush(newstruct->subnodes,(void*)node);
+    return newstruct;
 }
 
-static NCerror
-regridinsert(CDFnode* newgrid, CDFnode* node)
-{
-    int i;
-    CDFnode* parent;
-    /* Locate the index of the node in its current parent */
-    parent = node->container;
-    for(i=0;i<nclistlength(parent->subnodes);i++) {
-	CDFnode* subnode = (CDFnode*)nclistget(parent->subnodes,i);
-	if(subnode == node) {
-	    /* Insert the grid right before this node */
-	    nclistinsert(parent->subnodes,i,(ncelem)newgrid);
-	    return NC_NOERR;
-	}
-    }
-    PANIC("regridinsert failure");
-    return NC_EINVAL;
-}
-
-static NCerror
-regridremove(CDFnode* newgrid, CDFnode* node)
-{
-    int i;
-    CDFnode* parent;
-    /* Locate the index of the node in its current parent and remove */
-    parent = node->container;
-    for(i=0;i<nclistlength(parent->subnodes);i++) {
-	CDFnode* subnode = (CDFnode*)nclistget(parent->subnodes,i);
-	if(subnode == node) {
-	    nclistremove(parent->subnodes,i);
-	    return NC_NOERR;
-	}
-    }
-    PANIC("regridremove failure");
-    return NC_EINVAL;
-}    
-
 /**
-
 Make the constrained dds nodes (root)
 point to the corresponding unconstrained
 dds nodes (fullroot).
@@ -557,17 +532,31 @@ mapnodes3r(CDFnode* connode, CDFnode* fullnode, int depth)
     ASSERT((simplenodematch34(connode,fullnode)));
     
 #ifdef DEBUG
-{
+  {
 char* path1 = makecdfpathstring3(fullnode,".");
-char * path2 = makecdfpathstring3(connode,".");
+char* path2 = makecdfpathstring3(connode,".");
 fprintf(stderr,"mapnode: %s->%s\n",path1,path2);
 nullfree(path1); nullfree(path2);
-}
+  }
 #endif
 
     /* Map node */
     mapfcn(connode,fullnode);
 
+#if 0
+  {
+    int i;
+    for(i=0;i<nclistlength(fullnode->subnodes);i++) {
+	CDFnode* n = (CDFnode*)nclistget(fullnode->subnodes,i);
+	fprintf(stderr,"fullnode.subnode[%d]: (%d) %s\n",i,n->nctype,n->ocname);
+    }
+    for(i=0;i<nclistlength(connode->subnodes);i++) {
+	CDFnode* n = (CDFnode*)nclistget(connode->subnodes,i);
+	fprintf(stderr,"connode.subnode[%d]: (%d) %s\n",i,n->nctype,n->ocname);
+    }
+  }
+#endif
+
     /* Try to match connode subnodes against fullnode subnodes */
     ASSERT(nclistlength(connode->subnodes) <= nclistlength(fullnode->subnodes));
 
@@ -594,7 +583,6 @@ static NCerror
 mapfcn(CDFnode* dstnode, CDFnode* srcnode)
 {
     /* Mark node as having been mapped */
-    dstnode->visible = 1;
     dstnode->basenode = srcnode;
     return NC_NOERR;
 }
@@ -607,7 +595,6 @@ unmap3(CDFnode* root)
     for(i=0;i<nclistlength(tree->nodes);i++) {
 	CDFnode* node = (CDFnode*)nclistget(tree->nodes,i);
 	node->basenode = NULL;
-        node->visible = 0;
     }
 }
 
@@ -658,9 +645,9 @@ clonedim(NCDAPCOMMON* nccomm, CDFnode* dim, CDFnode* var)
 {
     CDFnode* clone;
     clone = makecdfnode34(nccomm,dim->ocname,OC_Dimension,
-			  OCNULL,dim->container);
+			  NULL,dim->container);
     /* Record its existence */
-    nclistpush(dim->container->root->tree->nodes,(ncelem)clone);
+    nclistpush(dim->container->root->tree->nodes,(void*)clone);
     clone->dim = dim->dim; /* copy most everything */
     clone->dim.dimflags |= CDFDIMCLONE;
     clone->dim.array = var;
@@ -674,14 +661,14 @@ clonedimset3(NCDAPCOMMON* nccomm, NClist* dimset, CDFnode* var)
     int i;
     for(i=0;i<nclistlength(dimset);i++) {
 	CDFnode* dim = (CDFnode*)nclistget(dimset,i);
-	nclistpush(result,(ncelem)clonedim(nccomm,dim,var));
+	nclistpush(result,(void*)clonedim(nccomm,dim,var));
     }
     return result;
 }
 
 /* Define the dimsetplus list for a node */
 static NCerror
-definedimsetplus3(NCDAPCOMMON* nccomm, CDFnode* node)
+definedimsetplus3(NCDAPCOMMON* nccomm/*notused*/, CDFnode* node)
 {
     int ncstat = NC_NOERR;
     NClist* dimset;
@@ -696,11 +683,11 @@ definedimsetplus3(NCDAPCOMMON* nccomm, CDFnode* node)
     /* Insert the sequence or string dims */
     if(node->array.stringdim != NULL) {
 	clone = node->array.stringdim;
-        nclistpush(dimset,(ncelem)clone);
+        nclistpush(dimset,(void*)clone);
     }
     if(node->array.seqdim != NULL) {
 	clone = node->array.seqdim;
-        nclistpush(dimset,(ncelem)clone);
+        nclistpush(dimset,(void*)clone);
     }
     node->array.dimsetplus = dimset;
     return ncstat;
@@ -708,19 +695,22 @@ definedimsetplus3(NCDAPCOMMON* nccomm, CDFnode* node)
 
 /* Define the dimsetall list for a node */
 static NCerror
-definedimsetall3(NCDAPCOMMON* nccomm, CDFnode* node)
+definedimsetall3(NCDAPCOMMON* nccomm/*notused*/, CDFnode* node)
 {
     int i;
     int ncstat = NC_NOERR;
     NClist* dimsetall;
 
-    ASSERT(node->array.dimsetall == NULL);
+    /* Because of upward recursion (see below) the dimsetall may
+       already be defined */
+    if(node->array.dimsetall != NULL)
+	return ncstat;
     if(node->container != NULL) {
         if(node->container->array.dimsetall == NULL) {
 #ifdef DEBUG1
-fprintf(stderr,"dimsetall: recurse %s\n",node->container->ocname);
+fprintf(stderr,"dimsetall: recurse to container%s\n",node->container->ocname);
 #endif
-	    ncstat = definedimsetall3(nccomm,node->container);
+    	    ncstat = definedimsetall3(nccomm,node->container);
 	    if(ncstat != NC_NOERR) return ncstat;
         }
 	/* We need to clone the parent dimensions because we will be assigning
@@ -731,25 +721,69 @@ fprintf(stderr,"dimsetall: recurse %s\n",node->container->ocname);
     // concat parentall and dimset;
     for(i=0;i<nclistlength(node->array.dimsetplus);i++) {
 	CDFnode* clone = (CDFnode*)nclistget(node->array.dimsetplus,i);
-	nclistpush(dimsetall,(ncelem)clone);
+	nclistpush(dimsetall,(void*)clone);
     }
     node->array.dimsetall = dimsetall;
 #ifdef DEBUG1
-fprintf(stderr,"dimsetall: |%s|=%d\n",node->ocname,nclistlength(dimsetall));
+fprintf(stderr,"dimsetall: |%s|=%d\n",node->ocname,(int)nclistlength(dimsetall));
 #endif
     return ncstat;
 }
 
-/* Define the dimsetplus and dimsetall lists for
-   all nodes with dimensions
-*/
-NCerror
-definedimsets3(NCDAPCOMMON* nccomm)
+/* Define the dimsettrans list for a node */
+static NCerror
+definedimsettrans3(NCDAPCOMMON* nccomm/*notused*/, CDFnode* node)
 {
     int i;
     int ncstat = NC_NOERR;
-    NClist* allnodes = nccomm->cdf.ddsroot->tree->nodes;
+    NClist* dimsettrans;
 
+    /* Because of upward recursion (see below) the dimsettrans may
+       already be defined */
+    if(node->array.dimsettrans != NULL)
+	return ncstat;
+    if(node->container != NULL) {
+        if(node->container->array.dimsettrans == NULL) {
+#ifdef DEBUG1
+fprintf(stderr,"dimsettrans: recurse to container%s\n",node->container->ocname);
+#endif
+	    ncstat = definedimsettrans3(nccomm,node->container);
+	    if(ncstat != NC_NOERR) return ncstat;
+        }
+	/* We need to clone the parent dimensions because we will be assigning
+           indices vis-a-vis this variable */
+        dimsettrans = clonedimset3(nccomm,node->container->array.dimsettrans,node);
+    } else
+	dimsettrans = nclistnew();
+    // concat parent dimset0 and dimset;
+    for(i=0;i<nclistlength(node->array.dimset0);i++) {
+	CDFnode* clone = (CDFnode*)nclistget(node->array.dimset0,i);
+	nclistpush(dimsettrans,(void*)clone);
+    }
+    node->array.dimsettrans = dimsettrans;
+#ifdef DEBUG1
+fprintf(stderr,"dimsettrans: |%s|=%d\n",node->ocname,(int)nclistlength(dimsettrans));
+#endif
+    return ncstat;
+}
+
+/* Define the dimsetplus, dimsettrans, and dimsetall lists for
+   all nodes with dimensions
+*/
+NCerror
+definedimsets3(NCDAPCOMMON* nccomm, CDFtree* tree)
+{
+    int i;
+    int ncstat = NC_NOERR;
+    NClist* allnodes = tree->nodes;
+
+    for(i=0;i<nclistlength(allnodes);i++) {
+	CDFnode* rankednode = (CDFnode*)nclistget(allnodes,i);
+	if(rankednode->nctype == NC_Dimension) continue; //ignore
+	ASSERT((rankednode->array.dimsettrans == NULL));
+	ncstat = definedimsettrans3(nccomm,rankednode);
+	if(ncstat != NC_NOERR) return ncstat;
+    }
     for(i=0;i<nclistlength(allnodes);i++) {
 	CDFnode* rankednode = (CDFnode*)nclistget(allnodes,i);
 	if(rankednode->nctype == NC_Dimension) continue; //ignore
@@ -760,7 +794,6 @@ definedimsets3(NCDAPCOMMON* nccomm)
     for(i=0;i<nclistlength(allnodes);i++) {
 	CDFnode* rankednode = (CDFnode*)nclistget(allnodes,i);
 	if(rankednode->nctype == NC_Dimension) continue; //ignore
-	ASSERT((rankednode->array.dimsetall == NULL));
 	ASSERT((rankednode->array.dimsetplus != NULL));
 	ncstat = definedimsetall3(nccomm,rankednode);
 	if(ncstat != NC_NOERR) return ncstat;
