@@ -20,8 +20,8 @@ extern CDFnode* v4node;
 
 /* Define the set of protocols known to be constrainable */
 static char* constrainableprotocols[] = {"http", "https",NULL};
-static NCerror buildcdftree34r(NCDAPCOMMON*,OCobject,CDFnode*,CDFtree*,CDFnode**);
-static void defdimensions(OCobject, CDFnode*, NCDAPCOMMON*, CDFtree*);
+static NCerror buildcdftree34r(NCDAPCOMMON*,OCddsnode,CDFnode*,CDFtree*,CDFnode**);
+static void defdimensions(OCddsnode, CDFnode*, NCDAPCOMMON*, CDFtree*);
 static NCerror  attachsubset34r(CDFnode*, CDFnode*);
 static void free1cdfnode34(CDFnode* node);
 
@@ -42,7 +42,7 @@ fix1node34(NCDAPCOMMON* nccomm, CDFnode* node)
     nullfree(node->ncfullname);
     node->ncfullname = makecdfpathstring3(node,nccomm->cdf.separator);
     if(node->ncfullname == NULL) return NC_ENOMEM;
-    if(node->nctype == NC_Primitive)
+    if(node->nctype == NC_Atomic)
         node->externaltype = nctypeconvert(nccomm,node->etype);
     return NC_NOERR;
 }
@@ -152,7 +152,7 @@ replacedims(NClist* dims)
         CDFnode* dim = (CDFnode*)nclistget(dims,i);
 	CDFnode* basedim = dim->dim.basedim;
 	if(basedim == NULL) continue;
-	nclistset(dims,i,(ncelem)basedim);
+	nclistset(dims,i,(void*)basedim);
     }
 }
 
@@ -182,7 +182,7 @@ computecdfdimnames34(NCDAPCOMMON* nccomm)
     int i,j;
     char tmp[NC_MAX_NAME*2];
     NClist* conflicts = nclistnew();
-    NClist* varnodes = nccomm->cdf.varnodes;
+    NClist* varnodes = nccomm->cdf.ddsroot->tree->varnodes;
     NClist* alldims;
     NClist* basedims;
     
@@ -250,7 +250,7 @@ fprintf(stderr,"conflict: %s[%lu] %s[%lu]\n",
 			basedim->ncfullname,(unsigned long)basedim->dim.declsize,
 			dim->ncfullname,(unsigned long)dim->dim.declsize);
 #endif
-	    nclistpush(conflicts,(ncelem)dim);
+	    nclistpush(conflicts,(void*)dim);
 	}
 	/* Give  all the conflicting dimensions an index */
 	for(j=0;j<nclistlength(conflicts);j++) {
@@ -273,13 +273,13 @@ fprintf(stderr,"conflict: %s[%lu] %s[%lu]\n",
     for(i=0;i<nclistlength(alldims);i++) {
 	CDFnode* dim = (CDFnode*)nclistget(alldims,i);
 	if(dim->dim.basedim == NULL) {
-	    if(!nclistcontains(basedims,(ncelem)dim)) {
-		nclistpush(basedims,(ncelem)dim);
+	    if(!nclistcontains(basedims,(void*)dim)) {
+		nclistpush(basedims,(void*)dim);
 	    }
 	}
     }
 
-    nccomm->cdf.dimnodes = basedims;
+    nccomm->cdf.ddsroot->tree->dimnodes = basedims;
 
     /* cleanup */
     nclistfree(alldims);
@@ -304,7 +304,7 @@ fprintf(stderr,"conflict: %s[%lu] %s[%lu]\n",
 	        char sindex[64];
 		snprintf(sindex,sizeof(sindex),"_%d",dim->dim.index1);
 		dim->ncbasename = (char*)malloc(strlen(sindex)+strlen(legalname)+1);
-		if(dim->ncbasename == NULL) return NC_ENOMEM;
+		if(dim->ncbasename == NULL) {nullfree(legalname); return NC_ENOMEM;}
 		strcpy(dim->ncbasename,legalname);
 		strcat(dim->ncbasename,sindex);
 		nullfree(legalname);
@@ -364,7 +364,7 @@ makegetvar34(NCDAPCOMMON* nccomm, CDFnode* var, void* data, nc_type dsttype, Get
 }
 
 int
-constrainable34(NC_URI* durl)
+constrainable34(NCURI* durl)
 {
    char** protocol = constrainableprotocols;
    for(;*protocol;protocol++) {
@@ -375,8 +375,8 @@ constrainable34(NC_URI* durl)
 }
 
 CDFnode*
-makecdfnode34(NCDAPCOMMON* nccomm, char* name, OCtype octype,
-             /*optional*/ OCobject ocnode, CDFnode* container)
+makecdfnode34(NCDAPCOMMON* nccomm, char* ocname, OCtype octype,
+             /*optional*/ OCddsnode ocnode, CDFnode* container)
 {
     CDFnode* node;
     assert(nccomm != NULL);
@@ -384,20 +384,20 @@ makecdfnode34(NCDAPCOMMON* nccomm, char* name, OCtype octype,
     if(node == NULL) return (CDFnode*)NULL;
 
     node->ocname = NULL;
-    if(name) {
-        size_t len = strlen(name);
+    if(ocname) {
+        size_t len = strlen(ocname);
         if(len >= NC_MAX_NAME) len = NC_MAX_NAME-1;
         node->ocname = (char*)malloc(len+1);
 	if(node->ocname == NULL) return NULL;
-	memcpy(node->ocname,name,len);
+	memcpy(node->ocname,ocname,len);
 	node->ocname[len] = '\0';
     }
     node->nctype = octypetonc(octype);
     node->ocnode = ocnode;
     node->subnodes = nclistnew();
     node->container = container;
-    if(ocnode != OCNULL) {
-	oc_inq_primtype(nccomm->oc.conn,ocnode,&octype);
+    if(ocnode != NULL) {
+	oc_dds_atomictype(nccomm->oc.conn,ocnode,&octype);
         node->etype = octypetonc(octype);
     }
     return node;
@@ -408,7 +408,7 @@ makecdfnode34(NCDAPCOMMON* nccomm, char* name, OCtype octype,
    of all nodes in preorder.
 */
 NCerror
-buildcdftree34(NCDAPCOMMON* nccomm, OCobject ocroot, OCdxd occlass, CDFnode** cdfrootp)
+buildcdftree34(NCDAPCOMMON* nccomm, OCddsnode ocroot, OCdxd occlass, CDFnode** cdfrootp)
 {
     CDFnode* root = NULL;
     CDFtree* tree = (CDFtree*)calloc(1,sizeof(CDFtree));
@@ -428,28 +428,48 @@ buildcdftree34(NCDAPCOMMON* nccomm, OCobject ocroot, OCdxd occlass, CDFnode** cd
 }        
 
 static NCerror
-buildcdftree34r(NCDAPCOMMON* nccomm, OCobject ocnode, CDFnode* container,
+buildcdftree34r(NCDAPCOMMON* nccomm, OCddsnode ocnode, CDFnode* container,
                 CDFtree* tree, CDFnode** cdfnodep)
 {
-    unsigned int i,ocrank,ocnsubnodes;
+    size_t i,ocrank,ocnsubnodes;
     OCtype octype;
+    OCtype ocatomtype;
     char* ocname = NULL;
     NCerror ncerr = NC_NOERR;
     CDFnode* cdfnode;
 
-    oc_inq_class(nccomm->oc.conn,ocnode,&octype);
-    oc_inq_name(nccomm->oc.conn,ocnode,&ocname);
-    oc_inq_rank(nccomm->oc.conn,ocnode,&ocrank);
-    oc_inq_nsubnodes(nccomm->oc.conn,ocnode,&ocnsubnodes);
+    oc_dds_class(nccomm->oc.conn,ocnode,&octype);
+    if(octype == OC_Atomic)
+	oc_dds_atomictype(nccomm->oc.conn,ocnode,&ocatomtype);
+    else
+	ocatomtype = OC_NAT;
+    oc_dds_name(nccomm->oc.conn,ocnode,&ocname);
+    oc_dds_rank(nccomm->oc.conn,ocnode,&ocrank);
+    oc_dds_nsubnodes(nccomm->oc.conn,ocnode,&ocnsubnodes);
+
+#ifdef DEBUG1
+    if(ocatomtype == OC_NAT)
+	fprintf(stderr,"buildcdftree: connect: %s %s\n",oc_typetostring(octype),ocname);
+    else
+	fprintf(stderr,"buildcdftree: connect: %s %s\n",oc_typetostring(ocatomtype),ocname);
+#endif
 
     switch (octype) {
     case OC_Dataset:
     case OC_Grid:
     case OC_Structure:
     case OC_Sequence:
-    case OC_Primitive:
 	cdfnode = makecdfnode34(nccomm,ocname,octype,ocnode,container);
-	nclistpush(tree->nodes,(ncelem)cdfnode);
+	nclistpush(tree->nodes,(void*)cdfnode);
+	if(tree->root == NULL) {
+	    tree->root = cdfnode;
+	    cdfnode->tree = tree;
+	}		
+	break;
+
+    case OC_Atomic:
+	cdfnode = makecdfnode34(nccomm,ocname,octype,ocnode,container);
+	nclistpush(tree->nodes,(void*)cdfnode);
 	if(tree->root == NULL) {
 	    tree->root = cdfnode;
 	    cdfnode->tree = tree;
@@ -465,12 +485,12 @@ buildcdftree34r(NCDAPCOMMON* nccomm, OCobject ocnode, CDFnode* container,
 
     if(ocrank > 0) defdimensions(ocnode,cdfnode,nccomm,tree);
     for(i=0;i<ocnsubnodes;i++) {
-	OCobject ocsubnode;
+	OCddsnode ocsubnode;
 	CDFnode* subnode;
-	oc_inq_ith(nccomm->oc.conn,ocnode,i,&ocsubnode);
+	oc_dds_ithfield(nccomm->oc.conn,ocnode,i,&ocsubnode);
 	ncerr = buildcdftree34r(nccomm,ocsubnode,cdfnode,tree,&subnode);
 	if(ncerr) return ncerr;
-	nclistpush(cdfnode->subnodes,(ncelem)subnode);
+	nclistpush(cdfnode->subnodes,(void*)subnode);
     }
     nullfree(ocname);
     if(cdfnodep) *cdfnodep = cdfnode;
@@ -478,31 +498,31 @@ buildcdftree34r(NCDAPCOMMON* nccomm, OCobject ocnode, CDFnode* container,
 }
 
 static void
-defdimensions(OCobject ocnode, CDFnode* cdfnode, NCDAPCOMMON* nccomm, CDFtree* tree)
+defdimensions(OCddsnode ocnode, CDFnode* cdfnode, NCDAPCOMMON* nccomm, CDFtree* tree)
 {
-    unsigned int i,ocrank;
+    size_t i,ocrank;
  
-    oc_inq_rank(nccomm->oc.conn,ocnode,&ocrank);
+    oc_dds_rank(nccomm->oc.conn,ocnode,&ocrank);
     assert(ocrank > 0);
     for(i=0;i<ocrank;i++) {
 	CDFnode* cdfdim;
-	OCobject ocdim;
+	OCddsnode ocdim;
 	char* ocname;
 	size_t declsize;
 
-	oc_inq_ithdim(nccomm->oc.conn,ocnode,i,&ocdim);
-	oc_inq_dim(nccomm->oc.conn,ocdim,&declsize,&ocname);
+	oc_dds_ithdimension(nccomm->oc.conn,ocnode,i,&ocdim);
+	oc_dimension_properties(nccomm->oc.conn,ocdim,&declsize,&ocname);
 
 	cdfdim = makecdfnode34(nccomm,ocname,OC_Dimension,
                               ocdim,cdfnode->container);
 	nullfree(ocname);
-	nclistpush(tree->nodes,(ncelem)cdfdim);
+	nclistpush(tree->nodes,(void*)cdfdim);
 	/* Initially, constrained and unconstrained are same */
 	cdfdim->dim.declsize = declsize;
 	cdfdim->dim.array = cdfnode;
 	if(cdfnode->array.dimset0 == NULL) 
 	    cdfnode->array.dimset0 = nclistnew();
-	nclistpush(cdfnode->array.dimset0,(ncelem)cdfdim);
+	nclistpush(cdfnode->array.dimset0,(void*)cdfdim);
     }    
 }
 
@@ -520,7 +540,7 @@ applyclientparams34(NCDAPCOMMON* nccomm)
     const char* value;
     char tmpname[NC_MAX_NAME+32];
     char* pathstr;
-    OCconnection conn = nccomm->oc.conn;
+    OClink conn = nccomm->oc.conn;
     unsigned long limit;
 
     ASSERT(nccomm->oc.url != NULL);
@@ -571,13 +591,13 @@ applyclientparams34(NCDAPCOMMON* nccomm)
     nccomm->cdf.defaultstringlength = dfaltstrlen;
 
     /* String dimension limits apply to variables */
-    for(i=0;i<nclistlength(nccomm->cdf.varnodes);i++) {
-	CDFnode* var = (CDFnode*)nclistget(nccomm->cdf.varnodes,i);
+    for(i=0;i<nclistlength(nccomm->cdf.ddsroot->tree->varnodes);i++) {
+	CDFnode* var = (CDFnode*)nclistget(nccomm->cdf.ddsroot->tree->varnodes,i);
 	/* Define the client param stringlength for this variable*/
 	var->maxstringlength = 0; /* => use global dfalt */
 	strcpy(tmpname,"stringlength_");
 	pathstr = makeocpathstring3(conn,var->ocnode,".");
-	strcat(tmpname,pathstr);
+	strncat(tmpname,pathstr,NC_MAX_NAME);
 	nullfree(pathstr);
 	value = oc_clientparam_get(conn,tmpname);	
         if(value != NULL && strlen(value) != 0) {
@@ -591,11 +611,11 @@ applyclientparams34(NCDAPCOMMON* nccomm)
 	var->sequencelimit = dfaltseqlim;
 	strcpy(tmpname,"nolimit_");
 	pathstr = makeocpathstring3(conn,var->ocnode,".");
-	strcat(tmpname,pathstr);
+	strncat(tmpname,pathstr,NC_MAX_NAME);
 	if(oc_clientparam_get(conn,tmpname) != NULL)
 	    var->sequencelimit = 0;
 	strcpy(tmpname,"limit_");
-	strcat(tmpname,pathstr);
+	strncat(tmpname,pathstr,NC_MAX_NAME);
 	value = oc_clientparam_get(conn,tmpname);
         if(value != NULL && strlen(value) != 0) {
             if(sscanf(value,"%d",&len) && len > 0)
@@ -633,12 +653,15 @@ freecdfroot34(CDFnode* root)
     /* Explicitly FREE the ocroot */
     nccomm = tree->owner;
     oc_root_free(nccomm->oc.conn,tree->ocroot);
-    tree->ocroot = OCNULL;
+    tree->ocroot = NULL;
     for(i=0;i<nclistlength(tree->nodes);i++) {
 	CDFnode* node = (CDFnode*)nclistget(tree->nodes,i);
 	free1cdfnode34(node);
     }
     nclistfree(tree->nodes);
+    nclistfree(tree->varnodes);
+    nclistfree(tree->seqnodes);
+    nclistfree(tree->gridnodes);
     nullfree(tree);
 }
 
@@ -685,30 +708,43 @@ nodematch34(CDFnode* node1, CDFnode* node2)
     return simplenodematch34(node1,node2);
 }
 
+/*
+Try to figure out if two nodes
+are the "related" =>
+    same name && same nc_type and same arity
+but: Allow Grid == Structure
+*/
+
 int
 simplenodematch34(CDFnode* node1, CDFnode* node2)
 {
-    if(node1 == NULL) return (node2==NULL);
-    if(node2 == NULL) return 0;
-    if(node1->nctype != node2->nctype) {
-	/* Check for Grid->Structure match */
-	if((node1->nctype == NC_Structure && node2->nctype == NC_Grid)
-	   || (node2->nctype == NC_Structure && node1->nctype == NC_Grid)){
-	   if(node1->ocname == NULL || node2->ocname == NULL
-	      || strcmp(node1->ocname,node2->ocname) !=0) return 0;	    	
-	} else return 0;
-    }
+    /* Test all the obvious stuff */
+    if(node1 == NULL || node2 == NULL)
+	return 0;
+    if(strcmp(node1->ocname,node2->ocname)!=0) /* same names */
+	return 0;
+    if(nclistlength(node1->array.dimset0)
+	!= nclistlength(node2->array.dimset0)) /* same arity */
+	return 0;
+
     /* Add hack to address the screwed up Columbia server */
     if(node1->nctype == NC_Dataset) return 1;
-    if(node1->nctype == NC_Primitive
-       && node1->etype != node2->etype) return 0;
-    if(node1->ocname != NULL && node2->ocname != NULL
-       && strcmp(node1->ocname,node2->ocname)!=0) return 0;
-    if(nclistlength(node1->array.dimset0)
-       != nclistlength(node2->array.dimset0)) return 0;
+
+    if(node1->nctype != node2->nctype) {
+	/* test for struct-grid match */
+	int structgrid = ((node1->nctype == NC_Grid && node2->nctype == NC_Structure)
+                          || (node1->nctype == NC_Structure && node2->nctype == NC_Grid) ? 1 : 0);
+	if(!structgrid)
+	    return 0;
+    }
+
+    if(node1->nctype == NC_Atomic && node1->etype != node2->etype)
+	return 0;
+
     return 1;
 }
 
+
 /*
 Given DDS node, locate the node
 in a DATADDS that matches the DDS node.
@@ -759,7 +795,7 @@ fprintf(stderr,"attachdim: %s->%s\n",xdim->ocname,tdim->ocname);
 
 /* 
 Match a DATADDS node to a DDS node.
-It is assumed that both trees have been regridded if necessary.
+It is assumed that both trees have been re-struct'ed if necessary.
 */
 
 static NCerror
@@ -798,7 +834,7 @@ fprintf(stderr,"attachnode: %s->%s\n",xnode->ocname,templatepathnode->ocname);
         if(simplenodematch34(xsubnode,templatepathnext)) {
 	    ncstat = attach34r(xsubnode,templatepath,depth+1);
 	    if(ncstat) goto done;
-        } else if(gridable && xsubnode->nctype == NC_Primitive) {
+        } else if(gridable && xsubnode->nctype == NC_Atomic) {
             /* grids may or may not appear in the datadds;
 	       try to match the xnode subnodes against the parts of the grid
 	    */
@@ -893,12 +929,12 @@ getalldims34a(NClist* dimset, NClist* alldims)
     int i;
     for(i=0;i<nclistlength(dimset);i++) {
 	CDFnode* dim = (CDFnode*)nclistget(dimset,i);
-	if(!nclistcontains(alldims,(ncelem)dim)) {
+	if(!nclistcontains(alldims,(void*)dim)) {
 #ifdef DEBUG3
 fprintf(stderr,"getalldims: %s[%lu]\n",
 			dim->ncfullname,(unsigned long)dim->dim.declsize);
 #endif
-	    nclistpush(alldims,(ncelem)dim);
+	    nclistpush(alldims,(void*)dim);
 	}
     }
 }
@@ -911,12 +947,12 @@ getalldims34(NCDAPCOMMON* nccomm, int visibleonly)
 {
     int i;
     NClist* alldims = nclistnew();
-    NClist* varnodes = nccomm->cdf.varnodes;
+    NClist* varnodes = nccomm->cdf.ddsroot->tree->varnodes;
 
     /* get bag of all dimensions */
     for(i=0;i<nclistlength(varnodes);i++) {
 	CDFnode* node = (CDFnode*)nclistget(varnodes,i);
-	if(!visibleonly || node->visible) {
+	if(!visibleonly || !node->invisible) {
 	    getalldims34a(node->array.dimsetall,alldims);
 	}
     }
