@@ -5,8 +5,7 @@
  *********************************************************************/
 
 #include "ncdap3.h"
-#include "dapodom.h"
-#include "dapdebug.h"
+#include "dapdump.h"
 #include "dapdump.h"
 #include "dceparselex.h"
 
@@ -44,7 +43,11 @@ parsedapconstraints(NCDAPCOMMON* dapcomm, char* constraints,
 
 /* Map constraint paths to CDFnode paths in specified tree and fill
    in the declsizes.
-   The difficulty is that suffix paths are legal.
+
+   Two things to watch out for:
+   1. suffix paths are legal (i.e. incomplete paths)
+   2. nc_virtual nodes (via restruct)
+
 */
 
 NCerror
@@ -55,24 +58,19 @@ mapconstraints3(DCEconstraint* constraint,
     NCerror ncstat = NC_NOERR;
     NClist* nodes = root->tree->nodes;
     NClist* dceprojections = constraint->projections;
-#if 0
-    NClist* dceselections = constraint->selections;
-#endif
 
     /* Convert the projection paths to leaves in the dds tree */
     for(i=0;i<nclistlength(dceprojections);i++) {
+	CDFnode* cdfmatch = NULL;
 	DCEprojection* proj = (DCEprojection*)nclistget(dceprojections,i);
 	if(proj->discrim != CES_VAR) continue; // ignore functions
-	ncstat = matchpartialname3(nodes,proj->var->segments,
-				   (CDFnode**)&proj->var->annotation);
+	ncstat = matchpartialname3(nodes,proj->var->segments,&cdfmatch);
 	if(ncstat) goto done;
+	/* Cross links */
+	assert(cdfmatch != NULL);
+	proj->var->annotation = (void*)cdfmatch;
     }
 
-#ifdef DEBUG
-fprintf(stderr,"mapconstraint.projections: %s\n",
-		dumpprojections(dceprojections));
-#endif
-
 done:
     return THROW(ncstat);
 }
@@ -117,7 +115,7 @@ qualifyprojectionnames3(DCEprojection* proj)
 
     ASSERT((proj->discrim == CES_VAR
             && proj->var->annotation != NULL
-            && ((CDFnode*)proj->var->annotation)->ocnode != OCNULL));
+            && ((CDFnode*)proj->var->annotation)->ocnode != NULL));
     collectnodepath3((CDFnode*)proj->var->annotation,fullpath,!WITHDATASET);
 #ifdef DEBUG
 fprintf(stderr,"qualify: %s -> ",
@@ -185,7 +183,7 @@ completesegments3(NClist* fullpath, NClist* segments)
         seg->name = nulldup(node->ocname);
         seg->annotation = (void*)node;
 	seg->rank = nclistlength(node->array.dimset0);
-        nclistinsert(segments,i,(ncelem)seg);
+        nclistinsert(segments,i,(void*)seg);
     }
     /* Now modify the segments to point to the appropriate node
        and fill in the slices.
@@ -256,10 +254,10 @@ matchpartialname3(NClist* nodes, NClist* segments, CDFnode** nodep)
 	if(node->nctype != NC_Sequence
                && node->nctype != NC_Structure
                && node->nctype != NC_Grid
-               && node->nctype != NC_Primitive
+               && node->nctype != NC_Atomic
           )
 	    continue;
-	nclistpush(namematches,(ncelem)node);
+	nclistpush(namematches,(void*)node);
     }    
     if(nclistlength(namematches)==0) {
         nclog(NCLOGERR,"No match for projection name: %s",lastseg->name);
@@ -274,7 +272,7 @@ matchpartialname3(NClist* nodes, NClist* segments, CDFnode** nodep)
 	collectnodepath3(matchnode,matchpath,0);
 	/* Do a suffix match */
         if(matchsuffix3(matchpath,segments)) {
-	    nclistpush(matches,(ncelem)matchnode);
+	    nclistpush(matches,(void*)matchnode);
 #ifdef DEBUG
 fprintf(stderr,"matchpartialname: pathmatch: %s :: %s\n",
 matchnode->ncfullname,dumpsegments(segments));
@@ -333,34 +331,43 @@ done:
 static int
 matchsuffix3(NClist* matchpath, NClist* segments)
 {
-    int i,j;
+    int i,pathstart;
     int nsegs = nclistlength(segments);
     int pathlen = nclistlength(matchpath);
-    ASSERT(pathlen >= nsegs);
-    for(i=0;i<pathlen;i++) {
-        int pathmatch = 1;
-	/* Starting at this point in the path, try to match the segment list */
-        for(j=0;j<nsegs && (i+j < pathlen);j++) {
-            int segmatch = 1;
-	    DCEsegment* seg = (DCEsegment*)nclistget(segments,j);
-   	    CDFnode* node = (CDFnode*)nclistget(matchpath,i+j);
-	    int rank = seg->rank;
-	    /* Do the names match (in oc name space) */
-	    if(strcmp(seg->name,node->ocname) != 0) {
-		segmatch = 0;/* no match */
-	    } else
+    int segmatch;
+
+    /* try to match the segment list as a suffix of the path list */
+
+    /* Find the maximal point in the path s.t. |suffix of path|
+       == |segments|
+    */
+    pathstart = (pathlen - nsegs);
+    if(pathstart < 0)
+	return 0; /* pathlen <nsegs => no match possible */
+
+    /* Walk the suffix of the path and the segments and
+       matching as we go
+    */
+    for(i=0;i<nsegs;i++) {
+	CDFnode* node = (CDFnode*)nclistget(matchpath,pathstart+i);
+	DCEsegment* seg = (DCEsegment*)nclistget(segments,i);
+	int rank = seg->rank;
+	segmatch = 1; /* until proven otherwise */
+	/* Do the names match (in oc name space) */
+	if(strcmp(seg->name,node->ocname) != 0) {
+	    segmatch = 0;
+	} else {
 	    /* Do the ranks match (watch out for sequences) */
-	    if(rank == 0) /* rank == 9 matches any set of dimensions */
-		segmatch = 1;
-	    else if(node->nctype == NC_Sequence)
-	        segmatch = (rank == 1?1:0);
-	    else /*!NC_Sequence*/
-	        segmatch = (rank == nclistlength(node->array.dimset0)?1:0);
-	    if(!segmatch) pathmatch = 0;
+	    if(node->nctype == NC_Sequence)
+		rank--; /* remove sequence pseudo-rank */
+	    if(rank > 0
+		&& rank != nclistlength(node->array.dimset0))
+		segmatch = 0; /* rank mismatch */
 	}
-	if(pathmatch) return 1;
+	if(!segmatch)
+	    return 0;
    }
-   return 0;
+   return 1; /* all segs matched */
 }
 
 
@@ -407,43 +414,52 @@ buildconstraintstring3(DCEconstraint* constraints)
    with any pseudo dimensions removed
 */
 NCerror
-buildvaraprojection3(Getvara* getvar,
+buildvaraprojection3(CDFnode* var,
 		     const size_t* startp, const size_t* countp, const ptrdiff_t* stridep,
 		     DCEprojection** projectionp)
 {
     int i,j;
     NCerror ncstat = NC_NOERR;
-    CDFnode* var = getvar->target;
     DCEprojection* projection = NULL;
     NClist* path = nclistnew();
     NClist* segments = NULL;
     int dimindex;
 
+    /* Build a skeleton projection that has 1 segment for
+       every cdfnode from root to the variable of interest.
+       Each segment has the slices from its corresponding node
+       in the path, including pseudo-dims
+    */
     ncstat = dapvar2projection(var,&projection);
+
+
 #ifdef DEBUG
-fprintf(stderr,"buildvaraprojection: %s\n",dumpprojection(projection));
+fprintf(stderr,"buildvaraprojection: skeleton: %s\n",dumpprojection(projection));
 #endif
 
-    /* We need to assign the start/count/stride info to each segment;
-       declsize will have been set
+    /* Now, modify the projection to reflect the corresponding
+       start/count/stride from the nc_get_vara arguments.
     */
     segments = projection->var->segments;
     dimindex = 0;
     for(i=0;i<nclistlength(segments);i++) {
 	DCEsegment* segment = (DCEsegment*)nclistget(segments,i);
         for(j=0;j<segment->rank;j++) {
+	    size_t count = 0;
 	    DCEslice* slice = &segment->slices[j];
 	    /* make each slice represent the corresponding
                start/count/stride */
 	    slice->first = startp[dimindex+j];
 	    slice->stride = stridep[dimindex+j];
-	    slice->count = countp[dimindex+j];
-	    slice->length = slice->count * slice->stride;
-	    if(slice->length > slice->declsize)
-		slice->length = slice->declsize;
-	    slice->stop = (slice->first + slice->length);
-	    if(slice->stop > slice->declsize)
-		slice->stop = slice->declsize;
+	    count = countp[dimindex+j];
+	    slice->count = count;
+	    slice->length = count * slice->stride;
+	    slice->last = (slice->first + slice->length) - 1;
+	    if(slice->last >= slice->declsize) {
+		slice->last = slice->declsize - 1;
+		/* reverse compute the new length */
+		slice->length = (slice->last - slice->first) + 1;
+	    }
 	}
 	dimindex += segment->rank;
     }
@@ -452,7 +468,7 @@ fprintf(stderr,"buildvaraprojection.final: %s\n",dumpprojection(projection));
 #endif
 
 #ifdef DEBUG
-fprintf(stderr,"buildvaraprojection3: projection=%s\n",
+fprintf(stderr,"buildvaraprojection3: final: projection=%s\n",
         dumpprojection(projection));
 #endif
 
@@ -468,10 +484,11 @@ iswholeslice(DCEslice* slice, CDFnode* dim)
 {
     if(slice->first != 0 || slice->stride != 1) return 0;
     if(dim != NULL) {
-	if(slice->stop != dim->dim.declsize) return 0;
+	if(slice->length != dim->dim.declsize) return 0;
     } else if(dim == NULL) {
+	size_t count = slice->count;
 	if(slice->declsize == 0
-	   || slice->count != slice->declsize) return 0;
+	   || count != slice->declsize) return 0;
     }
     return 1;
 }
@@ -571,7 +588,7 @@ fprintf(stderr,"fixprojection: list = %s\n",dumpprojections(list));
 		nclog(NCLOGWARN,"Malformed projection: same variable with different slicing");
 	    }
 	    /* remove p32 */
-	    nclistset(list,j,(ncelem)NULL);	    
+	    nclistset(list,j,(void*)NULL);	    
 	    dcefree((DCEnode*)p2);	    
 	}	
     }
@@ -592,7 +609,7 @@ fprintf(stderr,"fixprojection: list = %s\n",dumpprojections(list));
 	    for(k=0;k<nclistlength(tmp);k++) {
 		void* candidate = (void*)nclistget(tmp,k);
 	        if(candidate == p1->var->annotation) {
-		    nclistset(list,i,(ncelem)NULL);	    
+		    nclistset(list,i,(void*)NULL);	    
 	            dcefree((DCEnode*)p1);
 		    goto next;
 		}
@@ -613,9 +630,9 @@ next:   continue;
             leaf = (CDFnode*)target->var->annotation;
             ASSERT(leaf != NULL);
             if(iscontainer(leaf)) {/* capture container */
-		if(!nclistcontains(tmp,(ncelem)target))
-                    nclistpush(tmp,(ncelem)target);
-                nclistset(list,i,(ncelem)NULL);
+		if(!nclistcontains(tmp,(void*)target))
+                    nclistpush(tmp,(void*)target);
+                nclistset(list,i,(void*)NULL);
             }
         }
 	if(nclistlength(tmp) == 0) break; /*done*/
@@ -627,7 +644,7 @@ next:   continue;
                 CDFnode* field = (CDFnode*)nclistget(leaf->subnodes,j);
 		/* Convert field node to a proper constraint */
 		DCEprojection* proj = projectify(field,container);
-		nclistpush(list,(ncelem)proj);
+		nclistpush(list,(void*)proj);
 	    }	    
             /* reclaim the container */
 	    dcefree((DCEnode*)container);
@@ -670,7 +687,7 @@ projectify(CDFnode* field, DCEprojection* container)
     /* Dup the segment list */
     var->segments = dceclonelist(container->var->segments);
     seg->rank = 0;
-    nclistpush(var->segments,(ncelem)seg);
+    nclistpush(var->segments,(void*)seg);
     return proj;
 }
 
@@ -688,9 +705,13 @@ slicematch(NClist* seglist1, NClist* seglist2)
 	if(seg1->rank != seg2->rank)
 	    return 0;
 	for(j=0;j<seg1->rank;j++) {
-	    if(seg1->slices[j].first != seg2->slices[j].first 
-	       || seg1->slices[j].count != seg2->slices[j].count
-	       || seg1->slices[j].stride != seg2->slices[j].stride)
+	    DCEslice* slice1 = &seg1->slices[j];
+	    DCEslice* slice2 = &seg2->slices[j];
+	    size_t count1 = slice1->count;
+	    size_t count2 = slice2->count;
+	    if(slice1->first != slice2->first
+	       || count1 != count2
+	       || slice1->stride != slice2->stride)
 		return 0;
 	}
     }
@@ -710,7 +731,7 @@ dapvar2projection(CDFnode* var, DCEprojection** projectionp)
     DCEprojection* projection = NULL;
     int dimindex;
 
-    /* Collect the nodes needed to construct the projection segment */    
+    /* Collect the nodes needed to construct the projection segments */    
     collectnodepath3(var,path,!WITHDATASET);
 
     segments = nclistnew();
@@ -739,7 +760,7 @@ dapvar2projection(CDFnode* var, DCEprojection** projectionp)
 	segment->slicesdefined = 1;
 	segment->slicesdeclized = 1;
 	dimindex += localrank;
-	nclistpush(segments,(ncelem)segment);
+	nclistpush(segments,(void*)segment);
     }
     
     projection = (DCEprojection*)dcecreate(CES_PROJECT);
@@ -795,7 +816,8 @@ fprintf(stderr,"restrictprojection.before: constraints=|%s| vara=|%s|\n",
     result = (DCEprojection*)dceclone((DCEnode*)result); /* so we can modify */
 
 #ifdef DEBUG1
-fprintf(stderr,"restrictprojection.choice: |%s|\n",dumpprojection(result));
+fprintf(stderr,"restrictprojection.choice: base=|%s| add=|%s|\n",
+	dumpprojection(result),dumpprojection(var));
 #endif
     /* We need to merge the projection from the projection list
        with the var projection
@@ -821,7 +843,7 @@ dapshiftslice(DCEslice* slice)
     slice->first = 0;
     slice->stride = 1;
     slice->length = slice->count;
-    slice->stop = slice->count;
+    slice->last = slice->length - 1;
 }
 
 int
@@ -851,3 +873,36 @@ fprintf(stderr,"dapshiftprojection.after: %s\n",dumpprojection(projection));
 
     return ncstat;
 }
+
+/* Compute the set of variables referenced in the projections
+   of the input constraint.
+*/
+NCerror
+computeprojectedvars(NCDAPCOMMON* dapcomm, DCEconstraint* constraint)
+{
+    NCerror ncstat = NC_NOERR;
+    NClist* vars = NULL;
+    int i;
+
+    vars = nclistnew();
+
+    if(dapcomm->cdf.projectedvars != NULL)
+	nclistfree(dapcomm->cdf.projectedvars);
+    dapcomm->cdf.projectedvars = vars;
+
+    if(constraint == NULL || constraint->projections == NULL)
+	goto done;
+
+    for(i=0;i<nclistlength(constraint->projections);i++) {
+	CDFnode* node;
+	DCEprojection* proj = (DCEprojection*)nclistget(constraint->projections,i);
+	if(proj->discrim == CES_FCN) continue; /* ignore these */
+	node = (CDFnode*)proj->var->annotation;	
+	if(!nclistcontains(vars,(void*)node)) {
+	    nclistpush(vars,(void*)node);
+	}
+    }    
+
+done:
+    return ncstat;
+}
