@@ -22,12 +22,19 @@
 
 #define DEBUG
 
+#define LBRACE "{"
+#define RBRACE "}"
+
 int dceverbose = 0;
 
 static char* opstrings[] = OPSTRINGS ;
 
 static void ceallnodesr(DCEnode* node, NClist* allnodes, CEsort which);
+static void dcedump(DCEnode* node, NCbytes* buf);
+static void dcedumpraw(DCEnode* node, NCbytes* buf);
+static void dcedumprawlist(NClist* list, NCbytes* buf);
 
+#if 0 /*not currently used */
 /* Parse incoming url constraints, if any,
    to check for syntactic correctness
 */ 
@@ -54,107 +61,172 @@ fprintf(stderr,"constraint: %s",dcetostring((DCEnode*)dapconstraint));
 #endif
     return ncstat;
 }
+#endif
 
-/* Worksheet
+#ifdef DEBUG1
+static void
+slicedump(const char* prefix, DCEslice* s)
+{
+#if 1
+    int v = dceverbose;
+    dceverbose = 1;
+    fprintf(stderr,"%s: %s\n",prefix,dcetostring((DCEnode*)s));
+    dceverbose = v;
+#else
+    size_t last = (s->first+s->length)-1;
+    fprintf(stderr,"%s: [%lu:%lu:%lu p=%lu l=%lu c=%lu]\n",
+	prefix,s->first,s->stride,last,s->stop,s->length,s->count);
+#endif
+}
+#endif
 
-mg.st = md.st * ms.st
-mg.f  = md.f+(ms.f*md.st)
-mg.l  = ((ms.l-1) / ms.st) * mg.st + 1
-mg.p  = mg.f + mg.l
-mg.c  = mg.l / mg.st
 
+/*
+Compose slice s1 with slice s2 -> sr
+Compose means that the s2 constraint is applied
+to the output of the s1 constraint.
+
+Logical derivation of s1 compose s2 = sr
+We have three index sequences to deal with,
+where the sequence is the range [0..last].
+
+1. the original data indices [0..N]
+2. the indices resulting from applying slice s1;
+   this will be a subset of #1 with the sequence
+   [s1.first .. s1.last] where s1.last = s1.first + s1.length - 1
+3. the indices resulting from applying s2:[0..s2.last] wrt
+   to output of s2
+   [s2.first .. s2.last]
+   We can convert #3 into index sequence wrt
+   #1 as follows
+   [s2.first .. s2.last] -> [map(s2.first)..map(s2.last)]
+   where map(int index) = s1.first + (s1.stride * index)
+   Note that map(i) is undefined if map(i) > s1.last
+
+
+So, we can compute the result (sr) stride and first as follows
+
+. sr.stride = s1.stride * s2.stride
+
+. sr.first   = map(s2.first) =  s1.first * (s1.stride * s2.first)
+  This throws an exception if sr.first > s1.last
+
+. compute the s1.last wrt original data
+   last1 = s1.first + (s1.length - 1)
+
+. compute the s2.last wrt s2
+   last2 = s2.first + (s2.length - 1)
+
+. compute candidate last wrt original index sequence
+   lastx = map(last2) = s1.first + (s1.stride * last2)
+
+. It is possible that lastx is outside of the range (s1.first..s1.last)
+  so we take the min of last2 and lastx to ensure no overrun wrt s1
+   sr.last = min(last1,lastx)
+
+. If we want to be pedantic, we need to reduce
+  sr.last so that it is on an exact multiple of sr.stride
+  starting at sr.first
+    delta = ((sr.last + (sr.stride-1)) / sr.stride) * sr.stride
+    sr.last = sr.first + delta
+
+. compute result length using sr.last
+   sr.len = (sr.last + 1) - sr.first
+
+Example 1:
+0000000000111111111
+0123456789012345678
+ xxxxxxxxx
+ 0 1 2 3 4                s1=(f=1 st=2 len=9)
+ 0 1 2 3 4                s2=(f=0 st=1 len=5)
+ xxxxxxxxxy
+ 0 1 3 4 5		  sr=(f=1 st=2 len=9..10)
+
+Example 2:
 0000000000111111111122222222223
 0123456789012345678901234567890
-          xxxxxx
-                xxxxxx
- 0  1  2  3  4  5  6  7  8        md=(st=3 f=1 l=25 p=26)
-          0     1     2           ms=(st=2 f=3 l=5  p=8 )
-		                  ----------------------------
-                                  mg=(st=6 f=10 p=23 l=13)
-c = 4 / 2 = 2
-l = 2 * 6 + 1 = 13
+ xxxxxxxxxxxxxxxxxxxxxxxxx
+ 0  1  2  3  4  5  6  7  8  _  _  _  s1=(f=1 st=3 len=25)
+          0     1     2     3     4  s2=(f=3 st=2 len=5)
+          xxxxxxxxxxxxxyyyyy
+          0     1     2              sr=(f=10 st=6 l=13..17)
 
+Example 3:
 0000000000111111
 0123456789012345
- 0 1 2 3 4                md=(st=2 f=1 l=9 p=10)
-     0 1 2                ms=(st=1 f=2 l=3 p=5)
-                          ----------------------------
-                          mg=(st=2 f=5 p=10 l=5 )
-c = 2/1 = 2
-l = 2 * 2 + 1 = 13
-
+ xxxxxxxxx
+ 0 1 2 3 4 _ _            s1=(f=1 st=2 len=9)
+     0 1 2 3 4            s2=(f=2 st=1 len=4)
+     xxxxxy               ----------------------------
+			  sr=(f=5 st=2 len=5..6)
+Example 4:
 0000000000111111111
 0123456789012345678
- 0 1 2 3 4 5 6 7 8        md=(st=2 f=1 l=17 p=18)
-       0   1   2          ms=(st=2 f=3 l=5 p=8)
-		          ----------------------------
-                          mg=(st=4 f=7 p=16 l=9 )
-c = 4/2 = 2
-l = 2 * 4 + 1 = 9
+ xxxxxxxxx
+ 0 1 2 3 4 _ _ _ _       s1=(f=1 st=2 len=9)
+     0   1   2   3       s2=(f=2 st=2 len=4)
+     xxxxxxyy
+                          sr=(f=5 st=4 len=6..8)
 
-0000000000111111111
-0123456789012345678
- 0 1 2 3 4                md=(st=2 f=1 l=9 p=10)
- 0 1 2 3 4                ms=(st=1 f=0 l=5 p=5)
-		          ----------------------------
-                          mg=(st=2 f=1 p=10 l=9 )
-c = 4/1 = 4
-l = 4 * 2 + 1 = 9
+Example 5:
+00000000001
+01234567890
+xxx
+012                       s1=(f=0 st=1 l=3)
+012                       s2=(f=0 st=1 l=3)
+xxx		          ----------------------------
+012                       sr=(f=0 st=1 l=3)
 
+Example 6:
 00000
 01234
-01                        md=(st=1 f=0 l=2 p=2)
-0                         ms=(st=1 f=0 l=1 p=1)
-		          ----------------------------
-                          mg=(st=1 f=0 p=1 l=1 )
-c = 0/1 = 0
-l = 0 * 1 + 1 = 1
-
-000000000011
-012345678901
-012                       md=(st=1 f=0 l=3 p=3)
-012                       ms=(st=1 f=0 l=3 p=2)
-		          ----------------------------
-                          mg=(st=1 f=0 p=3 l=3 )
-c = 2/1 = 2
-l = 2 * 1 + 1 = 3
+xx
+01                        s1=(f=0 st=1 l=2)
+0                         s2=(f=0 st=1 l=1)
+x		          ----------------------------
+                          sr=(f=0 st=1 l=1)
 
 */
 
-/* Merge slice src into slice dst; dst != src */
+#define MAP(s1,i) ((s1)->first + ((s1)->stride*(i)))
+#define XMIN(x,y) ((x) < (y) ? (x) : (y))
+#define XMAX(x,y) ((x) > (y) ? (x) : (y))
 
 int
-dceslicemerge(DCEslice* dst, DCEslice* src)
+dceslicecompose(DCEslice* s1, DCEslice* s2, DCEslice* result)
 {
     int err = NC_NOERR;
-    DCEslice tmp;
-
-    tmp.node.sort = CES_SLICE;
-    tmp.stride    = (dst->stride * src->stride);
-    tmp.first     = (dst->first+((src->first)*(dst->stride)));
-    tmp.length    = (((src->length - 1) / src->stride) * tmp.stride) + 1;
-    tmp.stop      = tmp.first + tmp.length;
-    tmp.count     = tmp.length / tmp.stride;
-    /* use max declsize */
-    if(dst->declsize > src->declsize) {
-        tmp.declsize  = dst->declsize;
-    } else {
-        tmp.declsize  = src->declsize;
-    }
-    if(tmp.length % tmp.stride != 0) tmp.count++;
-    if(tmp.first >= dst->stop || tmp.stop > dst->stop)
-	err = NC_EINVALCOORDS;
-    else
-	*dst = tmp;
+    size_t lastx = 0;
+    DCEslice sr; /* For back compatability so s1 and result can be same object */
+#ifdef DEBUG1
+slicedump("compose: s1",s1);
+slicedump("compose: s2",s2);
+#endif
+    sr.node.sort = CES_SLICE;
+    sr.stride    = s1->stride * s2->stride;
+    sr.first     = MAP(s1,s2->first);
+    if(sr.first > s1->last)
+	return NC_EINVALCOORDS;
+    lastx        = MAP(s1,s2->last);
+    sr.last      = XMIN(s1->last,lastx);
+    sr.length    = (sr.last + 1) - sr.first;
+    sr.declsize = XMAX(s1->declsize,s2->declsize); /* use max declsize */
+    /* fill in other fields */
+    sr.count = (sr.length + (sr.stride - 1))/sr.stride;
+    *result = sr;
+#ifdef DEBUG1
+slicedump("compose: result",result);
+#endif
     return err;
 }
 
-
 /*
 Given two projection lists, merge
 src into dst taking
 overlapping projections into acct.
+Dst will be modified.
 */
+
 int
 dcemergeprojectionlists(NClist* dst, NClist* src)
 {
@@ -171,11 +243,11 @@ fprintf(stderr,"dapmergeprojection: src = %s\n",dcetostring((DCEnode*)src));
     nclistsetalloc(cat,nclistlength(dst)+nclistlength(src));
     for(i=0;i<nclistlength(dst);i++) {
 	DCEprojection* p = (DCEprojection*)nclistget(dst,i);
-	nclistpush(cat,(ncelem)p);
+	nclistpush(cat,(void*)p);
     }    
     for(i=0;i<nclistlength(src);i++) {
 	DCEprojection* p = (DCEprojection*)nclistget(src,i);
-	nclistpush(cat,(ncelem)dceclone((DCEnode*)p));
+	nclistpush(cat,(void*)dceclone((DCEnode*)p));
     }    
 
     nclistclear(dst);
@@ -197,11 +269,11 @@ fprintf(stderr,"dapmergeprojection: src = %s\n",dcetostring((DCEnode*)src));
 	    /* This entry matches our current target; merge  */
 	    ncstat = dcemergeprojections(target,p2);
 	    /* null out this merged entry and release it */
-	    nclistset(cat,i,(ncelem)NULL);	    
+	    nclistset(cat,i,(void*)NULL);	    
 	    dcefree((DCEnode*)p2);	    
 	}		    
 	/* Capture the clone */
-	nclistpush(dst,(ncelem)target);
+	nclistpush(dst,(void*)target);
     }	    
     nclistfree(cat);
     return ncstat;
@@ -222,7 +294,7 @@ dcemergeprojections(DCEprojection* merged, DCEprojection* addition)
 	/* If one segment has larger rank, then copy the extra slices unchanged */
 	for(j=0;j<addedseg->rank;j++) {
 	    if(j < mergedseg->rank)
-	        dceslicemerge(mergedseg->slices+j,addedseg->slices+j);
+	        dceslicecompose(mergedseg->slices+j,addedseg->slices+j,mergedseg->slices+j);
 	    else
 		mergedseg->slices[j] = addedseg->slices[j];
 	}
@@ -379,7 +451,7 @@ dceclonelist(NClist* list)
     for(i=0;i<nclistlength(list);i++) {
 	DCEnode* node = (DCEnode*)nclistget(list,i);
 	DCEnode* newnode = dceclone((DCEnode*)node);
-	nclistpush(clone,(ncelem)newnode);
+	nclistpush(clone,(void*)newnode);
     }
     return clone;
 }
@@ -479,6 +551,29 @@ dcetostring(DCEnode* node)
     return s;
 }
 
+/* Mostly for debugging */
+char*
+dcerawtostring(void* node)
+{
+    char* s;
+    NCbytes* buf = ncbytesnew();
+    dcedumpraw((DCEnode*)node,buf);
+    s = ncbytesextract(buf);
+    ncbytesfree(buf);
+    return s;
+}
+
+char*
+dcerawlisttostring(NClist* list)
+{
+    char* s;
+    NCbytes* buf = ncbytesnew();
+    dcedumprawlist(list,buf);
+    s = ncbytesextract(buf);
+    ncbytesfree(buf);
+    return s;
+}
+
 /* For debugging */
 #ifdef DEBUG
 static char*
@@ -500,6 +595,12 @@ dimdecl(size_t declsize)
 
 void
 dcetobuffer(DCEnode* node, NCbytes* buf)
+{
+    dcedump(node,buf);
+}
+
+static void
+dcedump(DCEnode* node, NCbytes* buf)
 {
     int i;
     char tmp[1024];
@@ -677,9 +778,9 @@ ceallnodesr(DCEnode* node, NClist* allnodes, CEsort which)
 {
     int i;
     if(node == NULL) return;
-    if(nclistcontains(allnodes,(ncelem)node)) return;
+    if(nclistcontains(allnodes,(void*)node)) return;
     if(which == CES_NIL || node->sort == which)
-        nclistpush(allnodes,(ncelem)node);
+        nclistpush(allnodes,(void*)node);
     switch(node->sort) {
     case CES_FCN: {
 	DCEfcn* fcn = (DCEfcn*)node;
@@ -811,7 +912,7 @@ dceiswholeslice(DCEslice* slice)
 {
     if(slice->first != 0
        || slice->stride != 1
-       || slice->stop != slice->declsize) return 0;
+       || slice->length != slice->declsize) return 0;
     return 1;
 }
 
@@ -834,9 +935,9 @@ dcemakewholeslice(DCEslice* slice, size_t declsize)
     slice->first = 0;
     slice->stride = 1;
     slice->length = declsize;
-    slice->stop = declsize;
     slice->declsize = declsize;
     slice->count = declsize;
+    slice->last = slice->length - 1;
 }
 
 /* Remove slicing from terminal segment of p */
@@ -864,3 +965,247 @@ dcesamepath(NClist* list1, NClist* list2)
     }
     return 1;
 }
+
+void
+dcesegment_transpose(DCEsegment* segment,
+				 size_t* start,
+				 size_t* count,
+				 size_t* stride,
+				 size_t* sizes
+				)
+{
+    int i;
+    if(segment != NULL && sizes != NULL) {
+        for(i=0;i<segment->rank;i++) {
+	    if(start != NULL) start[i] = segment->slices[i].first;
+	    if(count != NULL) count[i] = segment->slices[i].count;
+	    if(stride != NULL) stride[i] = (size_t)segment->slices[i].stride;
+	    if(sizes != NULL) sizes[i] = segment->slices[i].declsize;
+	}
+    }	
+}
+
+/* Compute segment size for subset of slices */
+size_t
+dcesegmentsize(DCEsegment* seg, size_t start, size_t stop)
+{
+    int i, count;
+    if(!seg->slicesdefined) return 0; /* actually, we don't know */
+    for(count=1,i=start;i<stop;i++) {
+	count *= seg->slices[i].count;
+    }
+    return count;
+}
+
+/* Return the index of the leftmost slice
+   starting at start and upto, but not including
+   stop, such that it and all slices to the right
+   are "safe". Safe means dceiswholeslice() is true.
+   In effect, we can read the safe index set as a
+   single chunk. Return stop if there is no safe index.
+*/
+
+size_t
+dcesafeindex(DCEsegment* seg, size_t start, size_t stop)
+{
+    size_t safe;
+    if(!seg->slicesdefined) return stop; /* actually, we don't know */
+    if(stop == 0) return stop;
+    /* watch out because safe is unsigned */
+    for(safe=stop-1;safe>start;safe--) {
+	if(!dceiswholeslice(&seg->slices[safe])) return safe+1;
+    }
+    return dceiswholeslice(&seg->slices[start]) ? start /*every slice is safe*/
+                                                 : start+1 ;
+}
+
+
+static const char*
+dcesortname(CEsort sort)
+{
+    switch (sort) {
+    case CES_SLICE: return "SLICE";
+    case CES_SEGMENT: return "SEGMENT";
+    case CES_VAR: return "VAR";
+    case CES_FCN: return "FCN";
+    case CES_CONST: return "CONST";
+    case CES_VALUE: return "VALUE";
+    case CES_PROJECT: return "PROJECT";
+    case CES_SELECT: return "SELECT";
+    case CES_CONSTRAINT: return "CONSTRAINT";
+    case CES_STR: return "STR";
+    case CES_INT: return "INT";
+    case CES_FLOAT: return "FLOAT";
+    default: break;
+    }
+    return "UNKNOWN";
+}
+
+static void
+dcedumpraw(DCEnode* node, NCbytes* buf)
+{
+    int i;
+    char tmp[1024];
+
+    if(buf == NULL) return;
+    if(node == NULL) {ncbytescat(buf,"<null>"); return;}
+
+    ncbytescat(buf,LBRACE);
+    ncbytescat(buf,(char*)dcesortname(node->sort));
+
+    switch (node->sort) {
+
+    case CES_SLICE: {
+	    DCEslice* slice = (DCEslice*)node;
+	    snprintf(tmp,sizeof(tmp),
+		    " [first=%lu stride=%lu last=%lu len=%lu count=%lu size=%lu]",
+		    (unsigned long)slice->first,
+		    (unsigned long)slice->stride,
+		    (unsigned long)slice->last,
+		    (unsigned long)slice->length,
+		    (unsigned long)slice->count,
+		    (unsigned long)slice->declsize);
+            ncbytescat(buf,tmp);
+    } break;
+
+    case CES_SEGMENT: {
+	DCEsegment* segment = (DCEsegment*)node;
+        int rank = segment->rank;
+	char* name = (segment->name?segment->name:"<unknown>");
+	ncbytescat(buf," name=");
+	ncbytescat(buf,name);
+        snprintf(tmp,sizeof(tmp)," rank=%lu",(unsigned long)rank);
+	ncbytescat(buf,tmp);
+        ncbytescat(buf," defined=");
+        ncbytescat(buf,(segment->slicesdefined?"1":"0"));
+        ncbytescat(buf," declized=");
+        ncbytescat(buf,(segment->slicesdeclized?"1":"0"));
+	if(rank > 0) {
+            ncbytescat(buf," slices=");
+            for(i=0;i<rank;i++) {
+	        DCEslice* slice = segment->slices+i;
+                dcedumpraw((DCEnode*)slice,buf);
+	    }
+	}
+    } break;
+
+    case CES_VAR: {
+	DCEvar* var = (DCEvar*)node;
+        ncbytescat(buf," segments=");
+	dcedumprawlist(var->segments,buf);
+    } break;
+
+    case CES_FCN: {
+	DCEfcn* fcn = (DCEfcn*)node;
+	ncbytescat(buf," name=");
+        ncbytescat(buf,fcn->name);
+        ncbytescat(buf,"args=");
+	dcedumprawlist(fcn->args,buf);
+    } break;
+
+    case CES_CONST: {
+	DCEconstant* value = (DCEconstant*)node;
+	ncbytescat(buf," discrim=");
+	ncbytescat(buf,dcesortname(value->discrim));
+	ncbytescat(buf," value=");
+        switch (value->discrim) {
+        case CES_STR:
+	    ncbytescat(buf,"|");
+	    ncbytescat(buf,value->text);
+	    ncbytescat(buf,"|");
+	    break;		
+        case CES_INT:
+            snprintf(tmp,sizeof(tmp),"%lld",value->intvalue);
+            ncbytescat(buf,tmp);
+    	break;
+        case CES_FLOAT:
+            snprintf(tmp,sizeof(tmp),"%g",value->floatvalue);
+            ncbytescat(buf,tmp);
+	    break;
+        default: assert(0);
+	}
+    } break;
+
+    case CES_VALUE: {
+	DCEvalue* value = (DCEvalue*)node;
+	ncbytescat(buf," discrim=");
+	ncbytescat(buf,dcesortname(value->discrim));
+        switch (value->discrim) {
+        case CES_CONST:
+    	    dcedumpraw((DCEnode*)value->constant,buf);
+      	    break;		
+        case CES_VAR:
+    	    dcedumpraw((DCEnode*)value->var,buf);
+    	    break;
+        case CES_FCN:
+    	    dcedumpraw((DCEnode*)value->fcn,buf);
+    	    break;
+        default: assert(0);
+        }
+    } break;
+
+    case CES_PROJECT: {
+	DCEprojection* target = (DCEprojection*)node;
+	ncbytescat(buf," discrim=");
+	ncbytescat(buf,dcesortname(target->discrim));
+	switch (target->discrim) {
+	case CES_VAR:
+	    dcedumpraw((DCEnode*)target->var,buf);
+	    break;
+	case CES_FCN:
+	    dcedumpraw((DCEnode*)target->fcn,buf);
+	    break;
+	default: assert(0);
+	}
+    } break;
+
+    case CES_SELECT: {
+	DCEselection* sel = (DCEselection*)node;
+	ncbytescat(buf," ");
+	dcedumpraw((DCEnode*)sel->lhs,buf);
+        if(sel->operator == CES_NIL) break;
+        ncbytescat(buf,opstrings[(int)sel->operator]);
+        if(nclistlength(sel->rhs) > 1)
+            ncbytescat(buf,"{");
+	dcedumprawlist(sel->rhs,buf);
+        if(nclistlength(sel->rhs) > 1)
+	    ncbytescat(buf,"}");
+    } break;
+
+    case CES_CONSTRAINT: {
+	DCEconstraint* con = (DCEconstraint*)node;
+        if(con->projections != NULL && nclistlength(con->projections) > 0) {
+	    ncbytescat(buf,"projections=");	
+            dcedumprawlist(con->projections,buf);
+	}
+        if(con->selections != NULL && nclistlength(con->selections) > 0) {
+	    ncbytescat(buf,"selections=");	
+            dcedumprawlist(con->selections,buf);
+	}
+    } break;
+
+    case CES_NIL: {
+	ncbytescat(buf,"<nil>");
+    } break;
+
+    default:
+	assert(0);
+    }
+    ncbytescat(buf,RBRACE);
+}
+
+static void
+dcedumprawlist(NClist* list, NCbytes* buf)
+{
+    int i;
+    if(list == NULL || buf == NULL) return;
+    ncbytescat(buf,"(");
+    for(i=0;i<nclistlength(list);i++) {
+	DCEnode* node = (DCEnode*)nclistget(list,i);
+	if(node == NULL) continue;
+	if(i>0) ncbytescat(buf,",");
+	dcedumpraw((DCEnode*)node,buf);
+    }
+    ncbytescat(buf,")");
+}
+
