@@ -32,7 +32,7 @@
 #define NC_Structure	54
 #define NC_Grid		55
 #define NC_Dimension	56
-#define NC_Primitive	57
+#define NC_Atomic	57
 
 #undef OCCOMPILEBYDEFAULT
 
@@ -70,16 +70,16 @@ typedef unsigned int NCFLAGS;
 #define NCF_NC4             (0x0002) /* DAP->netcdf-4 */
 #define NCF_NCDAP           (0x0004) /* Do libnc-dap mimic */
 #define NCF_CACHE           (0x0008) /* Cache enabled/disabled */
-#define NCF_PREFETCH        (0x0010) /* Cache prefetch enabled/disabled */
-#define NCF_UPGRADE         (0x0020) /* Do proper type upgrades */
-#define NCF_UNCONSTRAINABLE (0x0040) /* Not a constrainable URL */
-#define NCF_SHOWFETCH       (0x0080) /* show fetch calls */
-#define NCF_ONDISK          (0x0100) /* cause oc to store data on disk */
-#define NCF_WHOLEVAR        (0x0200) /* retrieve only whole variables (as opposed to partial variable)
-                                        into cache */
+#define NCF_UPGRADE         (0x0010) /* Do proper type upgrades */
+#define NCF_UNCONSTRAINABLE (0x0020) /* Not a constrainable URL */
+#define NCF_SHOWFETCH       (0x0040) /* show fetch calls */
+#define NCF_ONDISK          (0x0080) /* cause oc to store data on disk */
+#define NCF_WHOLEVAR        (0x0100) /* retrieve only whole variables (as opposed to partial variable) into cache */
+#define NCF_PREFETCH        (0x0200) /* Cache prefetch enabled/disabled */
+#define NCF_PREFETCH_EAGER  (0x0400) /* Do eager prefetch; 0=>lazy */
 
 /* Define all the default on flags */
-#define DFALT_ON_FLAGS (NCF_PREFETCH)
+#define DFALT_ON_FLAGS (NCF_CACHE|NCF_PREFETCH)
 
 typedef struct NCCONTROLS {
     NCFLAGS  flags;
@@ -93,16 +93,17 @@ struct NCTMODEL {
 
 /* Detail information about each cache item */
 typedef struct NCcachenode {
-    int wholevariable; /* does this node only have wholevariables? */
+    int wholevariable; /* does this cache node only have wholevariables? */
     int prefetch; /* is this the prefetch cache entry? */
-    size_t xdrsize;
+    off_t xdrsize;
     DCEconstraint* constraint; /* as used to create this node */
     NClist* vars; /* vars potentially covered by this cache node */
     struct CDFnode* datadds;
-    OCobject ocroot;
-    OCdata content;
+    OCddsnode ocroot;
+    OCdatanode content;
 } NCcachenode;
 
+
 /* All cache info */
 typedef struct NCcache {
     size_t cachelimit; /* max total size for all cached entries */
@@ -115,11 +116,11 @@ typedef struct NCcache {
 /**************************************************/
 /* The DAP packet info from OC */
 typedef struct NCOC {
-    OCconnection conn;
+    OClink conn;
     char* rawurltext; /* as given to nc3d_open */
     char* urltext;    /* as modified by nc3d_open */
-    NC_URI* url;      /* parse of rawuritext */
-    OCobject ocdasroot;
+    NCURI* url;      /* parse of rawuritext */
+    OCdasnode ocdasroot;
     DCEconstraint* dapconstraint; /* from url */
     int inmemory; /* store fetched data in memory? */
 } NCOC;
@@ -127,11 +128,7 @@ typedef struct NCOC {
 typedef struct NCCDF {
     struct CDFnode* ddsroot; /* constrained dds */
     struct CDFnode* fullddsroot; /* unconstrained dds */
-    /* Collected sets of useful nodes (in ddsroot tree space) */
-    NClist*  varnodes; /* nodes which can represent netcdf variables */
-    NClist*  seqnodes; /* sequence nodes; */
-    NClist*  gridnodes; /* grid nodes */
-    NClist*  dimnodes; /* (base) dimension nodes */
+    NClist*  projectedvars; /* vars appearing in nc_open url projections */
     unsigned int defaultstringlength;
     unsigned int defaultsequencelimit; /* global sequence limit;0=>no limit */
     struct NCcache* cache;
@@ -143,8 +140,10 @@ typedef struct NCCDF {
     struct CDFnode* globalstringdim;
     char* recorddimname; /* From DODS_EXTRA */
     struct CDFnode* recorddim;
+#if 0
     /* libncdap4 only */
     NClist*  usertypes; /* nodes which will represent netcdf types */
+#endif
 } NCCDF;
 
 /* Define a structure holding common info for NCDAP{3,4} */
@@ -161,13 +160,18 @@ typedef struct NCDAPCOMMON {
 
 /* Each root CDFnode contains info about the whole tree */
 typedef struct CDFtree {
-    OCobject ocroot;
+    OCddsnode ocroot;
     OCdxd occlass;
     NClist* nodes; /* all nodes in tree*/
     struct CDFnode* root; /* cross link */
     struct NCDAPCOMMON*          owner;
+    /* Collected sets of useful nodes */
+    NClist*  varnodes; /* nodes which can represent netcdf variables */
+    NClist*  seqnodes; /* sequence nodes; */
+    NClist*  gridnodes; /* grid nodes */
+    NClist*  dimnodes; /* (base) dimension nodes */
     /* Classification flags */
-    int regridded; /* Was this tree passed thru regrid3? */
+    int restructed; /* Was this tree passed thru restruct3? */
 } CDFtree;
 
 /* Track the kinds of dimensions */
@@ -192,9 +196,10 @@ typedef struct CDFdim {
 } CDFdim;
 
 typedef struct CDFarray {
-    NClist*  dimsetall; /* inherited+originals+pseudo */
-    NClist*  dimsetplus; /* originals+pseudo */
-    NClist*  dimset0; /* original dims from the dds */
+    NClist*  dimsetall;   /* dimsetplus + inherited */
+    NClist*  dimsetplus;  /* dimset0 + pseudo */
+    NClist*  dimsettrans; /* dimset0 plus inherited(transitive closure) */
+    NClist*  dimset0;     /* original dims from the dds */
     struct CDFnode* stringdim;
     /* Track sequence related information */
     struct CDFnode* seqdim; /* if this node is a sequence */
@@ -235,13 +240,13 @@ typedef struct CDFnode {
     char*            ocname;     /* oc base name */
     char*            ncbasename; /* generally cdflegalname(ocname) */
     char*            ncfullname; /* complete path name from root to this node*/
-    OCobject         ocnode;        /* oc mirror node*/
+    OCddsnode        ocnode;     /* oc mirror node*/
     struct CDFnode*  group;	 /* null => in root group */
     struct CDFnode*  container;  /* e.g. struct or sequence, but not group */
     struct CDFnode*  root;
-    CDFtree*         tree;          /* root level metadata;only defined if root*/
-    CDFdim           dim;           /* nctype == dimension */
-    CDFarray         array;         /* nctype == grid,var,etc. with dimensions */
+    CDFtree*         tree;       /* root level metadata;only defined if root*/
+    CDFdim           dim;        /* nctype == dimension */
+    CDFarray         array;      /* nctype == grid,var,etc. with dimensions */
     NClist*          subnodes;      /* if nctype == grid, sequence, etc. */
     NClist*          attributes;    /*NClist<NCattribute*>*/
     NCDODS           dodsspecial;   /* special attributes like maxStrlen */
@@ -252,13 +257,10 @@ typedef struct CDFnode {
     BOOL	     usesequence;   /* If this sequence is usable */
     BOOL             elided;        /* 1 => node does not partipate in naming*/
     struct CDFnode*  basenode;      /* derived tree map to template tree */
-    BOOL	     visible;       /* 1 => node is present in derived tree; independent of elided flag */
+    BOOL	     invisible;     /* 1 => do not show to user */
     BOOL	     zerodim;       /* 1 => node has a zero dimension */
     /* These two flags track the effects on grids of constraints */
-    BOOL             virtual;       /* node added by regrid */
-#ifdef PROJECTED
-    BOOL             projected;     /* node referenced by projection */
-#endif
+    BOOL             nc_virtual;       /* node added by regrid */
     struct CDFnode* attachment;     /* DDS<->DATADDS cross link*/
     struct CDFnode* template;       /* temporary field for regridding */
     /* Fields for use by libncdap4 */
@@ -269,32 +271,32 @@ typedef struct CDFnode {
     char*            vlenname;      /* for sequence types */
     int              singleton;     /* for singleton sequences */
     unsigned long    estimatedsize; /* > 0 Only for var nodes */
+    BOOL             whole;         /* projected as whole node */
+    BOOL             prefetchable;  /* eligible to be prefetched (if whole) */
 } CDFnode;
 
 /**************************************************/
 /* Shared procedures */
 
 /* From ncdap3.c*/
-extern NCerror freeNCDAPCOMMON(struct NCDAPCOMMON*);
+extern NCerror freeNCDAPCOMMON(NCDAPCOMMON*);
 extern NCerror fetchtemplatemetadata3(NCDAPCOMMON*);
 
 /* From error.c*/
 extern NCerror ocerrtoncerr(OCerror);
 
 /* From: common34.c */
-extern NCerror fixgrid34(struct NCDAPCOMMON* drno, CDFnode* grid);
-extern NCerror computecdfinfo34(struct NCDAPCOMMON*, NClist*);
+extern NCerror fixgrid34(NCDAPCOMMON* drno, CDFnode* grid);
+extern NCerror computecdfinfo34(NCDAPCOMMON*, NClist*);
 extern char* cdfname34(char* basename);
-extern NCerror augmentddstree34(struct NCDAPCOMMON*, NClist*);
-extern NCerror computecdfdimnames34(struct NCDAPCOMMON*);
-extern NCerror buildcdftree34(struct NCDAPCOMMON*, OCobject, OCdxd, CDFnode**);
-extern CDFnode* makecdfnode34(struct NCDAPCOMMON*, char* nm, OCtype,
-			    /*optional*/ OCobject ocnode, CDFnode* container);
+extern NCerror augmentddstree34(NCDAPCOMMON*, NClist*);
+extern NCerror computecdfdimnames34(NCDAPCOMMON*);
+extern NCerror buildcdftree34(NCDAPCOMMON*, OCddsnode, OCdxd, CDFnode**);
+extern CDFnode* makecdfnode34(NCDAPCOMMON*, char* nm, OCtype,
+			    /*optional*/ OCddsnode ocnode, CDFnode* container);
 extern void freecdfroot34(CDFnode*);
-
-extern NCerror findnodedds34(struct NCDAPCOMMON* drno, CDFnode* ddssrc);
-extern NCerror makegetvar34(struct NCDAPCOMMON*, struct CDFnode*, void*, nc_type, struct Getvara**);
-extern NCerror applyclientparams34(struct NCDAPCOMMON* drno);
+extern NCerror makegetvar34(NCDAPCOMMON*, struct CDFnode*, void*, nc_type, struct Getvara**);
+extern NCerror applyclientparams34(NCDAPCOMMON* drno);
 extern NCerror attach34(CDFnode* xroot, CDFnode* ddstarget);
 extern NCerror attachall34(CDFnode* xroot, CDFnode* ddsroot);
 extern NCerror attachsubset34(CDFnode*, CDFnode*);
@@ -302,7 +304,7 @@ extern void unattach34(CDFnode*);
 extern int nodematch34(CDFnode* node1, CDFnode* node2);
 extern int simplenodematch34(CDFnode* node1, CDFnode* node2);
 extern CDFnode* findxnode34(CDFnode* target, CDFnode* xroot);
-extern int constrainable34(NC_URI*);
+extern int constrainable34(NCURI*);
 extern char* makeconstraintstring34(DCEconstraint*);
 extern size_t estimatedataddssize34(CDFnode* datadds);
 extern void canonicalprojection34(NClist*, NClist*);
@@ -310,11 +312,12 @@ extern NClist* getalldims34(NCDAPCOMMON* nccomm, int visibleonly);
 
 /* From cdf3.c */
 extern NCerror dimimprint3(NCDAPCOMMON*);
-extern NCerror definedimsets3(struct NCDAPCOMMON*);
+extern NCerror definedimsets3(NCDAPCOMMON*,CDFtree*);
 
 /* From cache.c */
 extern int iscached(NCDAPCOMMON*, CDFnode* target, NCcachenode** cachenodep);
 extern NCerror prefetchdata3(NCDAPCOMMON*);
+extern NCerror markprefetch3(NCDAPCOMMON*);
 extern NCerror buildcachenode34(NCDAPCOMMON*,
 	        DCEconstraint* constraint,
 		NClist* varlist,
@@ -330,4 +333,9 @@ extern void freenccache(NCDAPCOMMON*, NCcache* cache);
 */
 extern int nc__opendap(void);
 
+/* Define accessors for the dispatchdata */
+#define NCD3_DATA(nc) ((NCDAPCOMMON*)(nc)->dispatchdata)
+#define NCD3_DATA_SET(nc,data) ((nc)->dispatchdata = (void*)(data))
+
+
 #endif /*NCCOMMON_H*/
