@@ -15,6 +15,9 @@
 #  endif
 #endif
 
+#include "oclog.h"
+
+#include "nc3internal.h"
 #include "nc3dispatch.h"
 #include "ncd3dispatch.h"
 #include "dapalign.h"
@@ -29,8 +32,10 @@ static NCerror buildattribute3a(NCDAPCOMMON*, NCattribute*, nc_type, int);
 
 static char* getdefinename(CDFnode* node);
 
-extern CDFnode* v4node;
-int nc3dinitialized = 0;
+static int ncd3initialized = 0;
+
+size_t dap_one[NC_MAX_VAR_DIMS];
+size_t dap_zero[NC_MAX_VAR_DIMS];
 
 /**************************************************/
 /* Add an extra function whose sole purpose is to allow
@@ -41,11 +46,22 @@ int nc__opendap(void) {return 0;}
 /**************************************************/
 /* Do local initialization */
 
-int
-nc3dinitialize(void)
+static int
+ncd3initialize(void)
 {
+    int i;
     compute_nccalignments();
-    nc3dinitialized = 1;
+    for(i=0;i<NC_MAX_VAR_DIMS;i++) {
+	dap_one[i] = 1;
+	dap_zero[i] = 0;
+    }
+    ncd3initialized = 1;
+#ifdef DEBUG
+    /* force logging to go to stderr */
+    nclogclose();
+    if(nclogopen(NULL))
+        ncsetlogging(1); /* turn it on */
+#endif
     return NC_NOERR;
 }
 
@@ -56,35 +72,25 @@ int
 NCD3_open(const char * path, int mode,
                int basepe, size_t *chunksizehintp,
  	       int useparallel, void* mpidata,
-               NC_Dispatch* dispatch, NC** ncpp)
+               NC_Dispatch* dispatch, NC* drno)
 {
     NCerror ncstat = NC_NOERR;
     OCerror ocstat = OC_NOERR;
-    NC* drno = NULL;
     NCDAPCOMMON* dapcomm = NULL;
     const char* value;
-    /* We will use a fake file descriptor as our internal in-memory filename */
-    char tmpname[32];
 
-    if(!nc3dinitialized) nc3dinitialize();
+    if(!ncd3initialized) ncd3initialize();
 
     if(path == NULL)
 	return NC_EDAPURL;
     if(dispatch == NULL) PANIC("NC3D_open: no dispatch table");
 
     /* Setup our NC and NCDAPCOMMON state*/
-    drno = (NC*)calloc(1,sizeof(NC));
-    if(drno == NULL) {ncstat = NC_ENOMEM; goto done;}
-
-    /* compute an ncid */
-    ncstat = add_to_NCList(drno);
-    if(ncstat != NC_NOERR) {THROWCHK(ncstat); goto done;}
 
     dapcomm = (NCDAPCOMMON*)calloc(1,sizeof(NCDAPCOMMON));
     if(dapcomm == NULL) {ncstat = NC_ENOMEM; goto done;}
 
-    drno->dispatch = dispatch;
-    drno->dispatchdata = dapcomm;
+    NCD3_DATA_SET(drno,dapcomm);
     drno->int_ncid = nc__pseudofd(); /* create a unique id */
     dapcomm->controller = (NC*)drno;
 
@@ -109,23 +115,37 @@ NCD3_open(const char * path, int mode,
     dapcomm->oc.rawurltext = strdup(path);
 #endif
 
-    nc_uriparse(dapcomm->oc.rawurltext,&dapcomm->oc.url);
+    ncuriparse(dapcomm->oc.rawurltext,&dapcomm->oc.url);
 
     /* parse the client parameters */
-    nc_uridecodeparams(dapcomm->oc.url);
+    ncuridecodeparams(dapcomm->oc.url);
 
     if(!constrainable34(dapcomm->oc.url))
 	SETFLAG(dapcomm->controls,NCF_UNCONSTRAINABLE);
 
-    /* Use libsrc code for storing metadata */
+    /* fail if we are unconstrainable but have constraints */
+    if(FLAGSET(dapcomm->controls,NCF_UNCONSTRAINABLE)) {
+	if(dapcomm->oc.url->constraint != NULL) {
+	    nclog(NCLOGWARN,"Attempt to constrain an unconstrainable data source: %s",
+		   dapcomm->oc.url->constraint);
+	    ncstat = THROW(NC_EDAPCONSTRAINT);
+	    goto done;
+	}
+    }
 
-    snprintf(tmpname,sizeof(tmpname),"%d",drno->int_ncid);
-    /* Now, use the file to create the netcdf file */
-    if(sizeof(size_t) == sizeof(unsigned int))
-	ncstat = nc_create(tmpname,NC_DISKLESS,&drno->substrate);
-    else
-	ncstat = nc_create(tmpname,NC_DISKLESS|NC_64BIT_OFFSET,&drno->substrate);
-    if(ncstat != NC_NOERR) {THROWCHK(ncstat); goto done;}
+    /* Use libsrc code for storing metadata */
+    {
+	char tmpname[32];
+
+        /* Create fake file name: exact name must be unique,
+           but is otherwise irrelevant because we are using NC_DISKLESS
+        */
+        snprintf(tmpname,sizeof(tmpname),"%d",drno->int_ncid);
+
+        /* Now, use the file to create the netcdf file; force classic.  */
+        ncstat = nc_create(tmpname,NC_DISKLESS|NC_CLASSIC_MODEL,&drno->substrate);
+        if(ncstat != NC_NOERR) {THROWCHK(ncstat); goto done;}
+    }
 
     /* Avoid fill */
     nc_set_fill(drno->substrate,NC_NOFILL,NULL);
@@ -133,23 +153,17 @@ NCD3_open(const char * path, int mode,
     dapcomm->oc.dapconstraint = (DCEconstraint*)dcecreate(CES_CONSTRAINT);
     dapcomm->oc.dapconstraint->projections = nclistnew();
     dapcomm->oc.dapconstraint->selections = nclistnew();
+    
+    if(dapcomm->oc.url != NULL) {
+        /* Parse constraints to make sure they are syntactically correct */
+        ncstat = parsedapconstraints(dapcomm,dapcomm->oc.url->constraint,dapcomm->oc.dapconstraint);
+        if(ncstat != NC_NOERR) {THROWCHK(ncstat); goto done;}
+     } else
+	dapcomm->oc.dapconstraint = NULL;
 
-    /* Parse constraints to make sure they are syntactically correct */
-    ncstat = parsedapconstraints(dapcomm,dapcomm->oc.url->constraint,dapcomm->oc.dapconstraint);
-    if(ncstat != NC_NOERR) {THROWCHK(ncstat); goto done;}
-
-    /* Complain if we are unconstrainable but have constraints */
-    if(FLAGSET(dapcomm->controls,NCF_UNCONSTRAINABLE)) {
-	if(dapcomm->oc.url->constraint != NULL
-	   && strlen(dapcomm->oc.url->constraint) > 0) {
-	    nclog(NCLOGWARN,"Attempt to constrain an unconstrainable data source: %s",
-		   dapcomm->oc.url->constraint);
-	}
-    }
-
-    /* Construct a url for oc minus any parameters */
-    dapcomm->oc.urltext = nc_uribuild(dapcomm->oc.url,NULL,NULL,
-				(NC_URIALL ^ NC_URICONSTRAINTS));
+    /* Construct a url for oc minus any constraint and params*/
+    dapcomm->oc.urltext = ncuribuild(dapcomm->oc.url,NULL,NULL,
+				      (NCURISTD ^ NCURICONSTRAINTS));
 
     /* Pass to OC */
     ocstat = oc_open(dapcomm->oc.urltext,&dapcomm->oc.conn);
@@ -164,18 +178,31 @@ NCD3_open(const char * path, int mode,
     /* Turn on logging; only do this after oc_open*/
     if((value = paramvalue34(dapcomm,"log")) != NULL) {
 	ncloginit();
-        ncsetlogging(1);
-        nclogopen(value);
-	oc_loginit();
-        oc_setlogging(1);
-        oc_logopen(value);
+        if(nclogopen(value))
+	    ncsetlogging(1);
+	ocloginit();
+        if(oclogopen(value))
+	    ocsetlogging(1);
     }
 
-    /* fetch and build the (almost) unconstrained DDS for use as
+    /* fetch and build the unconstrained DDS for use as
        template */
     ncstat = fetchtemplatemetadata3(dapcomm);
     if(ncstat != NC_NOERR) goto done;
 
+    /* Operations on the template tree */
+
+    /* Accumulate useful nodes sets  */
+    ncstat = computecdfnodesets3(dapcomm,dapcomm->cdf.fullddsroot->tree);
+    if(ncstat) {THROWCHK(ncstat); goto done;}
+
+    /* Define the dimsettrans list */
+    ncstat = definedimsets3(dapcomm,dapcomm->cdf.fullddsroot->tree);
+    if(ncstat) {THROWCHK(ncstat); goto done;}
+
+    /* Mark the nodes of the template that are eligible for prefetch */
+    ncstat = markprefetch3(dapcomm);
+
     /* fetch and build the constrained DDS */
     ncstat = fetchconstrainedmetadata3(dapcomm);
     if(ncstat != NC_NOERR) goto done;
@@ -184,11 +211,10 @@ NCD3_open(const char * path, int mode,
 fprintf(stderr,"constrained dds: %s\n",dumptree(dapcomm->cdf.ddsroot));
 #endif
 
-
-    /* The following actions are (mostly) WRT to the constrained tree */
+    /* Operations on the constrained tree */
 
     /* Accumulate useful nodes sets  */
-    ncstat = computecdfnodesets3(dapcomm);
+    ncstat = computecdfnodesets3(dapcomm,dapcomm->cdf.ddsroot->tree);
     if(ncstat) {THROWCHK(ncstat); goto done;}
 
     /* Fix grids */
@@ -211,14 +237,14 @@ fprintf(stderr,"constrained dds: %s\n",dumptree(dapcomm->cdf.ddsroot));
     ncstat = addstringdims(dapcomm);
     if(ncstat) {THROWCHK(ncstat); goto done;}
 
-    if(nclistlength(dapcomm->cdf.seqnodes) > 0) {
+    if(nclistlength(dapcomm->cdf.ddsroot->tree->seqnodes) > 0) {
 	/* Build the sequence related dimensions */
         ncstat = defseqdims(dapcomm);
         if(ncstat) {THROWCHK(ncstat); goto done;}
     }
 
     /* Define the dimsetplus and dimsetall lists */
-    ncstat = definedimsets3(dapcomm);
+    ncstat = definedimsets3(dapcomm,dapcomm->cdf.ddsroot->tree);
     if(ncstat) {THROWCHK(ncstat); goto done;}
 
     /* Re-compute the dimension names*/
@@ -236,13 +262,15 @@ fprintf(stderr,"constrained dds: %s\n",dumptree(dapcomm->cdf.ddsroot));
     ncstat = defrecorddim3(dapcomm);
     if(ncstat) {THROWCHK(ncstat); goto done;}
     if(dapcomm->cdf.recorddimname != NULL
-       && nclistlength(dapcomm->cdf.seqnodes) > 0) {
+       && nclistlength(dapcomm->cdf.ddsroot->tree->seqnodes) > 0) {
 	/*nclog(NCLOGWARN,"unlimited dimension specified, but sequences exist in DDS");*/
 	PANIC("unlimited dimension specified, but sequences exist in DDS");	
     }
 
     /* Re-compute the var names*/
-    ncstat = computecdfvarnames3(dapcomm,dapcomm->cdf.ddsroot,dapcomm->cdf.varnodes);
+    ncstat = computecdfvarnames3(dapcomm,
+				 dapcomm->cdf.ddsroot,
+				 dapcomm->cdf.ddsroot->tree->varnodes);
     if(ncstat) {THROWCHK(ncstat); goto done;}
 
     /* Transfer data from the unconstrained DDS data to the unconstrained DDS */
@@ -262,15 +290,19 @@ fprintf(stderr,"constrained dds: %s\n",dumptree(dapcomm->cdf.ddsroot));
     ncstat = qualifyconstraints3(dapcomm->oc.dapconstraint);
     if(ncstat != NC_NOERR) goto done;
 
+    /* Accumulate set of variables in the constraint's projections */
+    ncstat = computeprojectedvars(dapcomm,dapcomm->oc.dapconstraint);
+    if(ncstat) {THROWCHK(ncstat); goto done;}
+
     /* using the modified constraint, rebuild the constraint string */
     if(FLAGSET(dapcomm->controls,NCF_UNCONSTRAINABLE)) {
 	/* ignore all constraints */
-	dapcomm->oc.urltext = nc_uribuild(dapcomm->oc.url,NULL,NULL,0);
+	dapcomm->oc.urltext = ncuribuild(dapcomm->oc.url,NULL,NULL,0);
     } else {
 	char* constraintstring = buildconstraintstring3(dapcomm->oc.dapconstraint);
-        nc_urisetconstraints(dapcomm->oc.url,constraintstring);
+        ncurisetconstraints(dapcomm->oc.url,constraintstring);
 	nullfree(constraintstring);
-        dapcomm->oc.urltext = nc_uribuild(dapcomm->oc.url,NULL,NULL,NC_URICONSTRAINTS);
+        dapcomm->oc.urltext = ncuribuild(dapcomm->oc.url,NULL,NULL,NCURICONSTRAINTS);
     }
 
 #ifdef DEBUG
@@ -284,8 +316,40 @@ fprintf(stderr,"ncdap3: final constraint: %s\n",dapcomm->oc.url->constraint);
     ncstat = buildncstructures3(dapcomm);
     if(ncstat != NC_NOERR) {THROWCHK(ncstat); goto done;}
 
+    /* Explicitly do not call enddef because it will complain
+       about variables that are too large.
+    */
+#if 0
+    ncstat = nc_endef(drno->substrate,NC_NOFILL,NULL);
+    if(ncstat != NC_NOERR && ncstat != NC_EVARSIZE)
+        {THROWCHK(ncstat); goto done;}
+#endif
+	
+    {
+        NC* ncsub;
+        NC* drno = dapcomm->controller;
+	CDFnode* unlimited = dapcomm->cdf.recorddim;
+        /* (for now) break abstractions*/
+	NC3_INFO* nc3i;
+
+        /* get the id for the substrate */
+        ncstat = NC_check_id(drno->substrate,&ncsub);
+        if(ncstat != NC_NOERR) {THROWCHK(ncstat); goto done;}
+	nc3i = (NC3_INFO*)ncsub->dispatchdata;
+
+        if(unlimited != NULL) {
+            /* Set the effective size of UNLIMITED */
+            NC_set_numrecs(nc3i,unlimited->dim.declsize);
+        }
+
+        /* Pretend the substrate is read-only */
+	NC_set_readonly(nc3i);
+	
+    }
+
     /* Do any necessary data prefetch */
-    if(FLAGSET(dapcomm->controls,NCF_PREFETCH)) {
+    if(FLAGSET(dapcomm->controls,NCF_PREFETCH)
+       && FLAGSET(dapcomm->controls,NCF_PREFETCH_EAGER)) {
         ncstat = prefetchdata3(dapcomm);
         if(ncstat != NC_NOERR) {
             del_from_NCList((NC*)drno); /* undefine here */
@@ -293,20 +357,6 @@ fprintf(stderr,"ncdap3: final constraint: %s\n",dapcomm->oc.url->constraint);
 	}
     }
 
-#ifdef BUG
-    /* The libsrc code (NC_begins) assumes that
-       a created files is new and hence must have an
-       unlimited dimension of 0 initially, which will
-       wipe out the effect of the NC_set_numrecs in builddims.
-       There is no easy workaround, so we suppress the call
-       to nc_enddef
-    */
-    ncstat = nc_enddef(drno->substrate);
-    if(ncstat != NC_NOERR) {THROWCHK(ncstat); goto done;}
-#endif
-
-    if(ncpp) *ncpp = (NC*)drno;
-
     return ncstat;
 
 done:
@@ -324,16 +374,16 @@ NCD3_close(int ncid)
 
     ncstatus = NC_check_id(ncid, (NC**)&drno); 
     if(ncstatus != NC_NOERR) return THROW(ncstatus);
-
     dapcomm = (NCDAPCOMMON*)drno->dispatchdata;
+
+    /* We call abort rather than close to avoid
+       trying to write anything or try to pad file length
+     */
     ncstatus = nc_abort(drno->substrate);
 
-    /* remove ourselves from NClist */
-    del_from_NCList(drno);
     /* clean NC* */
     freeNCDAPCOMMON(dapcomm);
-    if(drno->path != NULL) free(drno->path);
-    free(drno);
+
     return THROW(ncstatus);
 }
 
@@ -344,6 +394,7 @@ buildncstructures3(NCDAPCOMMON* dapcomm)
     NCerror ncstat = NC_NOERR;
     CDFnode* dds = dapcomm->cdf.ddsroot;
     NC* ncsub;
+
     NC_check_id(dapcomm->controller->substrate,&ncsub);
 
     ncstat = buildglobalattrs3(dapcomm,dds);
@@ -371,7 +422,7 @@ builddims(NCDAPCOMMON* dapcomm)
     char* definename;
 
     /* collect all dimensions from variables */
-    dimset = dapcomm->cdf.dimnodes;
+    dimset = dapcomm->cdf.ddsroot->tree->dimnodes;
 
     /* Sort by fullname just for the fun of it */
     for(;;) {
@@ -381,8 +432,8 @@ builddims(NCDAPCOMMON* dapcomm)
 	    CDFnode* dim1 = (CDFnode*)nclistget(dimset,i);
 	    CDFnode* dim2 = (CDFnode*)nclistget(dimset,i+1);
    	    if(strcmp(dim1->ncfullname,dim2->ncfullname) > 0) {
-		nclistset(dimset,i,(ncelem)dim2);
-		nclistset(dimset,i+1,(ncelem)dim1);
+		nclistset(dimset,i,(void*)dim2);
+		nclistset(dimset,i+1,(void*)dim1);
 		swap = 1;
 		break;
 	    }
@@ -404,10 +455,13 @@ builddims(NCDAPCOMMON* dapcomm)
         /* get the id for the substrate */
         ncstat = NC_check_id(drno->substrate,&ncsub);
         if(ncstat != NC_NOERR) {THROWCHK(ncstat); goto done;}
-
+#if 0
+	nc3sub = (NC3_INFO*)&ncsub->dispatchdata;
         /* Set the effective size of UNLIMITED;
            note that this cannot easily be done thru the normal API.*/
-        NC_set_numrecs(ncsub,unlimited->dim.declsize);
+        NC_set_numrecs(nc3sub,unlimited->dim.declsize);
+#endif
+
     }
 
     for(i=0;i<nclistlength(dimset);i++) {
@@ -447,7 +501,7 @@ buildvars(NCDAPCOMMON* dapcomm)
     int i,j;
     NCerror ncstat = NC_NOERR;
     int varid;
-    NClist* varnodes = dapcomm->cdf.varnodes;
+    NClist* varnodes = dapcomm->cdf.ddsroot->tree->varnodes;
     NC* drno = dapcomm->controller;
     char* definename;
 
@@ -458,7 +512,7 @@ buildvars(NCDAPCOMMON* dapcomm)
 	unsigned int ncrank;
         NClist* vardims = NULL;
 
-	if(!var->visible) continue;
+	if(var->invisible) continue;
 	if(var->array.basevar != NULL) continue;
 
 #ifdef DEBUG1
@@ -573,7 +627,7 @@ buildglobalattrs3(NCDAPCOMMON* dapcomm, CDFnode* root)
     if(paramcheck34(dapcomm,"show","dds")) {
 	txt = NULL;
 	if(dapcomm->cdf.ddsroot != NULL)
-  	    txt = oc_inq_text(dapcomm->oc.conn,dapcomm->cdf.ddsroot->ocnode);
+  	    txt = oc_tree_text(dapcomm->oc.conn,dapcomm->cdf.ddsroot->ocnode);
 	if(txt != NULL) {
 	    /* replace newlines with spaces*/
 	    nltxt = nulldup(txt);
@@ -584,8 +638,8 @@ buildglobalattrs3(NCDAPCOMMON* dapcomm, CDFnode* root)
     }
     if(paramcheck34(dapcomm,"show","das")) {
 	txt = NULL;
-	if(dapcomm->oc.ocdasroot != OCNULL)
-	    txt = oc_inq_text(dapcomm->oc.conn,dapcomm->oc.ocdasroot);
+	if(dapcomm->oc.ocdasroot != NULL)
+	    txt = oc_tree_text(dapcomm->oc.conn,dapcomm->oc.ocdasroot);
 	if(txt != NULL) {
 	    nltxt = nulldup(txt);
 	    for(p=nltxt;*p;p++) {if(*p == '\n' || *p == '\r' || *p == '\t') {*p = ' ';}};
@@ -664,7 +718,7 @@ getdefinename(CDFnode* node)
     NClist* path = NULL;
 
     switch (node->nctype) {
-    case NC_Primitive:
+    case NC_Atomic:
 	/* The define name is same as the fullname with elided nodes */
 	path = nclistnew();
         collectnodepath3(node,path,!WITHDATASET);
