@@ -1,4 +1,4 @@
-/*********************************************************************
+/********************************************************************* \
  *   Copyright 1993, UCAR/Unidata
  *   See netcdf/COPYRIGHT file for copying and redistribution conditions.
  *********************************************************************/
@@ -34,10 +34,7 @@ freeNCDAPCOMMON(NCDAPCOMMON* dapcomm)
     /* abort the metadata file */
     (void)nc_abort(getncid(dapcomm));
     freenccache(dapcomm,dapcomm->cdf.cache);
-    nclistfree(dapcomm->cdf.varnodes);
-    nclistfree(dapcomm->cdf.seqnodes);
-    nclistfree(dapcomm->cdf.gridnodes);
-    nclistfree(dapcomm->cdf.usertypes);
+    nclistfree(dapcomm->cdf.projectedvars);
     nullfree(dapcomm->cdf.recorddimname);
 
     /* free the trees */
@@ -49,7 +46,7 @@ freeNCDAPCOMMON(NCDAPCOMMON* dapcomm)
         oc_root_free(dapcomm->oc.conn,dapcomm->oc.ocdasroot);
     dapcomm->oc.ocdasroot = NULL;
     oc_close(dapcomm->oc.conn); /* also reclaims remaining OC trees */
-    nc_urifree(dapcomm->oc.url);
+    ncurifree(dapcomm->oc.url);
     nullfree(dapcomm->oc.urltext);
     nullfree(dapcomm->oc.rawurltext);
 
@@ -71,7 +68,7 @@ addstringdims(NCDAPCOMMON* dapcomm)
        All such dimensions are global.
     */
     int i;
-    NClist* varnodes = dapcomm->cdf.varnodes;
+    NClist* varnodes = dapcomm->cdf.ddsroot->tree->varnodes;
     CDFnode* globalsdim = NULL;
     char dimname[4096];
     size_t dimsize;
@@ -79,9 +76,9 @@ addstringdims(NCDAPCOMMON* dapcomm)
     /* Start by creating the global string dimension */
     snprintf(dimname,sizeof(dimname),"maxStrlen%lu",
 	    (unsigned long)dapcomm->cdf.defaultstringlength);
-    globalsdim = makecdfnode34(dapcomm, dimname, OC_Dimension, OCNULL,
+    globalsdim = makecdfnode34(dapcomm, dimname, OC_Dimension, NULL,
                                  dapcomm->cdf.ddsroot);
-    nclistpush(dapcomm->cdf.ddsroot->tree->nodes,(ncelem)globalsdim);
+    nclistpush(dapcomm->cdf.ddsroot->tree->nodes,(void*)globalsdim);
     DIMFLAGSET(globalsdim,CDFDIMSTRING);
     globalsdim->dim.declsize = dapcomm->cdf.defaultstringlength;
     globalsdim->dim.declsize0 = globalsdim->dim.declsize;
@@ -113,10 +110,10 @@ addstringdims(NCDAPCOMMON* dapcomm)
 	    else
 	        snprintf(dimname,sizeof(dimname),"maxStrlen%lu",
 			 (unsigned long)dimsize);
-	    sdim = makecdfnode34(dapcomm, dimname, OC_Dimension, OCNULL,
+	    sdim = makecdfnode34(dapcomm, dimname, OC_Dimension, NULL,
                                  dapcomm->cdf.ddsroot);
 	    if(sdim == NULL) return THROW(NC_ENOMEM);
-	    nclistpush(dapcomm->cdf.ddsroot->tree->nodes,(ncelem)sdim);
+	    nclistpush(dapcomm->cdf.ddsroot->tree->nodes,(void*)sdim);
 	    DIMFLAGSET(sdim,CDFDIMSTRING);
 	    sdim->dim.declsize = dimsize;
 	    sdim->dim.declsize0 = dimsize;
@@ -139,7 +136,7 @@ defrecorddim3(NCDAPCOMMON* dapcomm)
 
     if(dapcomm->cdf.recorddimname == NULL) return NC_NOERR; /* ignore */
     /* Locate the base dimension matching the record dim */
-    basedims = dapcomm->cdf.dimnodes;
+    basedims = dapcomm->cdf.ddsroot->tree->dimnodes;
     for(i=0;i<nclistlength(basedims);i++) {
         CDFnode* dim = (CDFnode*)nclistget(basedims,i);
 	if(strcmp(dim->ocname,dapcomm->cdf.recorddimname) != 0) continue;
@@ -170,8 +167,8 @@ defseqdims(NCDAPCOMMON* dapcomm)
 	All other sequences will be ignored.
     */
 
-    for(i=0;i<nclistlength(dapcomm->cdf.seqnodes);i++) {
-        CDFnode* seq = (CDFnode*)nclistget(dapcomm->cdf.seqnodes,i);
+    for(i=0;i<nclistlength(dapcomm->cdf.ddsroot->tree->seqnodes);i++) {
+        CDFnode* seq = (CDFnode*)nclistget(dapcomm->cdf.ddsroot->tree->seqnodes,i);
 	size_t seqsize;
 	CDFnode* sqdim = NULL;
 	CDFnode* container;
@@ -211,9 +208,9 @@ getseqdimsize(NCDAPCOMMON* dapcomm, CDFnode* seq, size_t* sizep)
 {
     NCerror ncstat = NC_NOERR;
     OCerror ocstat = OC_NOERR;
-    OCconnection conn = dapcomm->oc.conn;
-    OCdata rootcontent = OCNULL;
-    OCobject ocroot;
+    OClink conn = dapcomm->oc.conn;
+    OCdatanode rootcontent = NULL;
+    OCddsnode ocroot;
     CDFnode* dxdroot;
     CDFnode* xseq;
     NCbytes* seqcountconstraints = ncbytesnew();
@@ -280,9 +277,9 @@ makeseqdim(NCDAPCOMMON* dapcomm, CDFnode* seq, size_t count, CDFnode** sqdimp)
     CDFtree* tree = root->tree;
 
     /* build the dimension with given size; keep the dimension anonymous */
-    sqdim = makecdfnode34(dapcomm,seq->ocname,OC_Dimension,OCNULL,root);
+    sqdim = makecdfnode34(dapcomm,seq->ocname,OC_Dimension,NULL,root);
     if(sqdim == NULL) return THROW(NC_ENOMEM);
-    nclistpush(tree->nodes,(ncelem)sqdim);
+    nclistpush(tree->nodes,(void*)sqdim);
     /* Assign a name to the sequence node */
     sqdim->ncbasename = cdflegalname3(seq->ocname);
     sqdim->ncfullname = nulldup(sqdim->ncbasename);
@@ -302,12 +299,10 @@ countsequence(NCDAPCOMMON* dapcomm, CDFnode* xseq, size_t* sizep)
     int index;
     OCerror ocstat = OC_NOERR;
     NCerror ncstat = NC_NOERR;
-    OCconnection conn = dapcomm->oc.conn;
+    OClink conn = dapcomm->oc.conn;
     size_t recordcount;
     CDFnode* xroot;
-    CDFnode* current;
-    OCdata datacontainer = OCNULL;
-    OCmode mode;
+    OCdatanode data = NULL;
 
     ASSERT((xseq->nctype == NC_Sequence));
 
@@ -315,50 +310,58 @@ countsequence(NCDAPCOMMON* dapcomm, CDFnode* xseq, size_t* sizep)
     collectnodepath3(xseq,path,WITHDATASET);
 
     /* Get tree root */
+    ASSERT(xseq->root == (CDFnode*)nclistget(path,0));
     xroot = xseq->root;
-    datacontainer = oc_data_new(conn);
-    ocstat = oc_data_root(conn,xroot->tree->ocroot,datacontainer);
-    if(ocstat) goto fail;
+    ocstat = oc_data_getroot(conn,xroot->tree->ocroot,&data);
+    if(ocstat) goto done;
 
-    /* walk to the sequence object; control the movement to the next node
-       based on mode */
-    current = (CDFnode*)nclistget(path,0);
-    for(i=0;;) {
-	OCdata child = OCNULL;
-        CDFnode* next = NULL;
-	ocstat = oc_data_mode(conn,datacontainer,&mode);
-	if(ocstat != OC_NOERR) goto fail;
-	switch (mode) {
-	case OCFIELDMODE:
-	    i++;
-	    next = (CDFnode*)nclistget(path,i);
+    /* Basically we use the path to walk the data instances to reach
+       the sequence instance
+    */
+    for(i=0;i<nclistlength(path);i++) {
+        CDFnode* current = (CDFnode*)nclistget(path,i);
+	OCdatanode nextdata = NULL;
+	CDFnode* next = NULL;
+	
+	/* invariant: current = ith node in path; data = corresponding
+           datanode
+        */
+
+	/* get next node in next and next instance in nextdata */
+	if(current->nctype == NC_Structure
+	   || current->nctype == NC_Dataset) {
+	    if(nclistlength(current->array.dimset0) > 0) {
+		/* Cannot handle this case */
+		ncstat = THROW(NC_EDDS);
+		goto done;
+	    }
+	    /* get next node in path; structure/dataset => exists */
+	    next = (CDFnode*)nclistget(path,i+1);
 	    index = fieldindex(current,next);
-	    break;	    
-	case OCARRAYMODE:
-	    index = 0;
-	    break;
-	case OCSEQUENCEMODE:
-	    goto exitloop;
-	default:
+            /* Move to appropriate field */
+	    ocstat = oc_data_ithfield(conn,data,index,&nextdata);
+	    if(ocstat) goto done;
+	    oc_data_free(conn,data);
+	    data = nextdata; /* set up for next loop iteration */
+	} else if(current->nctype ==  NC_Sequence) {
+	    /* Check for nested Sequences */
+	    if(current != xseq) {
+		/* Cannot handle this case */
+		ncstat = THROW(NC_EDDS);
+		goto done;
+	    }
+	    /* Get the record count */
+	    ocstat = oc_data_recordcount(conn,data,&recordcount);
+    	    if(sizep) *sizep = recordcount;
+	    oc_data_free(conn,data); /* reclaim */
+	    break; /* leave the loop */
+	} else {
 	    PANIC("unexpected mode");
 	    return NC_EINVAL;
         }
-	child = oc_data_new(conn);
-	ocstat = oc_data_ith(conn,datacontainer,index,child);
-	if(ocstat) goto fail;
-	/* move to the next node only if it is defined */
-	if(next != NULL)
-	    current = next;
-	oc_data_free(conn,datacontainer);
-        datacontainer = child;
     }
-exitloop:
-    ASSERT(current == xseq && mode == OCSEQUENCEMODE);
-    oc_data_count(conn,datacontainer,&recordcount);
-    if(sizep) *sizep = recordcount;
 
-fail:
-    oc_data_free(conn,datacontainer);
+done:
     nclistfree(path);
     if(ocstat) ncstat = ocerrtoncerr(ocstat);
     return THROW(ncstat);
@@ -493,7 +496,7 @@ prefer(CDFnode* candidate, CDFnode* newchoice)
     newisscalar = (nclistlength(newchoice->array.dimset0) == 0);
     canisscalar = (nclistlength(candidate->array.dimset0) == 0);
 
-    ASSERT(candidate->nctype == NC_Primitive && newchoice->nctype == NC_Primitive);
+    ASSERT(candidate->nctype == NC_Atomic && newchoice->nctype == NC_Atomic);
     
     /* choose non-string over string */
     if(canisstring && !newisstring)
@@ -522,13 +525,12 @@ computeseqcountconstraints3r(NCDAPCOMMON* dapcomm, CDFnode* node, CDFnode** cand
     candidate = NULL;
     compound = NULL;
 
-    for(i=0;i<nclistlength(node->subnodes);i++){
+    for(i=0;i<nclistlength(node->subnodes);i++) {
         CDFnode* subnode = (CDFnode*)nclistget(node->subnodes,i);
         if(subnode->nctype == NC_Structure || subnode->nctype == NC_Grid)
 	    compound = subnode; /* save for later recursion */
-	else if(subnode->nctype == NC_Primitive) {
+	else if(subnode->nctype == NC_Atomic)
 	    candidate = prefer(candidate,subnode);
-	}
     }
     if(candidate == NULL && compound == NULL) {
 	PANIC("cannot find candidate for seqcountconstraints for a sequence");
@@ -565,8 +567,8 @@ estimatevarsizes3(NCDAPCOMMON* dapcomm)
     unsigned int rank;
     size_t totalsize = 0;
 
-    for(ivar=0;ivar<nclistlength(dapcomm->cdf.varnodes);ivar++) {
-        CDFnode* var = (CDFnode*)nclistget(dapcomm->cdf.varnodes,ivar);
+    for(ivar=0;ivar<nclistlength(dapcomm->cdf.ddsroot->tree->varnodes);ivar++) {
+        CDFnode* var = (CDFnode*)nclistget(dapcomm->cdf.ddsroot->tree->varnodes,ivar);
 	NClist* ncdims = var->array.dimset0;
 	rank = nclistlength(ncdims);
 	if(rank == 0) { /* use instance size of the type */
@@ -597,7 +599,7 @@ fetchtemplatemetadata3(NCDAPCOMMON* dapcomm)
 {
     NCerror ncstat = NC_NOERR;
     OCerror ocstat = OC_NOERR;
-    OCobject ocroot = OCNULL;
+    OCddsnode ocroot = NULL;
     CDFnode* ddsroot = NULL;
     char* ce = NULL;
 
@@ -635,11 +637,11 @@ fetchtemplatemetadata3(NCDAPCOMMON* dapcomm)
     if(ocstat != OC_NOERR) {
 	/* Ignore but complain */
 	nclog(NCLOGWARN,"Could not read DAS; ignored");
-        dapcomm->oc.ocdasroot = OCNULL;	
+        dapcomm->oc.ocdasroot = NULL;	
 	ocstat = OC_NOERR;
     }
 
-    /* Construct our parallel dds tree */
+    /* Construct the netcdf cdf tree corresponding to the dds tree*/
     ncstat = buildcdftree34(dapcomm,ocroot,OCDDS,&ddsroot);
     if(ncstat != NC_NOERR) {THROWCHK(ncstat); goto done;}
     dapcomm->cdf.fullddsroot = ddsroot;
@@ -655,7 +657,7 @@ fetchconstrainedmetadata3(NCDAPCOMMON* dapcomm)
 {
     NCerror ncstat = NC_NOERR;
     OCerror ocstat = OC_NOERR;
-    OCobject ocroot;
+    OCddsnode ocroot;
     CDFnode* ddsroot; /* constrained */
     char* ce = NULL;
 
@@ -663,7 +665,6 @@ fetchconstrainedmetadata3(NCDAPCOMMON* dapcomm)
 	ce = NULL;
     else
         ce = buildconstraintstring3(dapcomm->oc.dapconstraint);
-
     {
         ocstat = dap_fetch(dapcomm,dapcomm->oc.conn,ce,OCDDS,&ocroot);
         if(ocstat != OC_NOERR) {THROWCHK(ocstat); goto fail;}
@@ -671,27 +672,30 @@ fetchconstrainedmetadata3(NCDAPCOMMON* dapcomm)
         /* Construct our parallel dds tree; including attributes*/
         ncstat = buildcdftree34(dapcomm,ocroot,OCDDS,&ddsroot);
         if(ncstat) goto fail;
+	ocroot = NULL; /* avoid duplicate reclaim */
 
 	dapcomm->cdf.ddsroot = ddsroot;
+	ddsroot = NULL; /* to avoid double reclamation */
 
         if(!FLAGSET(dapcomm->controls,NCF_UNCONSTRAINABLE)) {
-            /* fix DAP server problem by adding back any missing grid structure nodes */
-            ncstat = regrid3(ddsroot,dapcomm->cdf.fullddsroot,dapcomm->oc.dapconstraint->projections);    
+            /* fix DAP server problem by adding back any inserting needed structure nodes */
+            ncstat = restruct3(dapcomm->cdf.ddsroot,dapcomm->cdf.fullddsroot,dapcomm->oc.dapconstraint->projections);    
             if(ncstat) goto fail;
 	}
 
 #ifdef DEBUG
-fprintf(stderr,"constrained:\n%s",dumptree(ddsroot));
+fprintf(stderr,"constrained:\n%s",dumptree(dapcomm->cdf.ddsroot));
 #endif
 
         /* Combine DDS and DAS */
 	if(dapcomm->oc.ocdasroot != NULL) {
-            ncstat = dapmerge3(dapcomm,ddsroot,dapcomm->oc.ocdasroot);
+            ncstat = dapmerge3(dapcomm,dapcomm->cdf.ddsroot->ocnode,
+                               dapcomm->oc.ocdasroot);
             if(ncstat != NC_NOERR) {THROWCHK(ncstat); goto fail;}
 	}
 
         /* map the constrained DDS to the unconstrained DDS */
-        ncstat = mapnodes3(ddsroot,dapcomm->cdf.fullddsroot);
+        ncstat = mapnodes3(dapcomm->cdf.ddsroot,dapcomm->cdf.fullddsroot);
         if(ncstat) goto fail;
 
     }
@@ -713,8 +717,8 @@ suppressunusablevars3(NCDAPCOMMON* dapcomm)
     while(found) {
 	found = 0;
 	/* Walk backwards to aid removal semantics */
-	for(i=nclistlength(dapcomm->cdf.varnodes)-1;i>=0;i--) {
-	    CDFnode* var = (CDFnode*)nclistget(dapcomm->cdf.varnodes,i);
+	for(i=nclistlength(dapcomm->cdf.ddsroot->tree->varnodes)-1;i>=0;i--) {
+	    CDFnode* var = (CDFnode*)nclistget(dapcomm->cdf.ddsroot->tree->varnodes,i);
 	    /* See if this var is under an unusable sequence */
 	    nclistclear(path);
 	    collectnodepath3(var,path,WITHOUTDATASET);
@@ -731,7 +735,7 @@ fprintf(stderr,"suppressing var in unusable sequence: %s.%s\n",node->ncfullname,
 	    }
 	    if(found) break;
 	}
-        if(found) nclistremove(dapcomm->cdf.varnodes,i);
+        if(found) nclistremove(dapcomm->cdf.ddsroot->tree->varnodes,i);
     }
     nclistfree(path);
     return NC_NOERR;
@@ -746,15 +750,15 @@ NCerror
 fixzerodims3(NCDAPCOMMON* dapcomm)
 {
     int i,j;
-    for(i=0;i<nclistlength(dapcomm->cdf.varnodes);i++) {
-	CDFnode* var = (CDFnode*)nclistget(dapcomm->cdf.varnodes,i);
+    for(i=0;i<nclistlength(dapcomm->cdf.ddsroot->tree->varnodes);i++) {
+	CDFnode* var = (CDFnode*)nclistget(dapcomm->cdf.ddsroot->tree->varnodes,i);
         NClist* ncdims = var->array.dimsetplus;
 	if(nclistlength(ncdims) == 0) continue;
         for(j=0;j<nclistlength(ncdims);j++) {
 	    CDFnode* dim = (CDFnode*)nclistget(ncdims,j);
 	    if(dim->dim.declsize == 0) {
 	 	/* make node invisible */
-		var->visible = 0;
+		var->invisible = 1;
 		var->zerodim = 1;
 	    }
 	}
@@ -767,10 +771,11 @@ applyclientparamcontrols3(NCDAPCOMMON* dapcomm)
 {
     /* clear the flags */
     CLRFLAG(dapcomm->controls,NCF_CACHE);
-    CLRFLAG(dapcomm->controls,NCF_PREFETCH);
     CLRFLAG(dapcomm->controls,NCF_SHOWFETCH);
     CLRFLAG(dapcomm->controls,NCF_NC3);
     CLRFLAG(dapcomm->controls,NCF_NCDAP);
+    CLRFLAG(dapcomm->controls,NCF_PREFETCH);
+    CLRFLAG(dapcomm->controls,NCF_PREFETCH_EAGER);
 
     /* Turn on any default on flags */
     SETFLAG(dapcomm->controls,DFALT_ON_FLAGS);    
@@ -782,12 +787,16 @@ applyclientparamcontrols3(NCDAPCOMMON* dapcomm)
     else if(paramcheck34(dapcomm,"nocache",NULL))
 	CLRFLAG(dapcomm->controls,NCF_CACHE);
 
-    /* enable/disable cache prefetch */
-    if(paramcheck34(dapcomm,"prefetch",NULL))
-	SETFLAG(dapcomm->controls,NCF_PREFETCH);
-    else if(paramcheck34(dapcomm,"noprefetch",NULL))
-	CLRFLAG(dapcomm->controls,NCF_PREFETCH);
-
+    /* enable/disable cache prefetch and lazy vs eager*/
+    if(paramcheck34(dapcomm,"prefetch","eager")) {
+        SETFLAG(dapcomm->controls,NCF_PREFETCH);
+        SETFLAG(dapcomm->controls,NCF_PREFETCH_EAGER);
+    } else if(paramcheck34(dapcomm,"prefetch","lazy")
+              || paramcheck34(dapcomm,"prefetch",NULL)) {
+        SETFLAG(dapcomm->controls,NCF_PREFETCH);
+        CLRFLAG(dapcomm->controls,NCF_PREFETCH_EAGER);
+    } else if(paramcheck34(dapcomm,"noprefetch",NULL))
+        CLRFLAG(dapcomm->controls,NCF_PREFETCH);
 
     if(FLAGSET(dapcomm->controls,NCF_UNCONSTRAINABLE))
 	SETFLAG(dapcomm->controls,NCF_CACHE);
