@@ -91,6 +91,7 @@ NC_check_file_type(const char *path, int use_parallel, void *mpi_info,
    /* Get the 4-byte magic from the beginning of the file. Don't use posix
     * for parallel, use the MPI functions instead. */
 #ifdef USE_PARALLEL_MPIO
+/* Note that this assumes netcdf-4 support is enabled */
    if (use_parallel) 
    {
       MPI_File fh;
@@ -111,37 +112,59 @@ NC_check_file_type(const char *path, int use_parallel, void *mpi_info,
 	 return NC_EPARINIT;
       if((retval = MPI_File_close(&fh)) != MPI_SUCCESS)
 	 return NC_EPARINIT;
+      /* Pretend this is an HDF5 file */
+      *hdf = 5;
    } else
 #endif /* USE_PARALLEL */
    {
       FILE *fp;
       int i;
+#ifdef HAVE_SYS_STAT_H
+      struct stat st;
+#endif
 
       if(path == NULL || strlen(path)==0)
 	return NC_EINVAL;
 	
       if (!(fp = fopen(path, "r")))
 	 return errno;
+
+#ifdef HAVE_SYS_STAT_H
+      /* The file must be at least MAGIC_NUMBER_LEN in size,
+         or otherwise the following fread will exhibit unexpected
+         behavior. */
+      if(!(fstat(fileno(fp),&st) == 0))
+	return errno;
+
+      if(st.st_size < MAGIC_NUMBER_LEN)
+	return NC_ENOTNC;
+#endif
+            
+
       i = fread(magic, MAGIC_NUMBER_LEN, 1, fp);
       fclose(fp);
-      if(i != 1)
-	 return errno;
-   }
-    
-   /* Ignore the first byte for HDF */
-   if(magic[1] == 'H' && magic[2] == 'D' && magic[3] == 'F')
-      *hdf = 5;
-   else if(magic[0] == '\016' && magic[1] == '\003'
-	   && magic[2] == '\023' && magic[3] == '\001')
-      *hdf = 4;
-   else if(magic[0] == 'C' && magic[1] == 'D' && magic[2] == 'F') 
-   {
-      if(magic[3] == '\001') 
-	 *cdf = 1; /* netcdf classic version 1 */
-      else if(magic[3] == '\002') 
-	 *cdf = 2; /* netcdf classic version 2 */
-   }
+	  if(i == 0) 
+		return NC_ENOTNC;
+      if(i != 1) 
+		 return errno;
     
+      /* Ignore the first byte for HDF */
+      if(magic[1] == 'H' && magic[2] == 'D' && magic[3] == 'F')
+         *hdf = 5;
+      else if(magic[0] == '\016' && magic[1] == '\003'
+              && magic[2] == '\023' && magic[3] == '\001')
+         *hdf = 4;
+      else if(magic[0] == 'C' && magic[1] == 'D' && magic[2] == 'F') 
+      {
+         if(magic[3] == '\001') 
+            *cdf = 1; /* netcdf classic version 1 */
+         else if(magic[3] == '\002') 
+            *cdf = 2; /* netcdf classic version 2 */
+         else
+            return NC_ENOTNC;
+      } else
+            return NC_ENOTNC;
+    }       
    return NC_NOERR;
 }
 
@@ -995,9 +1018,10 @@ nc_abort(int ncid)
    NC* ncp;
    int stat = NC_check_id(ncid, &ncp);
    if(stat != NC_NOERR) return stat;
-   if(ncp->path != NULL) free(ncp->path);
-   ncp->path = NULL;   
-   return ncp->dispatch->abort(ncid);
+   stat = ncp->dispatch->abort(ncid);
+   del_from_NCList(ncp);
+   free_NC(ncp);
+   return stat;   
 }
 
 /**
@@ -1046,7 +1070,11 @@ nc_close(int ncid)
    NC* ncp;
    int stat = NC_check_id(ncid, &ncp);
    if(stat != NC_NOERR) return stat;
-   return ncp->dispatch->close(ncid);
+   stat = ncp->dispatch->close(ncid);
+   /* Remove from the nc list */
+   del_from_NCList(ncp);
+   free_NC(ncp);
+   return stat;
 }
 
 /**
@@ -1414,7 +1442,6 @@ NC_create(const char *path, int cmode, size_t initialsz,
    int model = 0; /* one of the NC_DISPATCH_XXX values */
    int isurl = 0;   /* dap or cdmremote or neither */
    int xcmode = 0; /* for implied cmode flags */
-   extern int default_create_format;
 
    /* Initialize the dispatch table. The function pointers in the
     * dispatch table will depend on how netCDF was built
@@ -1435,11 +1462,13 @@ NC_create(const char *path, int cmode, size_t initialsz,
    if(model == 0) {
       if(cmode & NC_NETCDF4 || cmode & NC_PNETCDF)
 	model = NC_DISPATCH_NC4;
+      else if(cmode & NC_CLASSIC_MODEL)
+	model = NC_DISPATCH_NC3;
    }
 
    if(model == 0) {
       /* Check default format */
-      int format = default_create_format;
+      int format = nc_get_default_format();
       switch (format) {
 #ifdef USE_NETCDF4
 	 case NC_FORMAT_NETCDF4:
@@ -1494,16 +1523,22 @@ NC_create(const char *path, int cmode, size_t initialsz,
 	 return NC_ENOTNC;
    }
 
-   if ((stat = dispatcher->create(path, cmode, initialsz, basepe, chunksizehintp,
-				   useparallel, mpi_info, dispatcher, &ncp)))
-      return stat;
+   /* Create the NC* instance and insert its dispatcher */
+   stat = new_NC(dispatcher,path,&ncp);
+   if(stat) return stat;
 
-   ncp->dispatch = dispatcher;
-   if(ncidp) 
-      *ncidp = ncp->ext_ncid;
-   if (!(ncp->path = nulldup(path)))
-      return NC_ENOMEM;
-   return NC_NOERR;
+   /* Add to list of known open files and define ext_ncid */
+   add_to_NCList(ncp);
+
+   /* Assume create will fill in remaining ncp fields */
+   if ((stat = dispatcher->create(path, cmode, initialsz, basepe, chunksizehintp,
+				   useparallel, mpi_info, dispatcher, ncp))) {
+	del_from_NCList(ncp); /* oh well */
+	free_NC(ncp);
+     } else {
+       if(ncidp)*ncidp = ncp->ext_ncid;
+     }
+   return stat;
 }
 
 /**
@@ -1535,7 +1570,6 @@ NC_open(const char *path, int cmode,
    int isurl = 0; 
    int cdfversion = 0;
    int hdfversion = 0;
-   extern int default_create_format;
 
    if(!nc_initialized) {
       stat = NC_initialize();
@@ -1558,8 +1592,8 @@ NC_open(const char *path, int cmode,
 	 } else if(cdfversion != 0) {
 	    model = NC_DISPATCH_NC3;
 	 }
-      } 
-      /* else ignore the file */
+      } else /* presumably not a netcdf file */
+	return stat;
    }
 
    /* Look to the incoming cmode for hints */
@@ -1605,14 +1639,23 @@ NC_open(const char *path, int cmode,
    else
       return  NC_ENOTNC;
 
-  havetable:
-  stat = dispatcher->open(path, cmode, basepe, chunksizehintp,
-			   useparallel, mpi_info, dispatcher, &ncp);
+havetable:
+
+   /* Create the NC* instance and insert its dispatcher */
+   stat = new_NC(dispatcher,path,&ncp);
+   if(stat) return stat;
+
+   /* Add to list of known open files */
+   add_to_NCList(ncp);
+
+   /* Assume open will fill in remaining ncp fields */
+   stat = dispatcher->open(path, cmode, basepe, chunksizehintp,
+			   useparallel, mpi_info, dispatcher, ncp);
    if(stat == NC_NOERR) {
-      ncp->dispatch = dispatcher;
-      if(ncidp) *ncidp = ncp->ext_ncid;
-      ncp->path = nulldup(path);
-      if(path == NULL) stat = NC_ENOMEM;	
+     if(ncidp) *ncidp = ncp->ext_ncid;
+   } else {
+	del_from_NCList(ncp);
+	free_NC(ncp);
    }
    return stat;
 }
