@@ -1,26 +1,27 @@
-/*
-This file is part of netcdf-4, a netCDF-like interface for HDF5, or a
-HDF5 backend for netCDF, depending on your point of view.
+/** \file \internal
+Internal netcdf-4 functions.
 
 This file contains functions internal to the netcdf4 library. None of
 the functions in this file are exposed in the exetnal API. These
 functions all relate to the manipulation of netcdf-4's in-memory
-buffer of metadata information, i.e. the linked list of NC_FILE_INFO_T
+buffer of metadata information, i.e. the linked list of NC
 structs.
 
-Copyright 2003-2005, University Corporation for Atmospheric
+Copyright 2003-2011, University Corporation for Atmospheric
 Research. See the COPYRIGHT file for copying and redistribution
 conditions.
 
-$Id: nc4internal.c,v 1.121 2010/05/26 21:43:35 dmh Exp $
 */
-
 #include "config.h"
 #include "nc4internal.h"
 #include "nc.h" /* from libsrc */
 #include "ncdispatch.h" /* from libdispatch */
 #include <utf8proc.h>
 
+#ifdef USE_PNETCDF
+#include <pnetcdf.h>
+#endif
+
 #define MEGABYTE 1048576
 
 /* These are the default chunk cache sizes for HDF5 files created or
@@ -125,7 +126,7 @@ find_var_shape_grp(NC_GRP_INFO_T *grp, int varid, int *ndims,
 	       dim in the space. */
 	    if ((dataset_ndims = H5Sget_simple_extent_ndims(spaceid)) < 0)
 	       BAIL(NC_EHDFERR);
-	    if (dataset_ndims != *ndims)
+	    if (ndims && dataset_ndims != *ndims)
 	       BAIL(NC_EHDFERR);
 	    if (!(h5dimlen = malloc(dataset_ndims * sizeof(hsize_t))))
 	       BAIL(NC_ENOMEM);
@@ -153,27 +154,22 @@ find_var_shape_grp(NC_GRP_INFO_T *grp, int varid, int *ndims,
    return retval;
 }
 
-/* Given an NC_FILE_INFO_T pointer, add the necessary stuff for a
+/* Given an NC pointer, add the necessary stuff for a
  * netcdf-4 file. */
 int
-nc4_nc4f_list_add(NC_FILE_INFO_T *nc, const char *path, int mode)
+nc4_nc4f_list_add(NC *nc, const char *path, int mode)
 {
    NC_HDF5_FILE_INFO_T *h5;
    NC_GRP_INFO_T *grp;
 
-   assert(nc && !nc->nc4_info && path);
+   assert(nc && !NC4_DATA(nc) && path);
 
-   /* The NC_FILE_INFO_T was allocated and inited by
-      ncfunc.c before this function is called. We need to malloc and
+   /* We need to malloc and
       initialize the substructure NC_HDF_FILE_INFO_T. */
-   if (!(nc->nc4_info = calloc(1, sizeof(NC_HDF5_FILE_INFO_T))))
+   if (!(h5 = calloc(1, sizeof(NC_HDF5_FILE_INFO_T))))
       return NC_ENOMEM;
-   h5 = nc->nc4_info;
-
-   /* Hang on to the filename for nc_abort. */
-   if (!(h5->path = malloc((strlen(path) + 1) * sizeof(char))))
-      return NC_ENOMEM;
-   strcpy(h5->path, path);
+   NC4_DATA_SET(nc,h5);
+   h5->controller = nc;
 
    /* Hang on to cmode, and note that we're in define mode. */
    h5->cmode = mode | NC_INDEF;
@@ -193,7 +189,7 @@ nc4_nc4f_list_add(NC_FILE_INFO_T *nc, const char *path, int mode)
 /* NC_GRP_INFO_T * */
 /* find_nc_grp(int ncid) */
 /* { */
-/*    NC_FILE_INFO_T *f; */
+/*    NC *f; */
 
 /*    for (f = nc_file; f; f = f->next) */
 /*    { */
@@ -215,19 +211,20 @@ nc4_nc4f_list_add(NC_FILE_INFO_T *nc, const char *path, int mode)
 int
 nc4_find_nc4_grp(int ncid, NC_GRP_INFO_T **grp)
 {
-   NC_FILE_INFO_T *f = nc4_find_nc_file(ncid);
+   NC_HDF5_FILE_INFO_T* h5;
+   NC *f = nc4_find_nc_file(ncid,&h5);
    if(f == NULL) return NC_EBADID;
 
    /* No netcdf-3 files allowed! */
-   if (!f->nc4_info) return NC_ENOTNC4;
-   assert(f->nc4_info->root_grp);
+   if (!h5) return NC_ENOTNC4;
+   assert(h5->root_grp);
 
    /* This function demands netcdf-4 files without strict nc3
     * rules.*/
-   if (f->nc4_info->cmode & NC_CLASSIC_MODEL) return NC_ESTRICTNC3;
+   if (h5->cmode & NC_CLASSIC_MODEL) return NC_ESTRICTNC3;
 
    /* If we can't find it, the grp id part of ncid is bad. */
-   if (!(*grp = nc4_rec_find_grp(f->nc4_info->root_grp, (ncid & GRP_ID_MASK))))
+   if (!(*grp = nc4_rec_find_grp(h5->root_grp, (ncid & GRP_ID_MASK))))
       return NC_EBADID;
    return NC_NOERR;
 }
@@ -236,43 +233,63 @@ nc4_find_nc4_grp(int ncid, NC_GRP_INFO_T **grp)
  * also set a pointer to the nc4_info struct of the related file. For
  * netcdf-3 files, *h5 will be set to NULL. */
 int
-nc4_find_grp_h5(int ncid, NC_GRP_INFO_T **grp, NC_HDF5_FILE_INFO_T **h5)
+nc4_find_grp_h5(int ncid, NC_GRP_INFO_T **grpp, NC_HDF5_FILE_INFO_T **h5p)
 {
-    NC_FILE_INFO_T *f = nc4_find_nc_file(ncid);
+    NC_HDF5_FILE_INFO_T *h5;
+    NC_GRP_INFO_T *grp;
+    NC *f = nc4_find_nc_file(ncid,&h5);
     if(f == NULL) return NC_EBADID;
-    if (f->nc4_info) {
-        assert(f->nc4_info->root_grp);
+    if (h5) {
+        assert(h5->root_grp);
         /* If we can't find it, the grp id part of ncid is bad. */
-	if (!(*grp = nc4_rec_find_grp(f->nc4_info->root_grp, (ncid & GRP_ID_MASK))))
+	if (!(grp = nc4_rec_find_grp(h5->root_grp, (ncid & GRP_ID_MASK))))
   	    return NC_EBADID;
-	*h5 = (*grp)->file->nc4_info;
-	assert(*h5);
+	h5 = (grp)->nc4_info;
+	assert(h5);
     } else {
-	*h5 = NULL;
-	*grp = NULL;
+	h5 = NULL;
+	grp = NULL;
     }
+    if(h5p) *h5p = h5;
+    if(grpp) *grpp = grp;
     return NC_NOERR;
 }
 
 int
-nc4_find_nc_grp_h5(int ncid, NC_FILE_INFO_T **nc, NC_GRP_INFO_T **grp, 
-		   NC_HDF5_FILE_INFO_T **h5)
+nc4_find_nc_grp_h5(int ncid, NC **nc, NC_GRP_INFO_T **grpp,
+		   NC_HDF5_FILE_INFO_T **h5p)
 {
-    NC_FILE_INFO_T *f = nc4_find_nc_file(ncid);
+    NC_GRP_INFO_T *grp;
+    NC_HDF5_FILE_INFO_T* h5;
+    NC *f = nc4_find_nc_file(ncid,&h5);
+
     if(f == NULL) return NC_EBADID;
     *nc = f;
-    if (f->nc4_info) {
-	assert(f->nc4_info->root_grp);
+
+#if 0 /* I do not understand this code at all */
+#ifdef USE_PNETCDF
+    if (h5->pnetcdf_file) {
+      *h5p = NULL;
+      *grpp = NULL;
+      return NC_NOERR;
+    }
+#endif
+#endif
+    
+    if (h5) {
+	assert(h5->root_grp);
 	/* If we can't find it, the grp id part of ncid is bad. */
-	if (!(*grp = nc4_rec_find_grp(f->nc4_info->root_grp, (ncid & GRP_ID_MASK))))
+	if (!(grp = nc4_rec_find_grp(h5->root_grp, (ncid & GRP_ID_MASK))))
 	       return NC_EBADID;
 
-	*h5 = (*grp)->file->nc4_info;
-	assert(*h5);
+	h5 = (grp)->nc4_info;
+	assert(h5);
     } else {
-	*h5 = NULL;
-	*grp = NULL;
+	h5 = NULL;
+	grp = NULL;
     }
+    if(h5p) *h5p = h5;
+    if(grpp) *grpp = grp;
     return NC_NOERR;
 }
 
@@ -301,13 +318,20 @@ nc4_rec_find_grp(NC_GRP_INFO_T *start_grp, int target_nc_grpid)
 /* Given an ncid and varid, get pointers to the group and var
  * metadata. */
 int
-nc4_find_g_var_nc(NC_FILE_INFO_T *nc, int ncid, int varid, 
+nc4_find_g_var_nc(NC *nc, int ncid, int varid, 
 		  NC_GRP_INFO_T **grp, NC_VAR_INFO_T **var)
 {
-   /* Find the group info. */
-   assert(grp && var && nc && nc->nc4_info && nc->nc4_info->root_grp);
-   *grp = nc4_rec_find_grp(nc->nc4_info->root_grp, (ncid & GRP_ID_MASK));
+   NC_HDF5_FILE_INFO_T* h5 = NC4_DATA(nc);
 
+   /* Find the group info. */
+   assert(grp && var && h5 && h5->root_grp);
+   *grp = nc4_rec_find_grp(h5->root_grp, (ncid & GRP_ID_MASK));
+
+   /* It is possible for *grp to be NULL. If it is,
+      return an error. */
+   if(*grp == NULL) 
+     return NC_ENOTVAR;
+   
    /* Find the var info. */
    for ((*var) = (*grp)->var; (*var); (*var) = (*var)->next)
      if ((*var)->varid == varid)
@@ -454,8 +478,8 @@ nc4_find_dim_len(NC_GRP_INFO_T *grp, int dimid, size_t **len)
 {
    NC_GRP_INFO_T *g;
    NC_VAR_INFO_T *var;
-   int d, ndims, dimids[NC_MAX_DIMS];
-   size_t dimlen[NC_MAX_DIMS];
+   int d, ndims, *dimids = NULL;
+   size_t *dimlen = NULL;
    int retval; 
    
    assert(grp && len);
@@ -471,24 +495,34 @@ nc4_find_dim_len(NC_GRP_INFO_T *grp, int dimid, size_t **len)
     * dimension, and remember the max length. */
    for (var = grp->var; var; var = var->next)
    {
-      /* Find dimensions of this var. */
-      if ((retval = find_var_shape_grp(grp, var->varid, &ndims, 
-				       dimids, dimlen)))
-	 return retval;
-
-      /* Check for any dimension that matches dimid. If found, check
-       * if its length is longer than *lenp. */
-      for (d = 0; d < ndims; d++)
-      {
+     if(var->ndims > 0) {
+       dimids = (int*)malloc(sizeof(int)*var->ndims);
+       dimlen = (size_t*)malloc(sizeof(size_t)*var->ndims);
+     }
+     
+     /* Find dimensions of this var. */
+     if ((retval = find_var_shape_grp(grp, var->varid, &ndims, 
+				      dimids, dimlen))) {
+       free(dimids);
+       free(dimlen);
+       return retval;
+     }
+     
+     /* Check for any dimension that matches dimid. If found, check
+      * if its length is longer than *lenp. */
+     for (d = 0; d < ndims; d++)
+       {
 	 if (dimids[d] == dimid)
-	 {
-	    /* Remember the max length in *lenp. */
-	    **len = dimlen[d] > **len ? dimlen[d] : **len;
-	    break;
-	 }
-      }
+	   {
+	     /* Remember the max length in *lenp. */
+	     **len = dimlen[d] > **len ? dimlen[d] : **len;
+	     break;
+	   }
+       }
+     free(dimids); dimids = NULL;
+     free(dimlen); dimlen = NULL;
    }
-
+   
    return NC_NOERR;
 }
 
@@ -579,72 +613,21 @@ nc4_find_nc_att(int ncid, int varid, const char *name, int attnum,
    return NC_ENOTATT;
 }
 
-void
-nc4_file_list_free(void)
-{
-    free_NCList();
-}
-
-
-int
-NC4_new_nc(NC** ncpp)
-{
-    NC_FILE_INFO_T** ncp;
-    /* Allocate memory for this info. */
-    if (!(ncp = calloc(1, sizeof(NC_FILE_INFO_T)))) 
-       return NC_ENOMEM;
-    if(ncpp) *ncpp = (NC*)ncp;
-    return NC_NOERR;
-}
-
-int
-nc4_file_list_add(NC_FILE_INFO_T** ncp, NC_Dispatch* dispatch)
-{
-    NC_FILE_INFO_T *nc;
-    int status = NC_NOERR;
-
-    /* Allocate memory for this info; use the dispatcher to do this */
-    status = dispatch->new_nc((NC**)&nc);
-    if(status) return status;
-
-    /* Add this file to the list. */
-    if ((status = add_to_NCList((NC *)nc)))
-    {
-       if(nc && nc->ext_ncid > 0) 
-       {
-	  del_from_NCList((NC *)nc);
-	  free(nc);
-       }
-       return status;
-    }
-    
-    /* Return a pointer to the new struct. */
-    if(ncp) 
-       *ncp = nc;
-
-    return NC_NOERR;
-}
-
-/* Remove a NC_FILE_INFO_T from the linked list. This will nc_free the
-   memory too. */
-void
-nc4_file_list_del(NC_FILE_INFO_T *nc)
-{
-   /* Remove file from master list. */
-   del_from_NCList((NC *)nc);
-   free(nc);
-}
-
 
 /* Given an id, walk the list and find the appropriate
-   NC_FILE_INFO_T. */
-NC_FILE_INFO_T*
-nc4_find_nc_file(int ext_ncid)
+   NC. */
+NC*
+nc4_find_nc_file(int ext_ncid, NC_HDF5_FILE_INFO_T** h5p)
 {
-   return (NC_FILE_INFO_T*)find_in_NCList(ext_ncid);
+   NC* nc;
+   int stat;
+   stat = NC_check_id(ext_ncid,&nc);
+   if(stat != NC_NOERR)
+	nc = NULL;
+   if(h5p) *h5p = (NC_HDF5_FILE_INFO_T*)nc->dispatchdata;
+   return nc;
 }
 
-
 /* Add to the end of a var list. Return a pointer to the newly
  * added var. */
 int
@@ -693,6 +676,7 @@ nc4_dim_list_add(NC_DIM_INFO_T **list)
 }
 
 /* Add to the beginning of a dim list. */
+#ifdef EXTERN_UNUSED
 int
 nc4_dim_list_add2(NC_DIM_INFO_T **list, NC_DIM_INFO_T **new_dim)
 {
@@ -709,6 +693,7 @@ nc4_dim_list_add2(NC_DIM_INFO_T **list, NC_DIM_INFO_T **new_dim)
       *new_dim = dim;
    return NC_NOERR;
 }
+#endif
 
 /* Add to the end of an att list. */
 int
@@ -737,7 +722,7 @@ nc4_att_list_add(NC_ATT_INFO_T **list)
  * it's reserverd for the root group. */
 int
 nc4_grp_list_add(NC_GRP_INFO_T **list, int new_nc_grpid, 
-		 NC_GRP_INFO_T *parent_grp, NC_FILE_INFO_T *nc, 
+		 NC_GRP_INFO_T *parent_grp, NC *nc, 
 		 char *name, NC_GRP_INFO_T **grp)
 {
    NC_GRP_INFO_T *g;
@@ -771,7 +756,7 @@ nc4_grp_list_add(NC_GRP_INFO_T **list, int new_nc_grpid,
    if (!((*grp)->name = malloc((strlen(name) + 1) * sizeof(char))))
       return NC_ENOMEM;
    strcpy((*grp)->name, name);
-   (*grp)->file = nc;
+   (*grp)->nc4_info = NC4_DATA(nc);
 
    return NC_NOERR;
 }
@@ -925,14 +910,18 @@ nc4_enum_member_add(NC_ENUM_MEMBER_INFO_T **list, size_t size,
 }
 
 /* Delete a var from a var list, and free the memory. */
-static int
-var_list_del(NC_VAR_INFO_T **list, NC_VAR_INFO_T *var)
+int
+nc4_var_list_del(NC_VAR_INFO_T **list, NC_VAR_INFO_T *var)
 {
    NC_ATT_INFO_T *a, *att;
    int ret;
 
+   if(var == NULL) {
+     return NC_NOERR;
+   }
+
    /* First delete all the attributes attached to this var. */
-   att = (*list)->att;
+   att = var->att;
    while (att)
    {
       a = att->next;
@@ -942,16 +931,20 @@ var_list_del(NC_VAR_INFO_T **list, NC_VAR_INFO_T *var)
    }
 
    /* Free some things that may be allocated. */
-   if (var->chunksizes)
-      free(var->chunksizes);
+   if (var->chunksizes) 
+     {free(var->chunksizes);var->chunksizes = NULL;}
+   
    if (var->hdf5_name)
-      free(var->hdf5_name);
+     {free(var->hdf5_name); var->hdf5_name = NULL;}
+ 
    if (var->name)
-      free(var->name);
+     {free(var->name); var->name = NULL;}
+
    if (var->dimids)
-      free(var->dimids);
+     {free(var->dimids); var->dimids = NULL;}
+   
    if (var->dim)
-      free(var->dim);
+     {free(var->dim); var->dim = NULL;}
 
    /* Remove the var from the linked list. */
    if(*list == var)
@@ -968,17 +961,20 @@ var_list_del(NC_VAR_INFO_T **list, NC_VAR_INFO_T *var)
    {
       if (var->hdf_datasetid)
       {
-	 if (var->type_info->class == NC_VLEN)
-	    nc_free_vlen((nc_vlen_t *)var->fill_value);
-	 else if (var->type_info->nc_typeid == NC_STRING)
-	    free(*(char **)var->fill_value);
+         if (var->type_info)
+         {
+            if (var->type_info->class == NC_VLEN)
+               nc_free_vlen((nc_vlen_t *)var->fill_value);
+            else if (var->type_info->nc_typeid == NC_STRING)
+               free(*(char **)var->fill_value);
+         }
       }
       free(var->fill_value);
+      var->fill_value = NULL;
    }
 
    /* For atomic types we have allocated space for type information. */
-/*   if (var->hdf_datasetid && var->xtype <= NC_STRING)*/
-   if (var->xtype <= NC_STRING)
+   if (var->type_info && var->xtype <= NC_STRING)
    {
       if (var->type_info->native_typeid)
 	 if ((H5Tclose(var->type_info->native_typeid)) < 0)
@@ -992,10 +988,13 @@ var_list_del(NC_VAR_INFO_T **list, NC_VAR_INFO_T *var)
 	    return NC_EHDFERR;
 
       /* Free the name. */
-      if (var->type_info->name)
+      if (var->type_info->name) {
 	 free(var->type_info->name);
-
+	 var->type_info->name = NULL;
+      }
+      
       free(var->type_info);
+      var->type_info = NULL;
    }
    
    /* Delete any HDF5 dimscale objid information. */
@@ -1008,6 +1007,7 @@ var_list_del(NC_VAR_INFO_T **list, NC_VAR_INFO_T *var)
 
    /* Delete the var. */
    free(var);
+   var = NULL;
 
    return NC_NOERR;
 }
@@ -1037,7 +1037,7 @@ field_list_del(NC_FIELD_INFO_T **list, NC_FIELD_INFO_T *field)
 }
 
 /* Delete a type from a type list, and nc_free the memory. */
-int
+static int
 type_list_del(NC_TYPE_INFO_T **list, NC_TYPE_INFO_T *type)
 {
    NC_FIELD_INFO_T *field, *f;
@@ -1183,7 +1183,7 @@ nc4_rec_grp_del(NC_GRP_INFO_T **list, NC_GRP_INFO_T *grp)
 	  H5Dclose(var->hdf_datasetid) < 0)
 	 return NC_EHDFERR;
       v = var->next;
-      if ((retval = var_list_del(&grp->var, var)))
+      if ((retval = nc4_var_list_del(&grp->var, var)))
 	 return retval;
       var = v;
    }
@@ -1348,7 +1348,7 @@ rec_print_metadata(NC_GRP_INFO_T *grp, int *tab_count)
    NC_TYPE_INFO_T *type;
    NC_FIELD_INFO_T *field;
    char tabs[MAX_NESTS] = "";
-   char dims_string[NC_MAX_DIMS*4];
+   char *dims_string = NULL;
    char temp_string[10];
    int t, retval, d;
 
@@ -1377,18 +1377,20 @@ rec_print_metadata(NC_GRP_INFO_T *grp, int *tab_count)
       ;
    for( ; var; var = var->prev)
    {
-      strcpy(dims_string, "");
-      for (d = 0; d < var->ndims; d++)
-      {
+     dims_string = (char*)malloc(sizeof(char)*(var->ndims*4));
+     strcpy(dims_string, "");
+     for (d = 0; d < var->ndims; d++)
+       {
 	 sprintf(temp_string, " %d", var->dimids[d]);
 	 strcat(dims_string, temp_string);
-      }
+       }
       LOG((2, "%s VARIABLE - varid: %d name: %s type: %d ndims: %d dimscale: %d dimids:%s",
 	   tabs, var->varid, var->name, var->xtype, var->ndims, var->dimscale, 
 	   dims_string));
       for(att = var->att; att; att = att->next)
 	 LOG((2, "%s VAR ATTRIBUTE - attnum: %d name: %s type: %d len: %d",
 	      tabs, att->attnum, att->name, att->xtype, att->len));
+      free(dims_string); dims_string = NULL;
    }
    
    for (type = grp->type; type; type = type->next)
@@ -1435,9 +1437,9 @@ rec_print_metadata(NC_GRP_INFO_T *grp, int *tab_count)
  * that netCDF is working! Nonetheless, this function will print
  * nothing if logging is not set to at least two. */
 int
-log_metadata_nc(NC_FILE_INFO_T *nc)
+log_metadata_nc(NC *nc)
 {
-   NC_HDF5_FILE_INFO_T *h5 = nc->nc4_info;
+   NC_HDF5_FILE_INFO_T *h5 = NC4_DATA(nc);
    int tab_count = 0;
 
    LOG((2, "*** NetCDF-4 Internal Metadata: int_ncid 0x%x ext_ncid 0x%x", 
@@ -1448,7 +1450,7 @@ log_metadata_nc(NC_FILE_INFO_T *nc)
       return NC_NOERR;
    }
    LOG((2, "FILE - hdfid: 0x%x path: %s cmode: 0x%x parallel: %d redef: %d "
-	"fill_mode: %d no_write: %d next_nc_grpid: %d",	h5->hdfid, h5->path, 
+	"fill_mode: %d no_write: %d next_nc_grpid: %d",	h5->hdfid, nc->path, 
 	h5->cmode, h5->parallel, h5->redef, h5->fill_mode, h5->no_write, 
 	h5->next_nc_grpid));
    return rec_print_metadata(h5->root_grp, &tab_count);
@@ -1462,11 +1464,11 @@ NC4_show_metadata(int ncid)
 {
    int retval = NC_NOERR;
 #ifdef LOGGING
-   NC_FILE_INFO_T *nc;
+   NC *nc;
    int old_log_level = nc_log_level;
    
    /* Find file metadata. */
-   if (!(nc = nc4_find_nc_file(ncid)))
+   if (!(nc = nc4_find_nc_file(ncid,NULL)))
       return NC_EBADID;
 
    /* Log level must be 2 to see metadata. */
