@@ -8,6 +8,7 @@
  * We potentially include <stdarg.h> before <stdio.h> in order to obtain a
  * definition for va_list from the GNU C compiler.
  */
+
 #include <config.h>
 #include <stdarg.h>
 #include <stdio.h>
@@ -103,7 +104,6 @@ init_epsilons(void)
 
 
 static char* has_c_format_att(int ncid, int varid);
-static idnode_t* newidnode(void);
 
 int float_precision_specified = 0; /* -p option specified float precision */
 int double_precision_specified = 0; /* -p option specified double precision */
@@ -188,8 +188,6 @@ safebuf_t *
 sbuf_new() {
     size_t len = SAFEBUF_INIT_LEN;
     safebuf_t *sb;
-    if (len == 0)
-	return 0;
     sb = (safebuf_t *) emalloc(sizeof(safebuf_t));
     sb->buf = (char *)emalloc(len + sizeof(int));
     sb->len = len;
@@ -419,68 +417,6 @@ get_fmt(
     return get_default_fmt(typeid);
 }
 
-static idnode_t*
-newidnode(void)
-{
-    idnode_t *newvp = (idnode_t*) emalloc(sizeof(idnode_t));
-    return newvp;
-}
-
-
-/*
- * Get a new, empty variable list.
- */
-idnode_t*
-newidlist(void)
-{
-    idnode_t *vp = newidnode();
-
-    vp -> next = 0;
-    vp -> id = -1;		/* bad id */
-
-    return vp;
-}
-
-
-void
-idadd(idnode_t* vlist, int varid)
-{
-    idnode_t *newvp = newidnode();
-    
-    newvp -> next = vlist -> next;
-    newvp -> id = varid;
-    vlist -> next = newvp;
-}
-
-
-/* 
- * return true if id is member of list that vlist points to.
- */
-boolean
-idmember(const idnode_t* idlist, int id)
-{
-    idnode_t *vp = idlist -> next;
-
-    for (; vp ; vp = vp->next)
-      if (vp->id == id)
-	return true;
-    return false;    
-}
-
-/* 
- * return true if group identified by grpid is member of group
- * list specified on command line by -g.
- */
-boolean
-group_wanted(int grpid)
-{
-    /* If -g not specified, all groups are wanted */
-    if(formatting_specs.nlgrps == 0)
-	return true;
-    /* if -g specified, look for match in group id list */
-    return idmember(formatting_specs.grpids, grpid);
-}
-
 /* Return primitive type name */
 static const char *
 prim_type_name(nc_type type)
@@ -594,25 +530,25 @@ get_typeinfo ( int typeid ) {
 /* } */
 
 
-boolean 
+bool_t 
 ncbyte_val_equals(const nctype_t *this, 
 		  const void *v1p, const void *v2p) {
     return ( *(signed char* )v1p == *(signed char* )v2p);
 }
 
-boolean 
+bool_t 
 ncchar_val_equals(const nctype_t *this, 
 		  const void *v1p, const void *v2p) {
     return ( *(char* )v1p == *(char* )v2p);
 }
 
-boolean 
+bool_t 
 ncshort_val_equals(const nctype_t *this, 
 		   const void *v1p, const void *v2p) {
     return ( *(short* )v1p == *(short* )v2p);
 }
 
-boolean 
+bool_t 
 ncint_val_equals(const nctype_t *this, 
 		 const void *v1p, const void *v2p) {
     return ( *(int* )v1p == *(int* )v2p);
@@ -625,7 +561,7 @@ ncint_val_equals(const nctype_t *this,
  * except use floating epsilon to compare very close vals as equal
  * and handle IEEE NaNs and infinities.
  */
-boolean 
+bool_t 
 ncfloat_val_equals(const nctype_t *this, 
 		   const void *v1p, const void *v2p) {
     float v1 = *(float* )v1p;
@@ -646,7 +582,7 @@ ncfloat_val_equals(const nctype_t *this,
  * except use floating epsilon to compare very close vals as equal
  * and handle IEEE NaNs and infinities.
  */
-boolean 
+bool_t 
 ncdouble_val_equals(const nctype_t *this, 
 		    const void *v1p, const void *v2p) {
     double v1 = *(double* )v1p;
@@ -663,43 +599,43 @@ ncdouble_val_equals(const nctype_t *this,
 }
 
 #ifdef USE_NETCDF4
-boolean 
+bool_t 
 ncubyte_val_equals(const nctype_t *this, 
 		   const void *v1p, const void *v2p) {
     return ( *(unsigned char* )v1p == *(unsigned char* )v2p);
 }
 
-boolean 
+bool_t 
 ncushort_val_equals(const nctype_t *this, 
 		    const void *v1p, const void *v2p) {
     return ( *(unsigned short* )v1p == *(unsigned short* )v2p);
 }
 
-boolean 
+bool_t 
 ncuint_val_equals(const nctype_t *this, 
 		  const void *v1p, const void *v2p) {
     return ( *(unsigned int* )v1p == *(unsigned int* )v2p);
 }
 
-boolean 
+bool_t 
 ncint64_val_equals(const nctype_t *this, 
 		   const void *v1p, const void *v2p) {
     return ( *(long long* )v1p == *(long long* )v2p);
 }
 
-boolean 
+bool_t 
 ncuint64_val_equals(const nctype_t *this, 
 		    const void *v1p, const void *v2p) {
     return ( *(unsigned long long* )v1p == *(unsigned long long* )v2p);
 }
 
-boolean
+bool_t
 ncstring_val_equals(const nctype_t *this, 
 		    const void *v1p, const void *v2p) {
     return (strcmp(*((char **)v1p), *((char **)v2p)) == 0);
 }
 
-boolean
+bool_t
 ncopaque_val_equals(const nctype_t *this, 
 		    const void *v1p, const void *v2p) {
     size_t nbytes = this->size;
@@ -713,7 +649,7 @@ ncopaque_val_equals(const nctype_t *this,
     return true;
 }
 
-boolean
+bool_t
 ncvlen_val_equals(const nctype_t *this, 
 		  const void *v1p, const void *v2p) {
     size_t v1len = ((nc_vlen_t *)v1p)->len;
@@ -741,7 +677,7 @@ ncvlen_val_equals(const nctype_t *this,
 
 /* Determine if two compound values are equal, by testing eqaulity of
  * each member field. */
-boolean
+bool_t
 nccomp_val_equals(const nctype_t *this, 
 		  const void *v1p, const void *v2p) {
     int nfields = this->nfields;
@@ -933,8 +869,6 @@ ncuint64_typ_tostring(const nctype_t *typ, safebuf_t *sfbf, const void *valp) {
 int ncstring_typ_tostring(const nctype_t *typ, safebuf_t *sfbf, const void *valp) {
     size_t slen;
     char *sout;
-    int res;
-    int iel;
     const char *cp;
     char *sp;
     unsigned char uc;
@@ -993,7 +927,7 @@ int ncstring_typ_tostring(const nctype_t *typ, safebuf_t *sfbf, const void *valp
 	}
     }
     *sp++ = '"' ;
-    *sp++ = '\0' ;
+    *sp = '\0' ;
     sbuf_cpy(sfbf, sout);
     free(sout);
     return sbuf_len(sfbf);
@@ -1147,8 +1081,9 @@ chars_tostring(
 	}
     }
     *cp++ = '"';
-    *cp++ = '\0';
+    *cp = '\0';
     sbuf_cpy(sbuf, sout);
+    free(sout);
     return sbuf_len(sbuf);
 }
 
@@ -1524,85 +1459,6 @@ set_tostring_func(ncvar_t *varp) {
     return;
 }
 
-#ifdef USE_NETCDF4
-/* Returns 1 if string s1 ends with string s2, 0 otherwise. */
-static int
-strendswith(const char *s1, const char *s2) {
-    size_t m1 = strlen(s1);
-    size_t m2 = strlen(s2);
-    if (m1 < m2)
-	return 0;
-    return (strcmp(s1 + (m1 - m2), s2) == 0);
-}
-#endif
-
-#define NC_GRP_DELIM '/'
-
-/* Get varid of variable with name using nested group syntax
- * "gp1/gp2/var" or "/gp1/gp2/var".  In the former case, grpname of
- * grp corresponding to grpid must end in "gp1/gp2".  In the latter
- * case, grpname for grpid must be exactly "/gp1/gp2".  If variable
- * named "var" is not in group grpid, returns NC_ENOTVAR, else sets
- * varid and returns NC_NOERR.  */
-int 
-nc_inq_gvarid(int grpid, const char *varname, int *varidp) {
-    /* if varname has no "/" chars, then
-          return varidp from nc_inq_varid(grpid, varname, varidp)
-       if varname begins with "/"
-          
-       else
-          get groupname corresponding to grpid
-          get vargroup = substring of varname up to last "/"
-          get relname = substring of varname after last "/"
-          if (varname starts with "/" and groupname == vargroup) ||
-             (groupname ends with vargroup)
-             return nc_inq_varid(grpid, relname, varidp)
-          else
-             return NC_ENOTVAR
-    */
-    
-#ifdef USE_NETCDF4
-    char *vargroup;
-    char *relname;
-    char *groupname;
-    int status;
-    if (varname[0] == '\0')
-	return NC_ENOTVAR;
-    vargroup = strdup(varname);
-    if (vargroup == NULL) 
-	return NC_ENOMEM;
-    relname = strrchr(vargroup, NC_GRP_DELIM);
-    if (relname != NULL) {	/* name has a "/" in it */
-	size_t len;		/* length of full group name for grpid */
-	*relname++ = '\0';	/* split vargroup string in two,
-				 * vargroup and relname */
-	if ( (status = nc_inq_grpname_full(grpid, &len, NULL)) != NC_NOERR ) {
-	    free(vargroup);
-	    return status;
-	}
-	groupname = (char *)emalloc(len + 1);
-	if ( (status = nc_inq_grpname_full(grpid, &len, groupname)) == NC_NOERR ) {
-	    if(varname[0] == NC_GRP_DELIM) {
-		if( strcmp(groupname, vargroup) == 0)
-		    status = nc_inq_varid(grpid, relname, varidp);
-		else
-		    status = NC_ENOTVAR;
-	    } else {
-		if(strendswith(groupname, vargroup))
-		    status = nc_inq_varid(grpid, relname, varidp);
-		else
-		    status = NC_ENOTVAR;
-	    }
-	}
-	free(vargroup);
-	free(groupname);
-	return status;
-    }
-    free(vargroup);
-#endif	/* USE_NETCDF4 */
-    return nc_inq_varid(grpid, varname, varidp);
-}
-
 
 /* Initialize typelist with primitive types.  For netCDF-3 only need primitive 
    types. */ 
@@ -1887,7 +1743,7 @@ get_type_name(int ncid, nc_type type, char *name)
 {
 #ifdef USE_NETCDF4
     if (is_user_defined_type(type)) {
-	nc_inq_user_type(ncid, type, name, NULL, NULL, NULL, NULL);
+	NC_CHECK(nc_inq_user_type(ncid, type, name, NULL, NULL, NULL, NULL));
     } else {
 	strncpy(name, prim_type_name(type), NC_MAX_NAME + 1);
     }
@@ -1909,7 +1765,7 @@ void
 print_type_name(int locid, int typeid) {
     char *ename;
 #ifdef USE_NETCDF4
-    char name[NC_MAX_NAME];
+    char name[NC_MAX_NAME+1];
     int type_inherited = 0;
     int curlocid;		/* group we are searching in */
     int parent_groupid = locid;
@@ -1923,6 +1779,7 @@ print_type_name(int locid, int typeid) {
     if(is_user_defined_type(typeid)) {
 	/* determine if type is inherited, that is if defined in this
 	 * group or any ancestor group */
+	name[NC_MAX_NAME] = '\0';
 	strncpy(name,nctypes[typeid]->name,NC_MAX_NAME);
 	do {
 	    curlocid = parent_groupid;
@@ -1956,3 +1813,94 @@ print_type_name(int locid, int typeid) {
     fputs(ename, stdout);
     free(ename);
 }
+
+/* Allocate and initialize table of unlimited dimensions for ncid, for
+ * use by is_unlim_dim() function.  If ncid is a subgroup of a netCDF
+ * dataset, the table will still be initialized for the whole dataset
+ * in which the subgroup resides. */
+static int 
+init_is_unlim(int ncid, int **is_unlim_p)
+{
+    int num_grps;	 /* total number of groups */
+    int num_dims = 0;    /* total number of dimensions in all groups */
+    int num_undims = 0;  /* total number of unlimited dimensions in all groups */
+    int *grpids = NULL;	 /* temporary list of all grpids */
+    int igrp;
+    int grpid;
+
+#ifdef USE_NETCDF4
+    /* if ncid is not root group, find its ancestor root group id */
+    int status = nc_inq_grp_parent(ncid, &grpid);
+    while(status == NC_NOERR && grpid != ncid) {
+	ncid = grpid;
+	status = nc_inq_grp_parent(ncid, &grpid);
+    }
+    if (status != NC_ENOGRP)
+	return NC_EBADGRPID;
+    /* Now ncid is root group.  Get total number of groups and their ids */
+    NC_CHECK( nc_inq_grps_full(ncid, &num_grps, NULL) );
+    grpids = emalloc((num_grps + 1) * sizeof(int));
+    NC_CHECK( nc_inq_grps_full(ncid, &num_grps, grpids) );
+#define DONT_INCLUDE_PARENTS 0
+    /* Get all dimensions in groups and info about which ones are unlimited */
+    for(igrp = 0; igrp < num_grps; igrp++) {
+	int ndims;
+	grpid = grpids[igrp];
+	NC_CHECK( nc_inq_dimids(grpid, &ndims, NULL, DONT_INCLUDE_PARENTS) );
+	num_dims += ndims;
+    }
+    *is_unlim_p = emalloc((num_dims + 1) * sizeof(int));
+    for(igrp = 0; igrp < num_grps; igrp++) {
+	int ndims, idim, *dimids, nundims;
+	grpid = grpids[igrp];
+	NC_CHECK( nc_inq_dimids(grpid, &ndims, NULL, DONT_INCLUDE_PARENTS) );
+	dimids = emalloc((ndims + 1) * sizeof(int));
+	NC_CHECK( nc_inq_dimids(grpid, &ndims, dimids, DONT_INCLUDE_PARENTS) );
+	/* mark all dims in this group as fixed-size */
+	for(idim = 0; idim < ndims; idim++) {
+	    (*is_unlim_p)[dimids[idim]] = 0;
+	}
+	NC_CHECK( nc_inq_unlimdims(grpid, &nundims, dimids) );
+	assert(nundims <= ndims);
+	/* mark the subset of dims in this group that are unlimited */
+	for(idim = 0; idim < nundims; idim++) {
+	    (*is_unlim_p)[dimids[idim]] = 1;
+	    num_undims++;
+	}
+	if(dimids)
+	    free(dimids);
+    }
+    free(grpids);
+#endif	/*  USE_NETCDF4 */
+    return NC_NOERR;
+}
+
+/* TODO: make list of these arrays for multiple open datasets, such as
+ * the idnode_t lists above.  For now, we just have one of these, for
+ * the unique input dataset for this invocation of ncdump. */
+
+#define UNLIM_NOT_INITIALIZED (-1)
+
+/* Is dimid the dimension ID of an unlimited dimension?  */
+bool_t
+is_unlim_dim(int ncid, int dimid) {
+    bool_t result;		/* 0 if fixed, 1 if unlimited size */
+    static int for_ncid = UNLIM_NOT_INITIALIZED; /* ensure only ever called for one ncid */
+#ifdef USE_NETCDF4
+    static int *is_unlim = NULL; /* gets allocated by init_is_unlim() */
+    if(for_ncid == UNLIM_NOT_INITIALIZED) {
+	NC_CHECK( init_is_unlim(ncid, &is_unlim) );
+	for_ncid = ncid;
+    }
+    result = is_unlim[dimid];	/* 0 if fixed, 1 if unlimited size */
+#else
+    static int unlimdimid;
+    if(for_ncid == UNLIM_NOT_INITIALIZED) {
+	NC_CHECK( nc_inq_unlimdim(ncid, &unlimdimid) );
+	for_ncid = ncid;
+    }
+    result = (dimid == unlimdimid) ;
+#endif 	 /* USE_NETCDF4 */
+    return result;
+}
+
