@@ -11,7 +11,7 @@
 #ifdef HAVE_GETOPT_H
 #include <getopt.h>
 #endif
-#ifndef _WIN32
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 #include <string.h>
@@ -20,13 +20,21 @@
 #include "chunkspec.h"
 #include "utils.h"
 #include "dimmap.h"
+#include "nccomps.h"
+
+#ifdef _MSC_VER
+#include "XGetopt.h"
+#define snprintf _snprintf
+int opterr;
+int optind;
+#endif
 
 /* default bytes of memory we are willing to allocate for variable
  * values during copy */
 #define COPY_BUFFER_SIZE (5000000)
 #define COPY_CHUNKCACHE_PREEMPTION (1.0f) /* for copying, can eject fully read chunks */
 #define SAME_AS_INPUT (-1)	/* default, if kind not specified */
-#define CHUNK_THRESHOLD (1024)	/* variables with fewer bytes don't get chunked */
+#define CHUNK_THRESHOLD (8192)	/* variables with fewer bytes don't get chunked */
 
 #ifndef USE_NETCDF4
 #define NC_CLASSIC_MODEL 0x0100 /* Enforce classic model if netCDF-4 not available. */
@@ -42,10 +50,23 @@ static char* option_chunkspec = 0;   /* default, no chunk specification */
 static size_t option_copy_buffer_size = COPY_BUFFER_SIZE;
 static size_t option_chunk_cache_size = CHUNK_CACHE_SIZE; /* default from config.h */
 static size_t option_chunk_cache_nelems = CHUNK_CACHE_NELEMS; /* default from config.h */
-static int option_compute_chunkcaches = 0; /* default, don't try still flaky estimate of
-					    * chunk cache for each variable */
 static int option_read_diskless = 0; /* default, don't read input into memory on open */
 static int option_write_diskless = 0; /* default, don't write output to diskless file */
+static int option_min_chunk_bytes = CHUNK_THRESHOLD; /* default, don't chunk variable if prod of
+						      * chunksizes of its dimensions is smaller
+						      * than this */
+static int option_nlgrps = 0;		    /* Number of groups specified with -g
+					     * option on command line */
+static char** option_lgrps = 0;		    /* list of group names specified with -g
+					     * option on command line */
+static idnode_t* option_grpids = 0; /* list of grpids matching list specified with -g option */
+static bool_t option_grpstruct = false; /* if -g set, copy structure for non-selected groups */
+static int option_nlvars = 0; /* Number of variables specified with -v * option on command line */
+static char** option_lvars = 0;         /* list of variable names specified with -v
+                                         * option on command line */
+static bool_t option_varstruct = false;   /* if -v set, copy structure for non-selected vars */
+static int option_compute_chunkcaches = 0; /* default, don't try still flaky estimate of
+					    * chunk cache for each variable */
 
 /* get group id in output corresponding to group igrp in input,
  * given parent group id (or root group id) parid in output. */
@@ -72,6 +93,15 @@ get_grpid(int igrp, int parid, int *ogrpp) {
     return stat;
 }
 
+/* Return size in bytes of a variable value */
+static size_t
+val_size(int grpid, int varid) {
+    nc_type vartype;
+    size_t value_size;
+    NC_CHECK(nc_inq_vartype(grpid, varid, &vartype));
+    NC_CHECK(nc_inq_type(grpid, vartype, NULL, &value_size));
+    return value_size;
+}
 
 #ifdef USE_NETCDF4
 /* Get parent id needed to define a new group from its full name in an
@@ -79,15 +109,14 @@ get_grpid(int igrp, int parid, int *ogrpp) {
  * already defined.  */
 static int
 nc_inq_parid(int ncid, const char *fullname, int *locidp) {
-    int stat = NC_NOERR;
     char *parent = strdup(fullname);
     char *slash = "/";		/* groupname separator */
     char *last_slash;
     if(parent == NULL) {
-	NC_CHECK(NC_ENOMEM);
-    } else
-	last_slash = strrchr(parent, '/');
-    if(last_slash == parent) {	/* parent is root */
+	return NC_ENOMEM;	/* exits */
+    }
+    last_slash = strrchr(parent, '/');
+    if(last_slash == parent || last_slash == NULL) {	/* parent is root */
 	free(parent);
 	parent = strdup(slash);
     } else {
@@ -95,7 +124,7 @@ nc_inq_parid(int ncid, const char *fullname, int *locidp) {
     }
     NC_CHECK(nc_inq_grp_full_ncid(ncid, parent, locidp));
        free(parent);
-    return stat;
+    return NC_NOERR;
 }
 
 /* Return size of chunk in bytes for a variable varid in a group igrp, or 0 if
@@ -376,29 +405,40 @@ copy_groups(int iroot, int oroot)
 
     /* get total number of groups and their ids, including all descendants */
     NC_CHECK(nc_inq_grps_full(iroot, &numgrps, NULL));
-    grpids = emalloc(numgrps * sizeof(int));
-    NC_CHECK(nc_inq_grps_full(iroot, NULL, grpids));
-    /* create corresponding new groups in ogrp, except for root group */
-    for(i = 1; i < numgrps; i++) {
-	char *grpname_full;
-	char grpname[NC_MAX_NAME];
-	size_t len_name;
-	int ogid, oparid;
-	/* get full group name of input group */
-	NC_CHECK(nc_inq_grpname_full(grpids[i], &len_name, NULL));
-	grpname_full = emalloc(len_name + 1);
-	NC_CHECK(nc_inq_grpname_full(grpids[i], &len_name, grpname_full));
-	/* get id of parent group of corresponding group in output.
-	 * Note that this exists, because nc_inq_groups returned
-	 * grpids in preorder, so parents are always copied before
-	 * their subgroups */
-	NC_CHECK(nc_inq_parid(oroot, grpname_full, &oparid));
-	NC_CHECK(nc_inq_grpname(grpids[i], grpname));
-	/* define corresponding group in output */
-	NC_CHECK(nc_def_grp(oparid, grpname, &ogid));
-	free(grpname_full);
+    if(numgrps > 1) {		/* there's always 1 root group */
+	grpids = emalloc(numgrps * sizeof(int));
+	NC_CHECK(nc_inq_grps_full(iroot, NULL, grpids));
+	/* create corresponding new groups in ogrp, except for root group */
+	for(i = 1; i < numgrps; i++) {
+	    char *grpname_full;
+	    char grpname[NC_MAX_NAME];
+	    size_t len_name;
+	    int ogid, oparid, iparid;
+	    /* get full group name of input group */
+	    NC_CHECK(nc_inq_grpname(grpids[i], grpname));
+	    if (option_grpstruct || group_wanted(grpids[i], option_nlgrps, option_grpids)) {
+	        NC_CHECK(nc_inq_grpname_full(grpids[i], &len_name, NULL));
+		grpname_full = emalloc(len_name + 1);
+		NC_CHECK(nc_inq_grpname_full(grpids[i], &len_name, grpname_full));
+		/* Make sure, the parent group is also wanted (root group is always wanted) */
+		NC_CHECK(nc_inq_parid(iroot, grpname_full, &iparid));
+		if (!option_grpstruct && !group_wanted(iparid, option_nlgrps, option_grpids) 
+		    && iparid != iroot) {
+		    error("ERROR: trying to copy a group but not the parent: %s", grpname_full);
+		}
+		/* get id of parent group of corresponding group in output.
+		 * Note that this exists, because nc_inq_groups returned
+		 * grpids in preorder, so parents are always copied before
+		 * their subgroups */
+		NC_CHECK(nc_inq_parid(oroot, grpname_full, &oparid));
+		NC_CHECK(nc_inq_grpname(grpids[i], grpname));
+		/* define corresponding group in output */
+		NC_CHECK(nc_def_grp(oparid, grpname, &ogid));
+		free(grpname_full);
+	    }
+	}
+	free(grpids);
     }
-    free(grpids);
     return stat;    
 }
 
@@ -433,12 +473,14 @@ copy_types(int igrp, int ogrp)
 	grpids = (int *)emalloc(sizeof(int) * numgrps);
 	NC_CHECK(nc_inq_grps(igrp, &numgrps, grpids));
 	for(i = 0; i < numgrps; i++) {
-	    int ogid;
-	    /* get groupid in output corresponding to grpids[i] in
-	     * input, given parent group (or root group) ogrp in
-	     * output */
-	    NC_CHECK(get_grpid(grpids[i], ogrp, &ogid));
-	    NC_CHECK(copy_types(grpids[i], ogid));
+	    if (option_grpstruct || group_wanted(grpids[i], option_nlgrps, option_grpids)) {
+		int ogid;
+		/* get groupid in output corresponding to grpids[i] in
+		 * input, given parent group (or root group) ogrp in
+		 * output */
+		NC_CHECK(get_grpid(grpids[i], ogrp, &ogid));
+		NC_CHECK(copy_types(grpids[i], ogid));
+	    }
 	}
 	free(grpids);
     }
@@ -456,33 +498,44 @@ copy_var_specials(int igrp, int varid, int ogrp, int o_varid)
 	NC_CHECK(nc_inq_varndims(igrp, varid, &ndims));
 	if (ndims > 0) {		/* no chunking for scalar variables */
 	    int contig = 0;
-	    NC_CHECK(nc_inq_var_chunking(igrp, varid, &contig, NULL));
-	    if(contig == 1) {
+	    size_t *chunkp = (size_t *) emalloc(ndims * sizeof(size_t));
+	    int *dimids = (int *) emalloc(ndims * sizeof(int));
+	    int idim;
+	     /* size of a chunk: product of dimension chunksizes and size of value */ 
+	    size_t csprod = val_size(ogrp, o_varid);
+	    int is_unlimited = 0;
+	    NC_CHECK(nc_inq_var_chunking(igrp, varid, &contig, chunkp));
+	    NC_CHECK(nc_inq_vardimid(igrp, varid, dimids));
+
+	    for(idim = 0; idim < ndims; idim++) {
+		int idimid = dimids[idim];
+		int odimid = dimmap_odimid(idimid);
+		size_t chunksize = chunkspec_size(idimid);
+		if(chunksize > 0) { /* found in chunkspec */
+		    chunkp[idim] = chunksize;
+		}
+		csprod *= chunkp[idim];
+		if(dimmap_ounlim(odimid))
+		    is_unlimited = 1;
+	    }
+	    /* Explicitly set chunking, even if default */
+	    /* If product of chunksizes is too small and no unlimited
+	     * dimensions used, don't chunk */
+	    if ((csprod < option_min_chunk_bytes && !is_unlimited) || contig == 1) {
 		NC_CHECK(nc_def_var_chunking(ogrp, o_varid, NC_CONTIGUOUS, NULL));
 	    } else {
-		size_t *chunkp = (size_t *) emalloc(ndims * sizeof(size_t));
-		int *dimids = (int *) emalloc(ndims * sizeof(int));
-		int idim;
-		NC_CHECK(nc_inq_var_chunking(igrp, varid, NULL, chunkp));
-		NC_CHECK(nc_inq_vardimid(igrp, varid, dimids));
-		for(idim = 0; idim < ndims; idim++) {
-		    int dimid = dimids[idim];
-		    size_t chunksize = chunkspec_size(dimid);
-		    if(chunkspec_size(dimid) > 0) { /* found in chunkspec */
-			chunkp[idim] = chunksize;
-		    }
-		}
-		/* explicitly set chunking, even if default */
 		NC_CHECK(nc_def_var_chunking(ogrp, o_varid, NC_CHUNKED, chunkp));
-		free(dimids);
-		free(chunkp);
 	    }
+	    free(dimids);
+	    free(chunkp);
 	}
     }
     { /* handle compression parameters, copying from input, overriding
        * with command-line options */
 	int shuffle, deflate, deflate_level;
 	NC_CHECK(nc_inq_var_deflate(igrp, varid, &shuffle, &deflate, &deflate_level));
+	if(deflate_level == 0 && deflate == 1) /* TODO: why is this needed??  Bug in nc_inq_var_deflate? */
+	    deflate = 0;
 	if(option_deflate_level >= 0) { /* change output compression, if requested */
 	  deflate_level = option_deflate_level;
  	  deflate=1;
@@ -803,11 +856,29 @@ copy_vars(int igrp, int ogrp)
     int stat = NC_NOERR;
     int nvars;
     int varid;
+
+    int iv;			/* variable number */
+    idnode_t* vlist = 0;		/* list for vars specified with -v option */
+
+    /*
+     * If any vars were specified with -v option, get list of
+     * associated variable ids relative to this group.  Assume vars
+     * specified with syntax like "grp1/grp2/varname" or
+     * "/grp1/grp2/varname" if they are in groups.
+     */
+    vlist = newidlist();	/* list for vars specified with -v option */
+    for (iv=0; iv < option_nlvars; iv++) {
+        if(nc_inq_gvarid(igrp, option_lvars[iv], &varid) == NC_NOERR)
+            idadd(vlist, varid);
+    }
     
     NC_CHECK(nc_inq_nvars(igrp, &nvars));
     for (varid = 0; varid < nvars; varid++) {
+	if (!option_varstruct && option_nlvars > 0 && ! idmember(vlist, varid))
+            continue;
 	NC_CHECK(copy_var(igrp, varid, ogrp));
     }
+    free(vlist);
     return stat;
 }
 
@@ -838,7 +909,9 @@ copy_schema(int igrp, int ogrp)
 	NC_CHECK(nc_inq_grps(igrp, &numgrps, grpids));
 	
 	for(i = 0; i < numgrps; i++) {
-	    NC_CHECK(copy_schema(grpids[i], ogid));
+	    if (option_grpstruct || group_wanted(grpids[i], option_nlgrps, option_grpids)) {
+	        NC_CHECK(copy_schema(grpids[i], ogid));
+	    }
 	}
 	free(grpids);
     }
@@ -897,8 +970,7 @@ copy_var_data(int igrp, int varid, int ogrp) {
     NC_CHECK(nc_inq_varname(igrp, varid, varname));
     NC_CHECK(nc_inq_varid(ogrp, varname, &ovarid));
     NC_CHECK(nc_inq_vartype(igrp, varid, &vartype));
-    /* from type, get size in memory needed for each value */
-    NC_CHECK(nc_inq_type(igrp, vartype, NULL, &value_size));
+    value_size = val_size(igrp, varid);
     if(value_size > option_copy_buffer_size) {
 	option_copy_buffer_size = value_size;
 	do_realloc = 1;
@@ -1004,6 +1076,21 @@ copy_data(int igrp, int ogrp)
     int i;
 #endif
 
+    int iv;			/* variable number */
+    idnode_t* vlist = 0;		/* list for vars specified with -v option */
+
+    /*
+     * If any vars were specified with -v option, get list of
+     * associated variable ids relative to this group.  Assume vars
+     * specified with syntax like "grp1/grp2/varname" or
+     * "/grp1/grp2/varname" if they are in groups.
+     */
+    vlist = newidlist();	/* list for vars specified with -v option */
+    for (iv=0; iv < option_nlvars; iv++) {
+        if(nc_inq_gvarid(igrp, option_lvars[iv], &varid) == NC_NOERR)
+            idadd(vlist, varid);
+    }
+    
     /* get groupid in output corresponding to group igrp in input,
      * given parent group (or root group) ogrp in output */
     NC_CHECK(get_grpid(igrp, ogrp, &ogid));
@@ -1012,6 +1099,10 @@ copy_data(int igrp, int ogrp)
     NC_CHECK(nc_inq_nvars(igrp, &nvars));
 
     for (varid = 0; varid < nvars; varid++) {
+	if (option_nlvars > 0 && ! idmember(vlist, varid))
+            continue;
+        if (!group_wanted(igrp, option_nlgrps, option_grpids))
+            continue;
 	NC_CHECK(copy_var_data(igrp, varid, ogid));
     }
 #ifdef USE_NETCDF4
@@ -1021,6 +1112,8 @@ copy_data(int igrp, int ogrp)
     NC_CHECK(nc_inq_grps(igrp, &numgrps, grpids));
 
     for(i = 0; i < numgrps; i++) {
+        if (!option_grpstruct && !group_wanted(grpids[i], option_nlgrps, option_grpids))
+            continue;
 	NC_CHECK(copy_data(grpids[i], ogid));
     }
     free(grpids);
@@ -1028,23 +1121,24 @@ copy_data(int igrp, int ogrp)
     return stat;
 }
 
-/* Count total number of dimensions in ncid and all its subgroups */
+/* Count total number of dimensions in ncid and all its descendant subgroups */
 int
 count_dims(ncid) {
     int numgrps;
-    int *grpids;
-    int igrp;
-    int ndims=0;
-    /* get total number of groups and their ids, including all descendants */
-    NC_CHECK(nc_inq_grps_full(ncid, &numgrps, NULL));
-    grpids = emalloc(numgrps * sizeof(int));
-    NC_CHECK(nc_inq_grps_full(ncid, NULL, grpids));
-    for(igrp = 0; igrp < numgrps; igrp++) {
-	int ndims_local;
-	nc_inq_ndims(grpids[igrp], &ndims_local);
-	ndims += ndims_local;
+    int ndims;
+    NC_CHECK(nc_inq_ndims(ncid, &ndims));
+#ifdef USE_NETCDF4
+    NC_CHECK(nc_inq_grps(ncid, &numgrps, NULL));
+    if(numgrps > 0) {
+	int igrp;
+	int *grpids = emalloc(numgrps * sizeof(int));
+	NC_CHECK(nc_inq_grps(ncid, &numgrps, grpids));
+	for(igrp = 0; igrp < numgrps; igrp++) {
+	    ndims += count_dims(grpids[igrp]);
+	}
+	free(grpids); 
     }
-    free(grpids); 
+#endif	/* USE_NETCDF4 */
     return ndims;
 }
 
@@ -1167,7 +1261,6 @@ copy_record_data(int ncid, int ogrp, size_t nrec_vars, int *rec_varids) {
 	int varid;
 	int ndims;
 	int *dimids;
-	nc_type vartype;
 	size_t value_size;
 	int dimid;
 	int ii;
@@ -1179,8 +1272,7 @@ copy_record_data(int ncid, int ogrp, size_t nrec_vars, int *rec_varids) {
 	start[ivar] = (size_t *) emalloc(ndims * sizeof(size_t));
 	count[ivar] = (size_t *) emalloc(ndims * sizeof(size_t));
 	NC_CHECK(nc_inq_vardimid (ncid, varid, dimids));
-	NC_CHECK(nc_inq_vartype(ncid, varid, &vartype));
-	NC_CHECK(nc_inq_type(ncid, vartype, NULL, &value_size));
+	value_size = val_size(ncid, varid);
 	nvals = 1;
 	for(ii = 1; ii < ndims; ii++) { /* for rec size, don't include first record dimension */
 	    size_t dimlen;
@@ -1287,6 +1379,20 @@ copy(char* infile, char* outfile)
     }
 #endif	/* USE_NETCDF4 */
 
+	/* Check if any vars in -v don't exist */
+    if(missing_vars(igrp, option_nlvars, option_lvars))
+	return EXIT_FAILURE;
+
+    if(option_nlgrps > 0) {
+	if(inkind != NC_FORMAT_NETCDF4) {
+	    error("Group list (-g ...) only permitted for netCDF-4 file");
+	    return EXIT_FAILURE;
+	}
+	/* Check if any grps in -g don't exist */
+	if(grp_matches(igrp, option_nlgrps, option_lgrps, option_grpids) == 0)
+	    return EXIT_FAILURE;
+    }
+
     if(option_write_diskless)
 	create_mode |= NC_WRITE | NC_DISKLESS; /* NC_WRITE persists diskless file on close */
     switch(outkind) {
@@ -1333,6 +1439,7 @@ copy(char* infile, char* outfile)
     /* For performance, special case netCDF-3 input or output file with record
      * variables, to copy a record-at-a-time instead of a
      * variable-at-a-time. */
+    /* TODO: check that these special cases work with -v option */
     if(nc3_special_case(igrp, inkind)) {
 	size_t nfixed_vars, nrec_vars;
 	int *fixed_varids;
@@ -1357,6 +1464,44 @@ copy(char* infile, char* outfile)
     return stat;
 }
 
+/* 
+ * For non-negative numeric string with multiplier suffix K, M, G, T,
+ * or P (or lower-case equivalent), return corresponding value
+ * incorporating multiplier 1000, 1000000, 1.0d9, ... 1.0d15, or -1.0
+ * for error.
+ */
+static double
+double_with_suffix(char *str) {
+    double dval;
+    char *suffix = 0;
+    errno = 0;
+    dval = strtod(str, &suffix);
+    if(dval < 0 || errno != 0)
+	return -1.0;
+    if(*suffix) {
+	switch (*suffix) {
+	case 'k': case 'K':
+	    dval *= 1000;
+	    break;
+	case 'm': case 'M':
+	    dval *= 1000000;
+	    break;
+	case 'g': case 'G':
+	    dval *= 1000000000;
+	    break;
+	case 't': case 'T':
+	    dval *= 1.0e12;
+	    break;
+	case 'p': case 'P':
+	    dval *= 1.0e15;
+	    break;
+	default:
+	    dval = -1.0;	/* error, suffix multiplier must be K, M, G, or T */
+	}		
+    }
+    return dval;
+}
+
 static void
 usage(void)
 {
@@ -1368,6 +1513,10 @@ usage(void)
   [-c chunkspec] specify chunking for dimensions, e.g. \"dim1/N1,dim2/N2,...\"\n\
   [-u]      convert unlimited dimensions to fixed-size dimensions in output copy\n\
   [-w]      write whole output file from diskless netCDF on close\n\
+  [-v var1,...] include data for only listed variables, but definitions for all variables\n\
+  [-V var1,...] include definitions and data for only listed variables\n\
+  [-g grp1,...] include data for only variables in listed groups, but all definitions\n\
+  [-G grp1,...] include definitions and data only for variables in listed groups\n\
   [-m n]    set size in bytes of copy buffer, default is 5000000 bytes\n\
   [-h n]    set size in bytes of chunk_cache for chunked variables\n\
   [-e n]    set number of elements that chunk_cache can hold\n\
@@ -1378,7 +1527,7 @@ usage(void)
     /* Don't document this flaky option until it works better */
     /* [-x]      use experimental computed estimates for variable-specific chunk caches\n\ */
 
-    error("%s [-k n] [-d n] [-s] [-c chunkspec] [-u] [-w] [-m n] [-h n] [-e n] [-r] infile outfile\n%s",
+    error("%s [-k n] [-d n] [-s] [-c chunkspec] [-u] [-w] [-[v|V] varlist] [-[g|G] grplist] [-m n] [-h n] [-e n] [-r] infile outfile\n%s",
 	  progname, USAGE);
 }
 
@@ -1386,8 +1535,9 @@ usage(void)
 @section  SYNOPSIS
 
 \code
-nccopy [-k kind] [-d n] [-s] [-u] [-w] [-c chunkspec] [-m bufsize] 
-       [-h chunk_cache] [-e cache_elems] [-r]   infile   outfile
+nccopy [-k kind] [-d n] [-s] [-c chunkspec] [-u] [-w] [-[v|V] var1,...]
+       [-[g|G] grp1,...] [-m bufsize] [-h chunk_cache] [-e cache_elems]
+       [-r]   infile   outfile
 \endcode
 
 @section  DESCRIPTION
@@ -1508,6 +1658,56 @@ programming interface has no such restriction.  If you need to
 customize chunking for variables independently, you will need to use
 the library API in a custom utility program.
 
+@par -v \a var1,...  
+
+@par 
+The output will include data values for the specified variables, in
+addition to the declarations of all dimensions, variables, and
+attributes. One or more variables must be specified by name in the
+comma-delimited list following this option. The list must be a single
+argument to the command, hence cannot contain unescaped blanks or
+other white space characters. The named variables must be valid netCDF
+variables in the input-file. A variable within a group in a netCDF-4
+file may be specified with an absolute path name, such as
+`/GroupA/GroupA2/var'.  Use of a relative path name such as `var' or
+`grp/var' specifies all matching variable names in the file.  The
+default, without this option, is to include data values for \e all variables
+in the output.
+
+@par -V \a var1,...  
+
+@par 
+The output will include the specified variables only but all dimensions and
+global or group attributes. One or more variables must be specified by name in the
+comma-delimited list following this option. The list must be a single argument
+to the command, hence cannot contain unescaped blanks or other white space
+characters. The named variables must be valid netCDF variables in the
+input-file. A variable within a group in a netCDF-4 file may be specified with
+an absolute path name, such as `/GroupA/GroupA2/var'.  Use of a relative path
+name such as `var' or `grp/var' specifies all matching variable names in the
+file.  The default, without this option, is to include \e all variables in the
+output.
+
+@par -g \e grp1,...
+
+@par
+The output will include data values only for the specified groups.
+One or more groups must be specified by name in the comma-delimited
+list following this option. The list must be a single argument to the
+command. The named groups must be valid netCDF groups in the
+input-file. The default, without this option, is to include data values for all
+groups in the output.
+
+@par -G \e grp1,...
+
+@par
+The output will include only the specified groups.
+One or more groups must be specified by name in the comma-delimited
+list following this option. The list must be a single argument to the
+command. The named groups must be valid netCDF groups in the
+input-file. The default, without this option, is to include all groups in the
+output.
+
 @par -m  \e  bufsize
 @par
 An integer or floating-point number that specifies the size, in bytes,
@@ -1520,7 +1720,7 @@ a value larger than the default for copying large files over high
 latency networks.  Using the '-w' option may provide better
 performance, if the output fits in memory.
 
-@par -e \e chunk_cache
+@par -h \e chunk_cache
 @par
 For netCDF-4 output, including netCDF-4 classic model, an integer or
 floating-point number that specifies the size in bytes of chunk cache
@@ -1537,10 +1737,12 @@ cache, but no general algorithm for computing the optimum chunk cache
 size has been implemented yet. Using the '-w' option may provide
 better performance, if the output fits in memory.
 
-@par -h \e cache_elems
+@par -e \e cache_elems
 @par
 For netCDF-4 output, including netCDF-4 classic model, specifies
-number of elements that the chunk cache can hold. This is not a
+number of elements that the chunk cache can hold. A suffix of K, M, G,
+or T multiplies the copy buffer size by one thousand, million,
+billion, or trillion, respectively.  This is not a
 property of the file, but merely a performance tuning parameter for
 avoiding compressing or decompressing the same data multiple times
 while copying and changing chunk shapes.  The default is 1009 (or
@@ -1628,7 +1830,7 @@ nccopy -w -c time/1000,lat/40,lon/40 slow.nc fast.nc
 \endcode
 @section see_also SEE ALSO
 
-netcdf(3), ncgen(1), netcdf(3)
+ncdump(1), ncgen(1), netcdf(3)
 
  */
 int
@@ -1677,7 +1879,7 @@ main(int argc, char**argv)
        usage();
     }
 
-    while ((c = getopt(argc, argv, "k:d:sum:c:h:e:rwx")) != -1) {
+    while ((c = getopt(argc, argv, "k:d:sum:c:h:e:rwxg:G:v:V:")) != -1) {
 	switch(c) {
         case 'k': /* for specifying variant of netCDF format to be generated 
                      Possible values are:
@@ -1721,64 +1923,28 @@ main(int argc, char**argv)
 	    break;
 	case 'm':		/* non-default size of data copy buffer */
 	{
-	    double dval;
-	    char *suffix = 0;	/* "K" for kilobytes. "M" for megabytes, ... */
-	    dval = strtod(optarg, &suffix);
-	    if(*suffix) {
-		switch (*suffix) {
-		case 'k': case 'K':
-		    dval *= 1000;
-		    break;
-		case 'm': case 'M':
-		    dval *= 1000000;
-		    break;
-		case 'g': case 'G':
-		    dval *= 1000000000;
-		    break;
-		case 't': case 'T':
-		    dval *= 1.0e12;
-		    break;
-		default:
-		    error("If suffix used for '-m' option value, it must be K, M, G, or T: %c", 
-			  *suffix);
-		}		
-	    }
+	    double dval = double_with_suffix(optarg);	/* "K" for kilobytes. "M" for megabytes, ... */
+	    if(dval < 0)
+		error("Suffix used for '-m' option value must be K, M, G, T, or P");
 	    option_copy_buffer_size = dval;
 	    break;
 	}
 	case 'h':		/* non-default size of chunk cache */
 	{
-	    double dval;
-	    char *suffix = 0;	/* "K" for kilobytes, "M" for megabytes, ... */
-	    dval = strtod(optarg, &suffix);
-	    if(*suffix) {
-		switch (*suffix) {
-		case 'k': case 'K':
-		    dval *= 1000;
-		    break;
-		case 'm': case 'M':
-		    dval *= 1000000;
-		    break;
-		case 'g': case 'G':
-		    dval *= 1000000000;
-		    break;
-		case 't': case 'T':
-		    dval *= 1.0e12;
-		    break;
-		default:
-		    error("If suffix used for '-h' option value, it must be K, M, G, or T: %c", 
-			  *suffix);
-		}		
-	    }
+	    double dval = double_with_suffix(optarg);	/* "K" for kilobytes. "M" for megabytes, ... */
+	    if(dval < 0)
+		error("Suffix used for '-h' option value must be K, M, G, T, or P");
 	    option_chunk_cache_size = dval;
 	    break;
-	    }
+	}
 	case 'e':		/* number of elements chunk cache can hold */
-	    option_chunk_cache_nelems = strtol(optarg, NULL, 10);
-	    if(option_chunk_cache_nelems <= 0) {
-		error("invalid value for number of chunk cache elements: %d", option_chunk_cache_nelems);
-	    }
+	{
+	    double dval = double_with_suffix(optarg);	/* "K" for kilobytes. "M" for megabytes, ... */
+	    if(dval < 0 )
+		error("Suffix used for '-e' option value must be K, M, G, T, or P");
+	    option_chunk_cache_nelems = (long)dval;
 	    break;
+	}
 	case 'r':
 	    option_read_diskless = 1; /* read into memory on open */
 	    break;
@@ -1789,11 +1955,29 @@ main(int argc, char**argv)
 	    option_compute_chunkcaches = 1;
 	    break;
 	case 'c':               /* optional chunking spec for each dimension in list */
-	{
 	    /* save chunkspec string for parsing later, once we know input ncid */
 	    option_chunkspec = strdup(optarg);
 	    break;
-	}
+	case 'g':		/* group names */
+	    /* make list of names of groups specified */
+	    make_lgrps (optarg, &option_nlgrps, &option_lgrps, &option_grpids);
+	    option_grpstruct = true;
+	    break;
+	case 'G':		/* group names */
+	    /* make list of names of groups specified */
+	    make_lgrps (optarg, &option_nlgrps, &option_lgrps, &option_grpids);
+	    option_grpstruct = false;
+	    break;
+	case 'v':		/* variable names */
+	    /* make list of names of variables specified */
+	    make_lvars (optarg, &option_nlvars, &option_lvars);
+	    option_varstruct = true;
+	    break;
+	case 'V':		/* variable names */
+	    /* make list of names of variables specified */
+	    make_lvars (optarg, &option_nlvars, &option_lvars);
+	    option_varstruct = false;
+	    break;
 	default: 
 	    usage();
         }
