@@ -11,6 +11,14 @@ Research/Unidata. See \ref copyright file for more info.  */
 #ifndef _WIN32
 #include <unistd.h>
 #endif
+
+#ifdef _MSC_VER
+#define snprintf _snprintf
+#include "XGetopt.h"
+int opterr;
+int optind;
+#endif
+
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
@@ -295,7 +303,7 @@ pr_att_string(
 	    printf ("\\\\");
 	    break;
 	case '\'':
-	    printf ("\\'");
+	    printf ("\\\'");
 	    break;
 	case '\"':
 	    printf ("\\\"");
@@ -639,9 +647,9 @@ pr_att(
 	    /* Prints text after semicolon and before final newline.
 	     * Prints nothing if not qualified for time interpretation.
 	     * Will include line breaks for longer lists. */
-	    print_att_times(ncid, varid, att);
+	    print_att_times(ncid, varid, &att);
 	    if(is_bounds_att(&att)) {
-		insert_bounds_info(ncid, varid, att);
+		insert_bounds_info(ncid, varid, &att);
 	    }
 	}
 #ifdef USE_NETCDF4
@@ -860,7 +868,7 @@ pr_att_specials(
     {
 	int endianness = 0;
 	NC_CHECK( nc_inq_var_endian(ncid, varid, &endianness) );
-	if(endianness != 0) {
+	if (endianness != NC_ENDIAN_NATIVE) { /* NC_ENDIAN_NATIVE is the default */
 	    pr_att_name(ncid, varp->name, NC_ATT_ENDIANNESS);
 	    printf(" = ");
 	    switch (endianness) {
@@ -870,9 +878,6 @@ pr_att_specials(
 	    case NC_ENDIAN_BIG:
 		printf("\"big\"");
 		break;
-	    case NC_ENDIAN_NATIVE:
-		printf("\"native\"");
-		break;
 	    default:
 		error("pr_att_specials: bad endianness: %d", endianness);
 		break;
@@ -913,12 +918,11 @@ pr_attx(
     )
 {
     ncatt_t att;			/* attribute */
-    char *attvals = "";
+    char *attvals = NULL;
     int attvalslen = 0;
 
     NC_CHECK( nc_inq_attname(ncid, varid, ia, att.name) );
     NC_CHECK( nc_inq_att(ncid, varid, att.name, &att.type, &att.len) );
-    att.tinfo = get_typeinfo(att.type);
 
     /* Put attribute values into a single string, with blanks in between */
 
@@ -980,7 +984,8 @@ pr_attx(
 	printf("%s\"",attvals);
     }
     printf (" />\n");
-    free (attvals);
+    if(attvals != NULL)
+      free (attvals);
 }
 
 
@@ -1527,7 +1532,8 @@ do_ncdump_rec(int ncid, const char *path)
    /* output variable data, unless "-h" option specified header only
     * or this group is not in list of groups specified by "-g"
     * option  */
-   if (! formatting_specs.header_only && group_wanted(ncid) ) {
+   if (! formatting_specs.header_only && 
+       group_wanted(ncid, formatting_specs.nlgrps, formatting_specs.grpids) ) {
       if (nvars > 0) {
 	  indent_out();
 	  printf ("data:\n");
@@ -1676,7 +1682,7 @@ do_ncdumpx(int ncid, const char *path)
     ncvar_t var;		/* variable */
     int ia;			/* attribute number */
     int iv;			/* variable number */
-    idnode_t* vlist = 0;		/* list for vars specified with -v option */
+    idnode_t* vlist = 0;        /* list for vars specified with -v option */
 
     /*
      * If any vars were specified with -v option, get list of associated
@@ -1763,52 +1769,6 @@ do_ncdumpx(int ncid, const char *path)
 	free(dims);
 }
 
-static void
-make_lvars(char *optarg)
-{
-    char *cp = optarg;
-    int nvars = 1;
-    char ** cpp;
-
-    /* compute number of variable names in comma-delimited list */
-    formatting_specs.nlvars = 1;
-    while (*cp++)
-      if (*cp == ',')
- 	nvars++;
-    formatting_specs.nlvars = nvars;
-    formatting_specs.lvars = (char **) emalloc(nvars * sizeof(char*));
-    cpp = formatting_specs.lvars;
-    /* copy variable names into list */
-    for (cp = strtok(optarg, ","); cp != NULL; cp = strtok((char *) NULL, ",")) {
-	*cpp = strdup(cp);
-	cpp++;
-    }
-}
-
-static void
-make_lgrps(char *optarg)
-{
-    char *cp = optarg;
-    int ngrps = 1;
-    char ** cpp;
-
-    /* compute number of group names in comma-delimited list */
-    while (*cp++)
-      if (*cp == ',')
- 	ngrps++;
-    formatting_specs.nlgrps = ngrps;
-    formatting_specs.lgrps = (char **) emalloc(ngrps * sizeof(char*));
-    cpp = formatting_specs.lgrps;
-    /* copy group names into list */
-    for (cp = strtok(optarg, ","); cp != NULL; cp = strtok((char *) NULL, ",")) {
-	*cpp = strdup(cp);
-	cpp++;
-    }
-    /* make empty list of grpids, to be filled in after input file opened */
-    formatting_specs.grpids = newidlist();
-}
-
-
 /*
  * Extract the significant-digits specifiers from the (deprecated and
  * undocumented) -d argument on the command-line and update the
@@ -1875,161 +1835,6 @@ set_precision(const char *optarg)
     set_formats(flt_digits, dbl_digits);
 }
 
-/* Determine whether a variable named varname exists in any group in
-   an open netCDF file with id ncid.  If so, return the count of how
-   many matching variables were found, else return a count of 0.  The
-   variable name can be absolute such as "/foo" or "/GRP1/GRP1A/foo",
-   in which case there is only one group to look in, given by the path
-   from the root group.  Alternatively, the variable name can be
-   relative, such as "foo" or "GRPA/GRPB/foo", in which case every
-   group is examined for a variable with that relative name.  */
-size_t
-nc_inq_varname_count(int ncid, char *varname) {
-    /* 
-       count = 0;
-       status = nc_inq_gvarid(ncid, varname, varid);
-       if (status == NC_NOERR)
-          count++;
-       for each subgroup gid {
-          count += nc_inq_varname_count(gid, varname);
-       }
-       return count;
-    */
-    size_t count = 0;
-    int varid;
-    /* look in this group */
-    int status = nc_inq_gvarid(ncid, varname, &varid);
-#ifdef USE_NETCDF4
-    int numgrps;
-    int *ncids;
-    int g;
-#endif
-
-    if (status == NC_NOERR)
-	count++;
-
-#ifdef USE_NETCDF4
-    /* if this group has subgroups, call recursively on each of them */
-    NC_CHECK( nc_inq_grps(ncid, &numgrps, NULL) );
-	 
-    /* Allocate memory to hold the list of group ids. */
-    ncids = emalloc((numgrps + 1) * sizeof(int));
-	
-    /* Get the list of group ids. */
-    NC_CHECK( nc_inq_grps(ncid, NULL, ncids) );
-	
-    /* Call this function for each group. */
-    for (g = 0; g < numgrps; g++) {
-	count += nc_inq_varname_count(ncids[g], varname);
-    }
-    free(ncids);
-#endif /* USE_NETCDF4 */
-    return count;    
-   
-}
-
-
-/* Check if any variable names specified with "-v var1,...,varn" are
- * missing.  Returns 0 if no missing variables detected, otherwise
- * exits. */
-static int
-missing_vars(int ncid) {
-    int iv;
-    for (iv=0; iv < formatting_specs.nlvars; iv++) {
-	if(nc_inq_varname_count(ncid, formatting_specs.lvars[iv]) == 0) {
-	    error("%s: No such variable", formatting_specs.lvars[iv]);
-	}
-    }
-    return 0;
-}
-
-/* Determine whether a group named formatting_specs.lgrps[igrp] exists
- * in a netCDF file or group with id ncid.  If so, return the count of
- * how many matching groups were found, else return a count of 0.  If
- * the name begins with "/", it is interpreted as an absolute group
- * name, in which case only 0 or 1 is returned.  Otherwise, interpret
- * it as a relative name, and the total number of occurrences within
- * the file/group identified by ncid is returned.  
- *
- * Also has side effect of updating the ngrpids and the associate
- * grpids array that represent the group list specified by the -g
- * option.  TODO: put this in its own function instead.
- */
-static size_t
-nc_inq_grpname_count(int ncid, int igrp) {
-    size_t count = 0;
-#ifdef USE_NETCDF4
-    int numgrps;
-    int *ncids;
-    int g;
-    int grpid;
-    int status;
-#endif
-    char *grpname=formatting_specs.lgrps[igrp];
-
-    /* permit empty string to also designate root group */
-    if(grpname[0] == '\0' || STREQ(grpname,"/")) { 
-	count = 1;
-	idadd(formatting_specs.grpids, ncid);
-	return count;
-    }
-#ifdef USE_NETCDF4
-    /* Handle absolute group names */
-    if(grpname[0] == '/') {
-	int grpid;
-	status = nc_inq_grp_full_ncid(ncid, grpname, &grpid);
-	if(status == NC_NOERR) {
-	    count = 1;
-	    idadd(formatting_specs.grpids, grpid);
-	} else if(status == NC_ENOGRP) {
-	    count = 0;
-	} else {
-	    error("when looking up group %s: %s ", grpname, nc_strerror(status));
-	}
-	return count;
-    }
-    
-    /* look in this group */
-    status = nc_inq_grp_ncid(ncid, grpname, &grpid);
-    if (status == NC_NOERR) {
-	count++;
-	idadd(formatting_specs.grpids, grpid);
-    }
-    /* if this group has subgroups, call recursively on each of them */
-    NC_CHECK( nc_inq_grps(ncid, &numgrps, NULL) );
-    if(numgrps > 0) {
-	/* Allocate memory to hold the list of group ids. */
-	ncids = emalloc(numgrps * sizeof(int));
-	/* Get the list of group ids. */
-	NC_CHECK( nc_inq_grps(ncid, NULL, ncids) );
-	/* Call this function recursively for each group. */
-	for (g = 0; g < numgrps; g++) {
-	    count += nc_inq_grpname_count(ncids[g], igrp);
-	}
-	free(ncids);
-    }
-#endif /* USE_NETCDF4 */
-    return count;    
-}
-
-/* Check if any group names specified with "-g grp1,...,grpn" are
- * missing.  Returns total number of matching groups if no missing
- * groups detected, otherwise exits. */
-static int
-grp_matches(int ncid) {
-    int ig;
-    size_t total = 0;
-
-    for (ig=0; ig < formatting_specs.nlgrps; ig++) {
-	size_t count = nc_inq_grpname_count(ncid, ig);
-	if(count == 0) {
-	    error("%s: No such group", formatting_specs.lgrps[ig]);
-	    return 0;
-	}
-	total += count;
-    }
-    return total;
-}
 
 #ifdef USE_DAP
 #define DAP_CLIENT_CACHE_DIRECTIVE	"[cache]"
@@ -2322,8 +2127,8 @@ main(int argc, char *argv[])
     int i;
     int max_len = 80;		/* default maximum line length */
     int nameopt = 0;
-    boolean xml_out = false;    /* if true, output NcML instead of CDL */
-    boolean kind_out = false;	/* if true, just output kind of netCDF file */
+    bool_t xml_out = false;    /* if true, output NcML instead of CDL */
+    bool_t kind_out = false;	/* if true, just output kind of netCDF file */
 
 #if defined(WIN32) || defined(msdos) || defined(WIN64)
     putenv("PRINTF_EXPONENT_DIGITS=2"); /* Enforce unix/linux style exponent formatting. */
@@ -2397,11 +2202,12 @@ main(int argc, char *argv[])
 	  break;
 	case 'v':		/* variable names */
 	  /* make list of names of variables specified */
-	  make_lvars (optarg);
+	  make_lvars (optarg, &formatting_specs.nlvars, &formatting_specs.lvars);
 	  break;
 	case 'g':		/* group names */
 	  /* make list of names of groups specified */
-	  make_lgrps (optarg);
+	  make_lgrps (optarg, &formatting_specs.nlgrps, &formatting_specs.lgrps, 
+			&formatting_specs.grpids);
 	  break;
 	case 'd':		/* specify precision for floats (deprecated, undocumented) */
 	  set_sigdigs(optarg);
@@ -2485,7 +2291,7 @@ main(int argc, char *argv[])
 		/* Initialize list of types. */
 		init_types(ncid);
 		/* Check if any vars in -v don't exist */
-		if(missing_vars(ncid))
+		if(missing_vars(ncid, formatting_specs.nlvars, formatting_specs.lvars))
 		    return EXIT_FAILURE;
 		if(formatting_specs.nlgrps > 0) {
 		    if(formatting_specs.nc_kind != NC_FORMAT_NETCDF4) {
@@ -2493,7 +2299,7 @@ main(int argc, char *argv[])
 			return EXIT_FAILURE;
 		    }
 		    /* Check if any grps in -g don't exist */
-		    if(grp_matches(ncid) == 0)
+		    if(grp_matches(ncid, formatting_specs.nlgrps, formatting_specs.lgrps, formatting_specs.grpids) == 0)
 			return EXIT_FAILURE;
 		}
 		if (xml_out) {
