@@ -1,7 +1,7 @@
 /* Copyright 2009, UCAR/Unidata and OPeNDAP, Inc.
    See the COPYRIGHT file for more information. */
 
-#include <config.h>
+#include "config.h"
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -75,7 +75,9 @@ int
 bbSetlength(Bytebuffer* bb, const unsigned int sz)
 {
   if(bb == NULL) return bbFail();
-  if(!bbSetalloc(bb,sz)) return bbFail();
+  if(bb->length < sz) {
+      if(!bbSetalloc(bb,sz)) return bbFail();
+  }
   bb->length = sz;
   return TRUE;
 }
