@@ -5,7 +5,7 @@
  *********************************************************************/
 
 #include "includes.h"
-#include "nciter.h"
+#include "nc_iter.h"
 #include <ctype.h>	/* for isprint() */
 
 #ifdef ENABLE_C
