@@ -7,7 +7,7 @@
 #include "includes.h"
 #include <ctype.h>	/* for isprint() */
 
-int derror_count;
+int error_count;
 
 #ifndef NO_STDARG
 #define vastart(argv,fmt) va_start(argv,fmt)
@@ -31,7 +31,7 @@ vderror(fmt,va_alist) const char* fmt; va_dcl
     (void) vfprintf(stderr,fmt,argv) ;
     (void) fputc('\n',stderr) ;
     (void) fflush(stderr);	/* to ensure log files are current */
-    derror_count++;
+    error_count++;
 }
 
 #ifndef NO_STDARG
@@ -64,11 +64,41 @@ verror(fmt,va_alist) const char* fmt; va_dcl
     vderror(newfmt,argv);
 }
 
+#ifndef NO_STDARG
+void
+semwarn(const int lno, const char *fmt, ...)
+#else
+void
+semwarn(lno,fmt,va_alist) const int lno; const char* fmt; va_dcl
+#endif
+{
+    va_list argv;
+    vastart(argv,fmt);
+    (void)fprintf(stderr,"%s: %s line %d: ", progname, cdlname, lno);
+    vderror(fmt,argv);
+}
+
+#ifndef NO_STDARG
+void
+semerror(const int lno, const char *fmt, ...)
+#else
+void
+semerror(lno,fmt,va_alist) const int lno; const char* fmt; va_dcl
+#endif
+{
+    va_list argv;
+    vastart(argv,fmt);
+    (void)fprintf(stderr,"%s: %s line %d: ", progname, cdlname, lno);
+    vderror(fmt,argv);
+    exit(1); /* immediately fatal */
+}
+
 /* Capture potential version errors */
 static char* markcdf4_msg = NULL;
 void
 markcdf4(const char* msg)
 {
+    enhanced_flag = 1;
     if(markcdf4_msg == NULL)
         markcdf4_msg = (char*)msg;
 }
