@@ -5,7 +5,7 @@
  *********************************************************************/
 
 #include "includes.h"
-#include "nciter.h"
+#include "nc_iter.h"
 
 #ifdef ENABLE_JAVA
 
