@@ -11,68 +11,58 @@
 #define TOPLEVEL 1
 
 /*Forward*/
-static Datalist* buildfill(Symbol* tsym);
 static void fill(Symbol* tsym, Datalist*);
 static void fillarray(Symbol* tsym, Dimset* dimset, int index, Datalist*);
 static void filllist(Symbol* tvsym, Datalist* dl);
-static int checkfill(Symbol* tvsym, Datasrc* src, Symbol*);
-static int checkarray(Symbol* tsym, Dimset* dimset, int index, Datasrc*, Symbol*, int);
-
 
 /* Construct a Datalist representing a complete fill value*/
-/* for a specified type*/
+/* for a specified variable */
 /* Cache if needed later*/
 
 Datalist*
-getfiller(Symbol* tvsym, Datalist* fillsrc)
+getfiller(Symbol* tvsym)
 {
-    Datalist* filler = fillsrc;
-    if(filler == NULL)
+    Datalist* filler = NULL;
+    Symbol* tsym = tvsym;
+    ASSERT(tvsym->objectclass == NC_VAR || tvsym->objectclass == NC_TYPE);
+    if(tvsym->objectclass == NC_VAR) {
+	tsym = tvsym->typ.basetype;
         filler = tvsym->var.special._Fillvalue; /* get cached value (if any)*/
-    if(filler == NULL) { /* need to compute it*/
-	filler = buildfill(tvsym);
-        tvsym->var.special._Fillvalue = filler; /* cache value*/
     }
-if(debug >= 1)
+    if(filler == NULL || tvsym->objectclass == NC_TYPE) {
+        filler = tvsym->typ._Fillvalue; /* get cached value (if any)*/
+    }
+    if(filler == NULL) { /* need to compute it*/
+        filler = builddatalist(0);
+	fill(tsym,filler);
+    }
+#ifdef DEBUG2
     dumpdatalist(filler,"getfiller");
+#endif
+    if(tvsym->objectclass == NC_VAR) {
+        tvsym->var.special._Fillvalue = filler;
+    } else if(tvsym->objectclass == NC_TYPE) {
+        tvsym->typ._Fillvalue = filler; /* cache value*/
+    }
     return filler;        
 }
 
-static Datalist*
-buildfill(Symbol* tvasym)
-{
-    Datalist* filler = builddatalist(0);
-    ASSERT(tvasym->objectclass == NC_VAR
-	   || tvasym->objectclass == NC_TYPE
-	   || tvasym->objectclass == NC_ATT);
-    if(tvasym->objectclass == NC_VAR) {
-	if(tvasym->typ.dimset.ndims > 0) {
-	    fillarray(tvasym->typ.basetype,&tvasym->typ.dimset,0,filler);
-	} else
-	    fill(tvasym->typ.basetype,filler);
-    } else if(tvasym->objectclass == NC_ATT) {
-	    fill(tvasym->typ.basetype,filler);
-    } else /*NC_TYPE*/
-	fill(tvasym,filler);
-    return filler;
-}
-
 static void
-fill(Symbol* tvsym, Datalist* filler)
+fill(Symbol* tsym, Datalist* filler)
 {
     int i;
     Constant con;
     Datalist* sublist;
-    /* NC_TYPE case*/
-    switch (tvsym->subclass) {
+    ASSERT(tsym->objectclass == NC_TYPE);
+    switch (tsym->subclass) {
     case NC_ENUM: case NC_OPAQUE: case NC_PRIM:
-        con.nctype = tvsym->typ.typecode;
+        con.nctype = tsym->typ.typecode;
         nc_getfill(&con);
 	break;
     case NC_COMPOUND:
-	sublist = builddatalist(listlength(tvsym->subnodes));
-        for(i=0;i<listlength(tvsym->subnodes);i++) {
-	    Symbol* field = (Symbol*)listget(tvsym->subnodes,i);
+	sublist = builddatalist(listlength(tsym->subnodes));
+        for(i=0;i<listlength(tsym->subnodes);i++) {
+	    Symbol* field = (Symbol*)listget(tsym->subnodes,i);
 	    if(field->typ.dimset.ndims > 0) {	
                 fillarray(field->typ.basetype,&field->typ.dimset,0,filler);
 	    } else
@@ -82,32 +72,32 @@ fill(Symbol* tvsym, Datalist* filler)
 	break;
     case NC_VLEN:
 	sublist = builddatalist(0);
-	filllist(tvsym->typ.basetype,sublist); /* generate a single instance*/
+	filllist(tsym->typ.basetype,sublist); /* generate a single instance*/
 	con = builddatasublist(sublist);
 	break;
-    default: PANIC1("fill: unexpected subclass %d",tvsym->subclass);
+    default: PANIC1("fill: unexpected subclass %d",tsym->subclass);
     }
     dlappend(filler,&con);
 }
 
 static void
-filllist(Symbol* tvsym, Datalist* dl)
+filllist(Symbol* tsym, Datalist* dl)
 {
     int i;
     Datalist* sublist;
     Constant con;
 
-    /* NC_TYPE case*/
-    switch (tvsym->subclass) {
+    ASSERT(tsym->objectclass == NC_TYPE);
+    switch (tsym->subclass) {
     case NC_ENUM: case NC_OPAQUE: case NC_PRIM:
-        con.nctype = tvsym->typ.typecode;
+        con.nctype = tsym->typ.typecode;
         nc_getfill(&con);
 	dlappend(dl,&con);
 	break;
     case NC_COMPOUND:
-	sublist = builddatalist(listlength(tvsym->subnodes));
-        for(i=0;i<listlength(tvsym->subnodes);i++) {
-	    Symbol* field = (Symbol*)listget(tvsym->subnodes,i);
+	sublist = builddatalist(listlength(tsym->subnodes));
+        for(i=0;i<listlength(tsym->subnodes);i++) {
+	    Symbol* field = (Symbol*)listget(tsym->subnodes,i);
 	    filllist(field->typ.basetype,sublist);
         }	  
 	con = builddatasublist(sublist);
@@ -115,11 +105,11 @@ filllist(Symbol* tvsym, Datalist* dl)
 	break;
     case NC_VLEN:
 	sublist = builddatalist(0);
-	filllist(tvsym->typ.basetype,sublist); /* generate a single instance*/
+	filllist(tsym->typ.basetype,sublist); /* generate a single instance*/
 	con = builddatasublist(sublist);
 	dlappend(dl,&con);
 	break;
-    default: PANIC1("fill: unexpected subclass %d",tvsym->subclass);
+    default: PANIC1("fill: unexpected subclass %d",tsym->subclass);
     }
 }
 
@@ -153,128 +143,6 @@ fillarray(Symbol* basetype, Dimset* dimset, int index, Datalist* arraylist)
     }
 }
 
-/* Verify that a user provided fill value does in fact*/
-/* conform to the structure of a given type or variable*/
-/* Structure conforms to fill procedures above.*/
-
-int
-checkfillvalue(Symbol* tvsym, Datalist* filler)
-{
-    Datasrc* src = datalist2src(filler);
-    int result;
-    ASSERT(tvsym->objectclass == NC_VAR || tvsym->objectclass == NC_TYPE);
-    if(tvsym->objectclass == NC_VAR) {
-	if(tvsym->typ.dimset.ndims > 0) {
-            result = checkarray(tvsym->typ.basetype,&tvsym->typ.dimset,0,src,tvsym,TOPLEVEL);
-	} else
-	    result = checkfill(tvsym->typ.basetype,src,tvsym);
-    } else /* NC_TYPE*/
-	result = checkfill(tvsym,src,tvsym);
-    freedatasrc(src);
-    return result;
-}
-
-static int
-checkfill(Symbol* tvsym, Datasrc* src, Symbol* original)
-{
-    int i,iscmpd,result;
-    Constant* con;
-
-    result = 1;
-    switch (tvsym->subclass) {
-    case NC_ENUM: case NC_OPAQUE: case NC_PRIM:
-	con = srcnext(src);
-	if(src == NULL) {
-	    semerror(srcline(src),"%s: malformed _FillValue",original->name);
-	    result = 0;
-	} else if(con->nctype != tvsym->typ.typecode) result = 0; /* wrong type*/
-	break;
-
-    case NC_COMPOUND:
-        if(!issublist(src)) {/* fail on no compound*/
-           semerror(srcline(src),"Compound constants must be enclosed in {..}");
-        }
-	srcpush(src);
-        for(i=0;i<listlength(tvsym->subnodes);i++) {
-	    Symbol* field = (Symbol*)listget(tvsym->subnodes,i);
-	    result = checkfill(field,src,original);
-	    if(!result) break;
-        }	  
-	srcpop(src);
-	break;
-
-    case NC_VLEN:
-	if(!issublist(src)) {
-	    semerror(srcline(src),"%s: vlen instances in _FillValue must be enclosed in {...}",original->name);
-	    result = 0;
-	} else {
-	    srcpush(src);
-            while(srcmore(src)) {
-		result = checkfill(tvsym->typ.basetype,src,original);
-		if(!result) break;
-	    }
-	    srcpop(src);
-	}
-	break;
-
-    case NC_FIELD:
-	/* Braces are optional */
-	if((iscmpd=issublist(src))) srcpush(src);
-	if(tvsym->typ.dimset.ndims > 0) {
-            result = checkarray(tvsym->typ.basetype,&tvsym->typ.dimset,0,src,original,!TOPLEVEL);
-	} else
-	    result = checkfill(tvsym->typ.basetype,src,original);
-	if(iscmpd) srcpop(src);
-	break;
-
-    default: PANIC1("checkfillvalue: unexpected subclass %d",tvsym->subclass);
-    }
-    return result;
-}
-
-static int
-checkarray(Symbol* basetype, Dimset* dimset, int index, Datasrc* src, Symbol* original, int toplevel)
-{
-    int i,result;
-    Symbol* dim = dimset->dimsyms[index];
-    unsigned int size = dim->dim.declsize;
-    int lastdim = (index == (dimset->ndims - 1));
-    int isunlimited = (size == 0);
-
-    result = 1;
-    if(isunlimited) {
-	if(!toplevel) {
-	    if(!issublist(src)) {
-		semerror(srcline(src),"UNLIMITED dimension constants (other than top level) must be enclosed in {...}");
-		result = 0;
-		goto done;
-	    } else
-		srcpush(src);
-	}
-        if(lastdim) {
-	    while(srcmore(src) && result) {
-		result = checkfill(basetype,src,original);
-	    }
-	} else { /*!lastdim*/
-	    while(srcmore(src) && result) {
-	        result = checkarray(basetype,dimset,index+1,src,original,toplevel);
-	    }
-	}
-    } else { /* bounded*/
-        if(lastdim) {
-	    for(i=0;i<size && result;i++) {
-		result = checkfill(basetype,src,original);
-	    }
-	} else { /* !lastdim*/
-	    for(i=0;i<size && result;i++) {
-	        result = checkarray(basetype,dimset,index+1,src,original,toplevel);
-	    }
-	}
-    }
-done:
-    return result;
-}
-
 /*
  * Given primitive netCDF type, return a default fill_value appropriate for
  * that type.
