@@ -223,7 +223,7 @@ processtypes(void)
 	}
 	if(keep) {
 	    sym->touched = 1;
-	    listpush(sorted,(elem_t)sym);
+	    listpush(sorted,(void*)sym);
 	}
     }	
     /* 2. repeated walk to collect level i types*/
@@ -253,7 +253,7 @@ processtypes(void)
 	    default: break;				
 	    }
 	    if(keep) {
-		listpush(sorted,(elem_t)sym);
+		listpush(sorted,(void*)sym);
 		sym->touched = 1;
 		added++;
 	    }	    
@@ -327,7 +327,7 @@ processenums(void)
 	for(j=0;j<listlength(sym->subnodes);j++) {
 	    Symbol* esym = (Symbol*)listget(sym->subnodes,j);
 	    ASSERT(esym->subclass == NC_ECONST);
-	    listpush(enumids,(elem_t)esym);
+	    listpush(enumids,(void*)esym);
 	}
     }	    
     /* Now walk set of enum ids to look for duplicates with same prefix*/
@@ -435,19 +435,16 @@ processvars(void)
     int i,j;
     for(i=0;i<listlength(vardefs);i++) {
 	Symbol* vsym = (Symbol*)listget(vardefs,i);
-	Symbol* tsym = vsym->typ.basetype;
+	Symbol* basetype = vsym->typ.basetype;
 	/* fill in the typecode*/
-	vsym->typ.typecode = tsym->typ.typecode;
-	for(j=0;j<tsym->typ.dimset.ndims;j++) {
-	    /* deref the dimensions*/
-	    tsym->typ.dimset.dimsyms[j] = tsym->typ.dimset.dimsyms[j];
-#ifndef USE_NETCDF4
-	    /* UNLIMITED must only be in first place*/
-	    if(tsym->typ.dimset.dimsyms[j]->dim.declsize == NC_UNLIMITED) {
-		if(j != 0)
+	vsym->typ.typecode = basetype->typ.typecode;
+	for(j=0;j<vsym->typ.dimset.ndims;j++) {
+	    /* validate the dimensions*/
+            /* UNLIMITED must only be in first place if using classic */
+	    if(vsym->typ.dimset.dimsyms[j]->dim.declsize == NC_UNLIMITED) {
+	        if(usingclassic && j != 0)
 		    semerror(vsym->lineno,"Variable: %s: UNLIMITED must be in first dimension only",fullname(vsym));
 	    }
-#endif
 	}	
     }
 }
@@ -467,6 +464,11 @@ processtypesizes(void)
     }
 }
 
+#ifdef NCF213 /*Jira NCF-213*/
+/* We should not create an actual
+   attribute for any of the
+   special attributes.
+*/
 static void
 makespecial(int tag, Symbol* vsym, nc_type typ, Datalist* dlist)
 {
@@ -475,13 +477,14 @@ makespecial(int tag, Symbol* vsym, nc_type typ, Datalist* dlist)
     attr->data = dlist;
     if(vsym) {
 	Symbol* grp = vsym->container;
-	if(grp) listpush(grp->subnodes,(elem_t)attr);
+	if(grp) listpush(grp->subnodes,(void*)attr);
 	attr->container = grp;
     }
     attr->att.var = vsym;
     attr->typ.basetype = primsymbols[typ==NC_STRING?NC_CHAR:typ];
-    listpush(attdefs,(elem_t)attr);
+    listpush(attdefs,(void*)attr);
 }
+#endif /*NCF213*/
 	
 static void
 processspecial1(Symbol* vsym)
@@ -489,7 +492,7 @@ processspecial1(Symbol* vsym)
     unsigned long flags = vsym->var.special.flags;
     int i,tag;
     Constant con;
-    Datalist* dlist;
+    Datalist* dlist = NULL;
     if(flags == 0) return; /* no specials defined */
     con = nullconstant;
     if((tag=(flags & _CHUNKSIZES_FLAG))) {
@@ -500,7 +503,6 @@ processspecial1(Symbol* vsym)
             con.value.int32v = (int)vsym->var.special._ChunkSizes[i];
             dlappend(dlist,&con);
         }
-        makespecial(tag,vsym,con.nctype,dlist);
     } else if((tag=(flags & _STORAGE_FLAG))) {
         con.nctype = NC_STRING;
         con.value.stringv.stringv
@@ -509,7 +511,6 @@ processspecial1(Symbol* vsym)
         con.value.stringv.len = strlen(con.value.stringv.stringv);
         dlist = builddatalist(1);
         dlappend(dlist,&con);
-        makespecial(tag,vsym,con.nctype,dlist);
     }
     if((tag=(flags & _FLETCHER32_FLAG))) {
         con.nctype = NC_STRING;
@@ -519,14 +520,12 @@ processspecial1(Symbol* vsym)
         con.value.stringv.len = strlen(con.value.stringv.stringv);
         dlist = builddatalist(1);
         dlappend(dlist,&con);
-        makespecial(tag,vsym,con.nctype,dlist);
     }
     if((tag=(flags & _DEFLATE_FLAG))) {
         con.nctype = NC_INT;
         con.value.int32v = vsym->var.special._DeflateLevel;
         dlist = builddatalist(1);
         dlappend(dlist,&con);
-        makespecial(tag,vsym,con.nctype,dlist);
     }
     if((tag=(flags & _SHUFFLE_FLAG))) {
         con.nctype = NC_STRING;
@@ -536,7 +535,6 @@ processspecial1(Symbol* vsym)
         con.value.stringv.len = strlen(con.value.stringv.stringv);
         dlist = builddatalist(1);
         dlappend(dlist,&con);
-        makespecial(tag,vsym,con.nctype,dlist);
     }
     if((tag=(flags & _ENDIAN_FLAG))) {
         con.nctype = NC_STRING;
@@ -546,7 +544,6 @@ processspecial1(Symbol* vsym)
         con.value.stringv.len = strlen(con.value.stringv.stringv);
         dlist = builddatalist(1);
         dlappend(dlist,&con);
-        makespecial(tag,vsym,con.nctype,dlist);
     }
     if((tag=(flags & _NOFILL_FLAG))) {
         con.nctype = NC_STRING;
@@ -560,8 +557,13 @@ processspecial1(Symbol* vsym)
         con.value.stringv.len = strlen(con.value.stringv.stringv);
         dlist = builddatalist(1);
         dlappend(dlist,&con);
-        makespecial(tag,vsym,con.nctype,dlist);
     }
+#ifdef NCF213 /*Jira NCF-213*/
+/* We should not create an actual
+   attribute for any of the
+   special attributes.
+*/
+#endif /*NCF213*/
 }
 
 static void
@@ -629,7 +631,7 @@ processattributes(void)
 	    Symbol* asym = (Symbol*)listget(attdefs,j);
 	    ASSERT(asym->att.var != NULL);
 	    if(asym->att.var != vsym) continue;	    
-            listpush(list,(elem_t)asym);
+            listpush(list,(void*)asym);
 	}
 	vsym->var.attributes = list;
     }
@@ -934,8 +936,10 @@ processunlimiteddims(void)
 	} else {
 	    for(i=0;i<var->data->length;i++) {
 	        Constant* con = var->data->data+i;
-	        ASSERT(con->nctype == NC_COMPOUND);
-	        computeunlimitedsizes(dimset,first,con->value.compoundv,ischar);
+	        if(con->nctype != NC_COMPOUND)
+		    semerror(con->lineno,"UNLIMITED dimension (other than first) must be enclosed in {}");
+		else
+	            computeunlimitedsizes(dimset,first,con->value.compoundv,ischar);
 	    }
 	}
     }
