@@ -1,9 +1,8 @@
-/* A Bison parser, made by GNU Bison 2.4.3.  */
+/* A Bison parser, made by GNU Bison 2.5.  */
 
-/* Skeleton implementation for Bison's Yacc-like parsers in C
+/* Bison implementation for Yacc-like parsers in C
    
-      Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006,
-   2009, 2010 Free Software Foundation, Inc.
+      Copyright (C) 1984, 1989-1990, 2000-2011 Free Software Foundation, Inc.
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -45,7 +44,7 @@
 #define YYBISON 1
 
 /* Bison version.  */
-#define YYBISON_VERSION "2.4.3"
+#define YYBISON_VERSION "2.5"
 
 /* Skeleton name.  */
 #define YYSKELETON_NAME "yacc.c"
@@ -74,16 +73,16 @@
 
 /* Copy the first part of user declarations.  */
 
-/* Line 189 of yacc.c  */
-#line 9 "ncgen.y"
+/* Line 268 of yacc.c  */
+#line 9 "/home/wfisher/Desktop/netcdf-cmake/ncgen3/ncgen.y"
 
 #ifdef sccs
 static char SccsId[] = "$Id: ncgen.y,v 1.34 2010/03/31 18:18:41 dmh Exp $";
 #endif
-#include        <config.h>
+#include        "config.h"
 #include        <string.h>
 #include	<stdlib.h>
-#include	<netcdf.h>
+#include	"netcdf.h"
 #include 	"generic.h"
 #include        "ncgen.h"
 #include	"genlib.h"	/* for grow_darray() et al */
@@ -149,8 +148,8 @@ int yyerror(char*);
 #endif
 
 
-/* Line 189 of yacc.c  */
-#line 154 "ncgen.tab.c"
+/* Line 268 of yacc.c  */
+#line 153 "ncgen.tab.c"
 
 /* Enabling traces.  */
 #ifndef YYDEBUG
@@ -213,8 +212,8 @@ typedef int YYSTYPE;
 /* Copy the second part of user declarations.  */
 
 
-/* Line 264 of yacc.c  */
-#line 218 "ncgen.tab.c"
+/* Line 343 of yacc.c  */
+#line 217 "ncgen.tab.c"
 
 #ifdef short
 # undef short
@@ -317,11 +316,11 @@ YYID (yyi)
 #    define alloca _alloca
 #   else
 #    define YYSTACK_ALLOC alloca
-#    if ! defined _ALLOCA_H && ! defined _STDLIB_H && (defined __STDC__ || defined __C99__FUNC__ \
+#    if ! defined _ALLOCA_H && ! defined EXIT_SUCCESS && (defined __STDC__ || defined __C99__FUNC__ \
      || defined __cplusplus || defined _MSC_VER)
 #     include <stdlib.h> /* INFRINGES ON USER NAME SPACE */
-#     ifndef _STDLIB_H
-#      define _STDLIB_H 1
+#     ifndef EXIT_SUCCESS
+#      define EXIT_SUCCESS 0
 #     endif
 #    endif
 #   endif
@@ -344,24 +343,24 @@ YYID (yyi)
 #  ifndef YYSTACK_ALLOC_MAXIMUM
 #   define YYSTACK_ALLOC_MAXIMUM YYSIZE_MAXIMUM
 #  endif
-#  if (defined __cplusplus && ! defined _STDLIB_H \
+#  if (defined __cplusplus && ! defined EXIT_SUCCESS \
        && ! ((defined YYMALLOC || defined malloc) \
 	     && (defined YYFREE || defined free)))
 #   include <stdlib.h> /* INFRINGES ON USER NAME SPACE */
-#   ifndef _STDLIB_H
-#    define _STDLIB_H 1
+#   ifndef EXIT_SUCCESS
+#    define EXIT_SUCCESS 0
 #   endif
 #  endif
 #  ifndef YYMALLOC
 #   define YYMALLOC malloc
-#   if ! defined malloc && ! defined _STDLIB_H && (defined __STDC__ || defined __C99__FUNC__ \
+#   if ! defined malloc && ! defined EXIT_SUCCESS && (defined __STDC__ || defined __C99__FUNC__ \
      || defined __cplusplus || defined _MSC_VER)
 void *malloc (YYSIZE_T); /* INFRINGES ON USER NAME SPACE */
 #   endif
 #  endif
 #  ifndef YYFREE
 #   define YYFREE free
-#   if ! defined free && ! defined _STDLIB_H && (defined __STDC__ || defined __C99__FUNC__ \
+#   if ! defined free && ! defined EXIT_SUCCESS && (defined __STDC__ || defined __C99__FUNC__ \
      || defined __cplusplus || defined _MSC_VER)
 void free (void *); /* INFRINGES ON USER NAME SPACE */
 #   endif
@@ -390,23 +389,7 @@ union yyalloc
      ((N) * (sizeof (yytype_int16) + sizeof (YYSTYPE)) \
       + YYSTACK_GAP_MAXIMUM)
 
-/* Copy COUNT objects from FROM to TO.  The source and destination do
-   not overlap.  */
-# ifndef YYCOPY
-#  if defined __GNUC__ && 1 < __GNUC__
-#   define YYCOPY(To, From, Count) \
-      __builtin_memcpy (To, From, (Count) * sizeof (*(From)))
-#  else
-#   define YYCOPY(To, From, Count)		\
-      do					\
-	{					\
-	  YYSIZE_T yyi;				\
-	  for (yyi = 0; yyi < (Count); yyi++)	\
-	    (To)[yyi] = (From)[yyi];		\
-	}					\
-      while (YYID (0))
-#  endif
-# endif
+# define YYCOPY_NEEDED 1
 
 /* Relocate STACK from its old location to the new one.  The
    local variables YYSIZE and YYSTACKSIZE give the old and new number of
@@ -426,6 +409,26 @@ union yyalloc
 
 #endif
 
+#if defined YYCOPY_NEEDED && YYCOPY_NEEDED
+/* Copy COUNT objects from FROM to TO.  The source and destination do
+   not overlap.  */
+# ifndef YYCOPY
+#  if defined __GNUC__ && 1 < __GNUC__
+#   define YYCOPY(To, From, Count) \
+      __builtin_memcpy (To, From, (Count) * sizeof (*(From)))
+#  else
+#   define YYCOPY(To, From, Count)		\
+      do					\
+	{					\
+	  YYSIZE_T yyi;				\
+	  for (yyi = 0; yyi < (Count); yyi++)	\
+	    (To)[yyi] = (From)[yyi];		\
+	}					\
+      while (YYID (0))
+#  endif
+# endif
+#endif /* !YYCOPY_NEEDED */
+
 /* YYFINAL -- State number of the termination state.  */
 #define YYFINAL  4
 /* YYLAST -- Last index in YYTABLE.  */
@@ -592,8 +595,8 @@ static const yytype_uint8 yyr2[] =
        0,     2,     1,     1,     1,     1,     1,     1,     1,     1
 };
 
-/* YYDEFACT[STATE-NAME] -- Default rule to reduce with in state
-   STATE-NUM when YYTABLE doesn't specify something else to do.  Zero
+/* YYDEFACT[STATE-NAME] -- Default reduction number in state STATE-NUM.
+   Performed when YYTABLE doesn't specify something else to do.  Zero
    means the default is an error.  */
 static const yytype_uint8 yydefact[] =
 {
@@ -652,8 +655,7 @@ static const yytype_int8 yypgoto[] =
 
 /* YYTABLE[YYPACT[STATE-NUM]].  What to do in state STATE-NUM.  If
    positive, shift that token.  If negative, reduce the rule which
-   number is the opposite.  If zero, do what YYDEFACT says.
-   If YYTABLE_NINF, syntax error.  */
+   number is the opposite.  If YYTABLE_NINF, syntax error.  */
 #define YYTABLE_NINF -1
 static const yytype_uint8 yytable[] =
 {
@@ -667,6 +669,12 @@ static const yytype_uint8 yytable[] =
       93,    66,   101,    94,   111,     0,    86,     0,    51
 };
 
+#define yypact_value_is_default(yystate) \
+  ((yystate) == (-73))
+
+#define yytable_value_is_error(yytable_value) \
+  YYID (0)
+
 static const yytype_int8 yycheck[] =
 {
       34,     4,     5,     6,     7,     8,     9,    10,     3,    26,
@@ -730,7 +738,6 @@ do								\
     {								\
       yychar = (Token);						\
       yylval = (Value);						\
-      yytoken = YYTRANSLATE (yychar);				\
       YYPOPSTACK (1);						\
       goto yybackup;						\
     }								\
@@ -772,19 +779,10 @@ while (YYID (0))
 #endif
 
 
-/* YY_LOCATION_PRINT -- Print the location on the stream.
-   This macro was not mandated originally: define only if we know
-   we won't break user code: when these are the locations we know.  */
+/* This macro is provided for backward compatibility. */
 
 #ifndef YY_LOCATION_PRINT
-# if defined YYLTYPE_IS_TRIVIAL && YYLTYPE_IS_TRIVIAL
-#  define YY_LOCATION_PRINT(File, Loc)			\
-     fprintf (File, "%d.%d-%d.%d",			\
-	      (Loc).first_line, (Loc).first_column,	\
-	      (Loc).last_line,  (Loc).last_column)
-# else
-#  define YY_LOCATION_PRINT(File, Loc) ((void) 0)
-# endif
+# define YY_LOCATION_PRINT(File, Loc) ((void) 0)
 #endif
 
 
@@ -976,7 +974,6 @@ int yydebug;
 # define YYMAXDEPTH 10000
 #endif
 
-
 
 #if YYERROR_VERBOSE
 
@@ -1079,115 +1076,142 @@ yytnamerr (char *yyres, const char *yystr)
 }
 # endif
 
-/* Copy into YYRESULT an error message about the unexpected token
-   YYCHAR while in state YYSTATE.  Return the number of bytes copied,
-   including the terminating null byte.  If YYRESULT is null, do not
-   copy anything; just return the number of bytes that would be
-   copied.  As a special case, return 0 if an ordinary "syntax error"
-   message will do.  Return YYSIZE_MAXIMUM if overflow occurs during
-   size calculation.  */
-static YYSIZE_T
-yysyntax_error (char *yyresult, int yystate, int yychar)
+/* Copy into *YYMSG, which is of size *YYMSG_ALLOC, an error message
+   about the unexpected token YYTOKEN for the state stack whose top is
+   YYSSP.
+
+   Return 0 if *YYMSG was successfully written.  Return 1 if *YYMSG is
+   not large enough to hold the message.  In that case, also set
+   *YYMSG_ALLOC to the required number of bytes.  Return 2 if the
+   required number of bytes is too large to store.  */
+static int
+yysyntax_error (YYSIZE_T *yymsg_alloc, char **yymsg,
+                yytype_int16 *yyssp, int yytoken)
 {
-  int yyn = yypact[yystate];
+  YYSIZE_T yysize0 = yytnamerr (0, yytname[yytoken]);
+  YYSIZE_T yysize = yysize0;
+  YYSIZE_T yysize1;
+  enum { YYERROR_VERBOSE_ARGS_MAXIMUM = 5 };
+  /* Internationalized format string. */
+  const char *yyformat = 0;
+  /* Arguments of yyformat. */
+  char const *yyarg[YYERROR_VERBOSE_ARGS_MAXIMUM];
+  /* Number of reported tokens (one for the "unexpected", one per
+     "expected"). */
+  int yycount = 0;
 
-  if (! (YYPACT_NINF < yyn && yyn <= YYLAST))
-    return 0;
-  else
+  /* There are many possibilities here to consider:
+     - Assume YYFAIL is not used.  It's too flawed to consider.  See
+       <http://lists.gnu.org/archive/html/bison-patches/2009-12/msg00024.html>
+       for details.  YYERROR is fine as it does not invoke this
+       function.
+     - If this state is a consistent state with a default action, then
+       the only way this function was invoked is if the default action
+       is an error action.  In that case, don't check for expected
+       tokens because there are none.
+     - The only way there can be no lookahead present (in yychar) is if
+       this state is a consistent state with a default action.  Thus,
+       detecting the absence of a lookahead is sufficient to determine
+       that there is no unexpected or expected token to report.  In that
+       case, just report a simple "syntax error".
+     - Don't assume there isn't a lookahead just because this state is a
+       consistent state with a default action.  There might have been a
+       previous inconsistent state, consistent state with a non-default
+       action, or user semantic action that manipulated yychar.
+     - Of course, the expected token list depends on states to have
+       correct lookahead information, and it depends on the parser not
+       to perform extra reductions after fetching a lookahead from the
+       scanner and before detecting a syntax error.  Thus, state merging
+       (from LALR or IELR) and default reductions corrupt the expected
+       token list.  However, the list is correct for canonical LR with
+       one exception: it will still contain any token that will not be
+       accepted due to an error action in a later state.
+  */
+  if (yytoken != YYEMPTY)
     {
-      int yytype = YYTRANSLATE (yychar);
-      YYSIZE_T yysize0 = yytnamerr (0, yytname[yytype]);
-      YYSIZE_T yysize = yysize0;
-      YYSIZE_T yysize1;
-      int yysize_overflow = 0;
-      enum { YYERROR_VERBOSE_ARGS_MAXIMUM = 5 };
-      char const *yyarg[YYERROR_VERBOSE_ARGS_MAXIMUM];
-      int yyx;
+      int yyn = yypact[*yyssp];
+      yyarg[yycount++] = yytname[yytoken];
+      if (!yypact_value_is_default (yyn))
+        {
+          /* Start YYX at -YYN if negative to avoid negative indexes in
+             YYCHECK.  In other words, skip the first -YYN actions for
+             this state because they are default actions.  */
+          int yyxbegin = yyn < 0 ? -yyn : 0;
+          /* Stay within bounds of both yycheck and yytname.  */
+          int yychecklim = YYLAST - yyn + 1;
+          int yyxend = yychecklim < YYNTOKENS ? yychecklim : YYNTOKENS;
+          int yyx;
 
-# if 0
-      /* This is so xgettext sees the translatable formats that are
-	 constructed on the fly.  */
-      YY_("syntax error, unexpected %s");
-      YY_("syntax error, unexpected %s, expecting %s");
-      YY_("syntax error, unexpected %s, expecting %s or %s");
-      YY_("syntax error, unexpected %s, expecting %s or %s or %s");
-      YY_("syntax error, unexpected %s, expecting %s or %s or %s or %s");
-# endif
-      char *yyfmt;
-      char const *yyf;
-      static char const yyunexpected[] = "syntax error, unexpected %s";
-      static char const yyexpecting[] = ", expecting %s";
-      static char const yyor[] = " or %s";
-      char yyformat[sizeof yyunexpected
-		    + sizeof yyexpecting - 1
-		    + ((YYERROR_VERBOSE_ARGS_MAXIMUM - 2)
-		       * (sizeof yyor - 1))];
-      char const *yyprefix = yyexpecting;
-
-      /* Start YYX at -YYN if negative to avoid negative indexes in
-	 YYCHECK.  */
-      int yyxbegin = yyn < 0 ? -yyn : 0;
-
-      /* Stay within bounds of both yycheck and yytname.  */
-      int yychecklim = YYLAST - yyn + 1;
-      int yyxend = yychecklim < YYNTOKENS ? yychecklim : YYNTOKENS;
-      int yycount = 1;
-
-      yyarg[0] = yytname[yytype];
-      yyfmt = yystpcpy (yyformat, yyunexpected);
-
-      for (yyx = yyxbegin; yyx < yyxend; ++yyx)
-	if (yycheck[yyx + yyn] == yyx && yyx != YYTERROR)
-	  {
-	    if (yycount == YYERROR_VERBOSE_ARGS_MAXIMUM)
-	      {
-		yycount = 1;
-		yysize = yysize0;
-		yyformat[sizeof yyunexpected - 1] = '\0';
-		break;
-	      }
-	    yyarg[yycount++] = yytname[yyx];
-	    yysize1 = yysize + yytnamerr (0, yytname[yyx]);
-	    yysize_overflow |= (yysize1 < yysize);
-	    yysize = yysize1;
-	    yyfmt = yystpcpy (yyfmt, yyprefix);
-	    yyprefix = yyor;
-	  }
-
-      yyf = YY_(yyformat);
-      yysize1 = yysize + yystrlen (yyf);
-      yysize_overflow |= (yysize1 < yysize);
-      yysize = yysize1;
-
-      if (yysize_overflow)
-	return YYSIZE_MAXIMUM;
-
-      if (yyresult)
-	{
-	  /* Avoid sprintf, as that infringes on the user's name space.
-	     Don't have undefined behavior even if the translation
-	     produced a string with the wrong number of "%s"s.  */
-	  char *yyp = yyresult;
-	  int yyi = 0;
-	  while ((*yyp = *yyf) != '\0')
-	    {
-	      if (*yyp == '%' && yyf[1] == 's' && yyi < yycount)
-		{
-		  yyp += yytnamerr (yyp, yyarg[yyi++]);
-		  yyf += 2;
-		}
-	      else
-		{
-		  yyp++;
-		  yyf++;
-		}
-	    }
-	}
-      return yysize;
+          for (yyx = yyxbegin; yyx < yyxend; ++yyx)
+            if (yycheck[yyx + yyn] == yyx && yyx != YYTERROR
+                && !yytable_value_is_error (yytable[yyx + yyn]))
+              {
+                if (yycount == YYERROR_VERBOSE_ARGS_MAXIMUM)
+                  {
+                    yycount = 1;
+                    yysize = yysize0;
+                    break;
+                  }
+                yyarg[yycount++] = yytname[yyx];
+                yysize1 = yysize + yytnamerr (0, yytname[yyx]);
+                if (! (yysize <= yysize1
+                       && yysize1 <= YYSTACK_ALLOC_MAXIMUM))
+                  return 2;
+                yysize = yysize1;
+              }
+        }
     }
+
+  switch (yycount)
+    {
+# define YYCASE_(N, S)                      \
+      case N:                               \
+        yyformat = S;                       \
+      break
+      YYCASE_(0, YY_("syntax error"));
+      YYCASE_(1, YY_("syntax error, unexpected %s"));
+      YYCASE_(2, YY_("syntax error, unexpected %s, expecting %s"));
+      YYCASE_(3, YY_("syntax error, unexpected %s, expecting %s or %s"));
+      YYCASE_(4, YY_("syntax error, unexpected %s, expecting %s or %s or %s"));
+      YYCASE_(5, YY_("syntax error, unexpected %s, expecting %s or %s or %s or %s"));
+# undef YYCASE_
+    }
+
+  yysize1 = yysize + yystrlen (yyformat);
+  if (! (yysize <= yysize1 && yysize1 <= YYSTACK_ALLOC_MAXIMUM))
+    return 2;
+  yysize = yysize1;
+
+  if (*yymsg_alloc < yysize)
+    {
+      *yymsg_alloc = 2 * yysize;
+      if (! (yysize <= *yymsg_alloc
+             && *yymsg_alloc <= YYSTACK_ALLOC_MAXIMUM))
+        *yymsg_alloc = YYSTACK_ALLOC_MAXIMUM;
+      return 1;
+    }
+
+  /* Avoid sprintf, as that infringes on the user's name space.
+     Don't have undefined behavior even if the translation
+     produced a string with the wrong number of "%s"s.  */
+  {
+    char *yyp = *yymsg;
+    int yyi = 0;
+    while ((*yyp = *yyformat) != '\0')
+      if (*yyp == '%' && yyformat[1] == 's' && yyi < yycount)
+        {
+          yyp += yytnamerr (yyp, yyarg[yyi++]);
+          yyformat += 2;
+        }
+      else
+        {
+          yyp++;
+          yyformat++;
+        }
+  }
+  return 0;
 }
 #endif /* YYERROR_VERBOSE */
-
 
 /*-----------------------------------------------.
 | Release the memory associated to this symbol.  |
@@ -1220,6 +1244,7 @@ yydestruct (yymsg, yytype, yyvaluep)
     }
 }
 
+
 /* Prevent warnings from -Wmissing-prototypes.  */
 #ifdef YYPARSE_PARAM
 #if defined __STDC__ || defined __cplusplus
@@ -1246,10 +1271,9 @@ YYSTYPE yylval;
 int yynerrs;
 
 
-
-/*-------------------------.
-| yyparse or yypush_parse.  |
-`-------------------------*/
+/*----------.
+| yyparse.  |
+`----------*/
 
 #ifdef YYPARSE_PARAM
 #if (defined __STDC__ || defined __C99__FUNC__ \
@@ -1273,8 +1297,6 @@ yyparse ()
 #endif
 #endif
 {
-
-
     int yystate;
     /* Number of tokens to shift before error messages enabled.  */
     int yyerrstatus;
@@ -1429,7 +1451,7 @@ yybackup:
 
   /* First try to decide what to do without reference to lookahead token.  */
   yyn = yypact[yystate];
-  if (yyn == YYPACT_NINF)
+  if (yypact_value_is_default (yyn))
     goto yydefault;
 
   /* Not known => get a lookahead token if don't already have one.  */
@@ -1460,8 +1482,8 @@ yybackup:
   yyn = yytable[yyn];
   if (yyn <= 0)
     {
-      if (yyn == 0 || yyn == YYTABLE_NINF)
-	goto yyerrlab;
+      if (yytable_value_is_error (yyn))
+        goto yyerrlab;
       yyn = -yyn;
       goto yyreduce;
     }
@@ -1516,48 +1538,48 @@ yyreduce:
     {
         case 2:
 
-/* Line 1464 of yacc.c  */
-#line 114 "ncgen.y"
-    { init_netcdf(); ;}
+/* Line 1806 of yacc.c  */
+#line 114 "/home/wfisher/Desktop/netcdf-cmake/ncgen3/ncgen.y"
+    { init_netcdf(); }
     break;
 
   case 3:
 
-/* Line 1464 of yacc.c  */
-#line 117 "ncgen.y"
+/* Line 1806 of yacc.c  */
+#line 117 "/home/wfisher/Desktop/netcdf-cmake/ncgen3/ncgen.y"
     {
 		       if (derror_count == 0)
 			 define_netcdf(netcdfname);
 		       if (derror_count > 0)
 			   exit(6);
-		   ;}
+		   }
     break;
 
   case 4:
 
-/* Line 1464 of yacc.c  */
-#line 125 "ncgen.y"
+/* Line 1806 of yacc.c  */
+#line 125 "/home/wfisher/Desktop/netcdf-cmake/ncgen3/ncgen.y"
     {
 		       if (derror_count == 0)
 			 close_netcdf();
-		   ;}
+		   }
     break;
 
   case 11:
 
-/* Line 1464 of yacc.c  */
-#line 140 "ncgen.y"
+/* Line 1806 of yacc.c  */
+#line 140 "/home/wfisher/Desktop/netcdf-cmake/ncgen3/ncgen.y"
     { if (int_val <= 0)
 			 derror("dimension length must be positive");
 		     dims[ndims].size = int_val;
 		     ndims++;
-		   ;}
+		   }
     break;
 
   case 12:
 
-/* Line 1464 of yacc.c  */
-#line 146 "ncgen.y"
+/* Line 1806 of yacc.c  */
+#line 146 "/home/wfisher/Desktop/netcdf-cmake/ncgen3/ncgen.y"
     { /* for rare case where 2^31 < dimsize < 2^32 */
 		       if (double_val <= 0)
 			 derror("dimension length must be positive");
@@ -1567,25 +1589,25 @@ yyreduce:
 			 derror("dimension length must be an integer");
 		       dims[ndims].size = (size_t) double_val;
 		       ndims++;
-                   ;}
+                   }
     break;
 
   case 13:
 
-/* Line 1464 of yacc.c  */
-#line 157 "ncgen.y"
+/* Line 1806 of yacc.c  */
+#line 157 "/home/wfisher/Desktop/netcdf-cmake/ncgen3/ncgen.y"
     {  if (rec_dim != -1)
 			 derror("only one NC_UNLIMITED dimension allowed");
 		     rec_dim = ndims; /* the unlimited (record) dimension */
 		     dims[ndims].size = NC_UNLIMITED;
 		     ndims++;
-		   ;}
+		   }
     break;
 
   case 14:
 
-/* Line 1464 of yacc.c  */
-#line 165 "ncgen.y"
+/* Line 1806 of yacc.c  */
+#line 165 "/home/wfisher/Desktop/netcdf-cmake/ncgen3/ncgen.y"
     { 
 		    if ((yyvsp[(1) - (1)])->is_dim == 1) {
 		        derror( "duplicate dimension declaration for %s",
@@ -1600,55 +1622,55 @@ yyreduce:
 		     (void) strcpy(dims[ndims].name, (yyvsp[(1) - (1)])->name);
 		     /* name for use in generated Fortran and C variables */
 		     dims[ndims].lname = decodify((yyvsp[(1) - (1)])->name);
-		   ;}
+		   }
     break;
 
   case 27:
 
-/* Line 1464 of yacc.c  */
-#line 197 "ncgen.y"
-    { type_code = NC_BYTE; ;}
+/* Line 1806 of yacc.c  */
+#line 197 "/home/wfisher/Desktop/netcdf-cmake/ncgen3/ncgen.y"
+    { type_code = NC_BYTE; }
     break;
 
   case 28:
 
-/* Line 1464 of yacc.c  */
-#line 198 "ncgen.y"
-    { type_code = NC_CHAR; ;}
+/* Line 1806 of yacc.c  */
+#line 198 "/home/wfisher/Desktop/netcdf-cmake/ncgen3/ncgen.y"
+    { type_code = NC_CHAR; }
     break;
 
   case 29:
 
-/* Line 1464 of yacc.c  */
-#line 199 "ncgen.y"
-    { type_code = NC_SHORT; ;}
+/* Line 1806 of yacc.c  */
+#line 199 "/home/wfisher/Desktop/netcdf-cmake/ncgen3/ncgen.y"
+    { type_code = NC_SHORT; }
     break;
 
   case 30:
 
-/* Line 1464 of yacc.c  */
-#line 200 "ncgen.y"
-    { type_code = NC_INT; ;}
+/* Line 1806 of yacc.c  */
+#line 200 "/home/wfisher/Desktop/netcdf-cmake/ncgen3/ncgen.y"
+    { type_code = NC_INT; }
     break;
 
   case 31:
 
-/* Line 1464 of yacc.c  */
-#line 201 "ncgen.y"
-    { type_code = NC_FLOAT; ;}
+/* Line 1806 of yacc.c  */
+#line 201 "/home/wfisher/Desktop/netcdf-cmake/ncgen3/ncgen.y"
+    { type_code = NC_FLOAT; }
     break;
 
   case 32:
 
-/* Line 1464 of yacc.c  */
-#line 202 "ncgen.y"
-    { type_code = NC_DOUBLE; ;}
+/* Line 1806 of yacc.c  */
+#line 202 "/home/wfisher/Desktop/netcdf-cmake/ncgen3/ncgen.y"
+    { type_code = NC_DOUBLE; }
     break;
 
   case 35:
 
-/* Line 1464 of yacc.c  */
-#line 208 "ncgen.y"
+/* Line 1806 of yacc.c  */
+#line 208 "/home/wfisher/Desktop/netcdf-cmake/ncgen3/ncgen.y"
     {
 		    static struct vars dummyvar;
 
@@ -1680,23 +1702,23 @@ yyreduce:
 		     * the variable attribute "_FillValue". */
 		    nc_getfill(type_code, &vars[nvars].fill_value);
 		    vars[nvars].has_data = 0; /* has no data (yet) */
-		   ;}
+		   }
     break;
 
   case 36:
 
-/* Line 1464 of yacc.c  */
-#line 241 "ncgen.y"
+/* Line 1806 of yacc.c  */
+#line 241 "/home/wfisher/Desktop/netcdf-cmake/ncgen3/ncgen.y"
     {
 		    vars[nvars].ndims = nvdims;
 		    nvars++;
-		   ;}
+		   }
     break;
 
   case 42:
 
-/* Line 1464 of yacc.c  */
-#line 255 "ncgen.y"
+/* Line 1806 of yacc.c  */
+#line 255 "/home/wfisher/Desktop/netcdf-cmake/ncgen3/ncgen.y"
     {
 		    if (nvdims >= NC_MAX_VAR_DIMS) {
 		       derror("%s has too many dimensions",vars[nvars].name);
@@ -1715,58 +1737,58 @@ yyreduce:
 				&vars[nvars].dims); /* grow as needed */
 		    vars[nvars].dims[nvdims] = dimnum;
                     nvdims++;
-		   ;}
+		   }
     break;
 
   case 43:
 
-/* Line 1464 of yacc.c  */
-#line 276 "ncgen.y"
+/* Line 1806 of yacc.c  */
+#line 276 "/home/wfisher/Desktop/netcdf-cmake/ncgen3/ncgen.y"
     {
                    defatt();
-		   ;}
+		   }
     break;
 
   case 44:
 
-/* Line 1464 of yacc.c  */
-#line 280 "ncgen.y"
+/* Line 1806 of yacc.c  */
+#line 280 "/home/wfisher/Desktop/netcdf-cmake/ncgen3/ncgen.y"
     {
                    equalatt();
-		   ;}
+		   }
     break;
 
   case 45:
 
-/* Line 1464 of yacc.c  */
-#line 285 "ncgen.y"
+/* Line 1806 of yacc.c  */
+#line 285 "/home/wfisher/Desktop/netcdf-cmake/ncgen3/ncgen.y"
     {
                    defatt();
-		   ;}
+		   }
     break;
 
   case 46:
 
-/* Line 1464 of yacc.c  */
-#line 289 "ncgen.y"
+/* Line 1806 of yacc.c  */
+#line 289 "/home/wfisher/Desktop/netcdf-cmake/ncgen3/ncgen.y"
     {
                    equalatt();
-		   ;}
+		   }
     break;
 
   case 48:
 
-/* Line 1464 of yacc.c  */
-#line 297 "ncgen.y"
+/* Line 1806 of yacc.c  */
+#line 297 "/home/wfisher/Desktop/netcdf-cmake/ncgen3/ncgen.y"
     {
 		    varnum = NC_GLOBAL;  /* handle of "global" attribute */
-		   ;}
+		   }
     break;
 
   case 49:
 
-/* Line 1464 of yacc.c  */
-#line 303 "ncgen.y"
+/* Line 1806 of yacc.c  */
+#line 303 "/home/wfisher/Desktop/netcdf-cmake/ncgen3/ncgen.y"
     { if ((yyvsp[(1) - (1)])->is_var == 1)
 		       varnum = (yyvsp[(1) - (1)])->vnum;
 		    else {
@@ -1774,13 +1796,13 @@ yyreduce:
 			     (yyvsp[(1) - (1)])->name);
 		      YYABORT;
 		      }
-		   ;}
+		   }
     break;
 
   case 50:
 
-/* Line 1464 of yacc.c  */
-#line 313 "ncgen.y"
+/* Line 1806 of yacc.c  */
+#line 313 "/home/wfisher/Desktop/netcdf-cmake/ncgen3/ncgen.y"
     {
 		       /* make sure atts array will hold attributes */
 		       grow_aarray(natts,  /* must hold natts+1 atts */
@@ -1789,36 +1811,36 @@ yyreduce:
 		       (void) strcpy(atts[natts].name,(yyvsp[(1) - (1)])->name);
 		       /* name for use in generated Fortran and C variables */
 		       atts[natts].lname = decodify((yyvsp[(1) - (1)])->name);
-		   ;}
+		   }
     break;
 
   case 53:
 
-/* Line 1464 of yacc.c  */
-#line 327 "ncgen.y"
+/* Line 1806 of yacc.c  */
+#line 327 "/home/wfisher/Desktop/netcdf-cmake/ncgen3/ncgen.y"
     {
 		    if (valtype == NC_UNSPECIFIED)
 		      valtype = atype_code;
 		    if (valtype != atype_code)
 		      derror("values for attribute must be all of same type");
-		   ;}
+		   }
     break;
 
   case 54:
 
-/* Line 1464 of yacc.c  */
-#line 336 "ncgen.y"
+/* Line 1806 of yacc.c  */
+#line 336 "/home/wfisher/Desktop/netcdf-cmake/ncgen3/ncgen.y"
     {
 		       atype_code = NC_CHAR;
 		       *char_valp++ = char_val;
 		       valnum++;
-		   ;}
+		   }
     break;
 
   case 55:
 
-/* Line 1464 of yacc.c  */
-#line 342 "ncgen.y"
+/* Line 1806 of yacc.c  */
+#line 342 "/home/wfisher/Desktop/netcdf-cmake/ncgen3/ncgen.y"
     {
 		       atype_code = NC_CHAR;
 		       {
@@ -1830,68 +1852,68 @@ yyreduce:
 			   valnum += len;
 			   char_valp += len;
 		       }
-		   ;}
+		   }
     break;
 
   case 56:
 
-/* Line 1464 of yacc.c  */
-#line 355 "ncgen.y"
+/* Line 1806 of yacc.c  */
+#line 355 "/home/wfisher/Desktop/netcdf-cmake/ncgen3/ncgen.y"
     {
 		       atype_code = NC_BYTE;
 		       *byte_valp++ = byte_val;
 		       valnum++;
-		   ;}
+		   }
     break;
 
   case 57:
 
-/* Line 1464 of yacc.c  */
-#line 361 "ncgen.y"
+/* Line 1806 of yacc.c  */
+#line 361 "/home/wfisher/Desktop/netcdf-cmake/ncgen3/ncgen.y"
     {
 		       atype_code = NC_SHORT;
 		       *short_valp++ = short_val;
 		       valnum++;
-		   ;}
+		   }
     break;
 
   case 58:
 
-/* Line 1464 of yacc.c  */
-#line 367 "ncgen.y"
+/* Line 1806 of yacc.c  */
+#line 367 "/home/wfisher/Desktop/netcdf-cmake/ncgen3/ncgen.y"
     {
 		       atype_code = NC_INT;
 		       *int_valp++ = int_val;
 		       valnum++;
-		   ;}
+		   }
     break;
 
   case 59:
 
-/* Line 1464 of yacc.c  */
-#line 373 "ncgen.y"
+/* Line 1806 of yacc.c  */
+#line 373 "/home/wfisher/Desktop/netcdf-cmake/ncgen3/ncgen.y"
     {
 		       atype_code = NC_FLOAT;
 		       *float_valp++ = float_val;
 		       valnum++;
-		   ;}
+		   }
     break;
 
   case 60:
 
-/* Line 1464 of yacc.c  */
-#line 379 "ncgen.y"
+/* Line 1806 of yacc.c  */
+#line 379 "/home/wfisher/Desktop/netcdf-cmake/ncgen3/ncgen.y"
     {
 		       atype_code = NC_DOUBLE;
 		       *double_valp++ = double_val;
 		       valnum++;
-		   ;}
+		   }
     break;
 
   case 66:
 
-/* Line 1464 of yacc.c  */
-#line 395 "ncgen.y"
+/* Line 1806 of yacc.c  */
+#line 395 "/home/wfisher/Desktop/netcdf-cmake/ncgen3/ncgen.y"
     {
 		       valtype = vars[varnum].type; /* variable type */
 		       valnum = 0;	/* values accumulated for variable */
@@ -1941,13 +1963,13 @@ yyreduce:
 			   break;
 			 default: break;
 		       }
-		 ;}
+		 }
     break;
 
   case 67:
 
-/* Line 1464 of yacc.c  */
-#line 446 "ncgen.y"
+/* Line 1806 of yacc.c  */
+#line 446 "/home/wfisher/Desktop/netcdf-cmake/ncgen3/ncgen.y"
     {
 		       if (valnum < var_len) { /* leftovers */
 			   nc_fill(valtype,
@@ -1961,13 +1983,13 @@ yyreduce:
 		       if (derror_count == 0)
 			   put_variable(rec_start);
 		       free ((char *) rec_start);
-		 ;}
+		 }
     break;
 
   case 70:
 
-/* Line 1464 of yacc.c  */
-#line 465 "ncgen.y"
+/* Line 1806 of yacc.c  */
+#line 465 "/home/wfisher/Desktop/netcdf-cmake/ncgen3/ncgen.y"
     {
 		       if(valnum >= var_len) {
 			   if (vars[varnum].dims[0] != rec_dim) { /* not recvar */
@@ -1991,13 +2013,13 @@ yyreduce:
 			   }
 		       }
 		       not_a_string = 1;
-                   ;}
+                   }
     break;
 
   case 71:
 
-/* Line 1464 of yacc.c  */
-#line 490 "ncgen.y"
+/* Line 1806 of yacc.c  */
+#line 490 "/home/wfisher/Desktop/netcdf-cmake/ncgen3/ncgen.y"
     {
 		       if (not_a_string) {
 			   switch (valtype) {
@@ -2022,13 +2044,13 @@ yyreduce:
 			     default: break;
 			   }
 		       }
-		   ;}
+		   }
     break;
 
   case 72:
 
-/* Line 1464 of yacc.c  */
-#line 518 "ncgen.y"
+/* Line 1806 of yacc.c  */
+#line 518 "/home/wfisher/Desktop/netcdf-cmake/ncgen3/ncgen.y"
     {
 		       atype_code = NC_CHAR;
 		       switch (valtype) {
@@ -2053,13 +2075,13 @@ yyreduce:
 			 default: break;
 		       }
 		       valnum++;
-		   ;}
+		   }
     break;
 
   case 73:
 
-/* Line 1464 of yacc.c  */
-#line 544 "ncgen.y"
+/* Line 1806 of yacc.c  */
+#line 544 "/home/wfisher/Desktop/netcdf-cmake/ncgen3/ncgen.y"
     {
 		       not_a_string = 0;
 		       atype_code = NC_CHAR;
@@ -2113,13 +2135,13 @@ yyreduce:
 			     default: break;
 			   }
 		       }
-		   ;}
+		   }
     break;
 
   case 74:
 
-/* Line 1464 of yacc.c  */
-#line 599 "ncgen.y"
+/* Line 1806 of yacc.c  */
+#line 599 "/home/wfisher/Desktop/netcdf-cmake/ncgen3/ncgen.y"
     {
 		       atype_code = NC_BYTE;
 		       switch (valtype) {
@@ -2144,13 +2166,13 @@ yyreduce:
 			 default: break;
 		       }
 		       valnum++;
-		   ;}
+		   }
     break;
 
   case 75:
 
-/* Line 1464 of yacc.c  */
-#line 625 "ncgen.y"
+/* Line 1806 of yacc.c  */
+#line 625 "/home/wfisher/Desktop/netcdf-cmake/ncgen3/ncgen.y"
     {
 		       atype_code = NC_SHORT;
 		       switch (valtype) {
@@ -2175,13 +2197,13 @@ yyreduce:
 			 default: break;			
 		       }
 		       valnum++;
-		   ;}
+		   }
     break;
 
   case 76:
 
-/* Line 1464 of yacc.c  */
-#line 651 "ncgen.y"
+/* Line 1806 of yacc.c  */
+#line 651 "/home/wfisher/Desktop/netcdf-cmake/ncgen3/ncgen.y"
     {
 		       atype_code = NC_INT;
 		       switch (valtype) {
@@ -2206,13 +2228,13 @@ yyreduce:
 			 default: break;
 		       }
 		       valnum++;
-		   ;}
+		   }
     break;
 
   case 77:
 
-/* Line 1464 of yacc.c  */
-#line 677 "ncgen.y"
+/* Line 1806 of yacc.c  */
+#line 677 "/home/wfisher/Desktop/netcdf-cmake/ncgen3/ncgen.y"
     {
 		       atype_code = NC_FLOAT;
 		       switch (valtype) {
@@ -2237,13 +2259,13 @@ yyreduce:
 			 default: break;
 		       }
 		       valnum++;
-		   ;}
+		   }
     break;
 
   case 78:
 
-/* Line 1464 of yacc.c  */
-#line 703 "ncgen.y"
+/* Line 1806 of yacc.c  */
+#line 703 "/home/wfisher/Desktop/netcdf-cmake/ncgen3/ncgen.y"
     {
 		       atype_code = NC_DOUBLE;
 		       switch (valtype) {
@@ -2271,13 +2293,13 @@ yyreduce:
 			 default: break;
 		       }
 		       valnum++;
-		   ;}
+		   }
     break;
 
   case 79:
 
-/* Line 1464 of yacc.c  */
-#line 732 "ncgen.y"
+/* Line 1806 of yacc.c  */
+#line 732 "/home/wfisher/Desktop/netcdf-cmake/ncgen3/ncgen.y"
     {
 		       /* store fill_value */
 		       switch (valtype) {
@@ -2308,15 +2330,26 @@ yyreduce:
 			default: break;
 		       }
 		       valnum++;
-		   ;}
+		   }
     break;
 
 
 
-/* Line 1464 of yacc.c  */
-#line 2318 "ncgen.tab.c"
+/* Line 1806 of yacc.c  */
+#line 2340 "ncgen.tab.c"
       default: break;
     }
+  /* User semantic actions sometimes alter yychar, and that requires
+     that yytoken be updated with the new translation.  We take the
+     approach of translating immediately before every use of yytoken.
+     One alternative is translating here after every semantic action,
+     but that translation would be missed if the semantic action invokes
+     YYABORT, YYACCEPT, or YYERROR immediately after altering yychar or
+     if it invokes YYBACKUP.  In the case of YYABORT or YYACCEPT, an
+     incorrect destructor might then be invoked immediately.  In the
+     case of YYERROR or YYBACKUP, subsequent parser actions might lead
+     to an incorrect destructor call or verbose syntax error message
+     before the lookahead is translated.  */
   YY_SYMBOL_PRINT ("-> $$ =", yyr1[yyn], &yyval, &yyloc);
 
   YYPOPSTACK (yylen);
@@ -2344,6 +2377,10 @@ yyreduce:
 | yyerrlab -- here on detecting error |
 `------------------------------------*/
 yyerrlab:
+  /* Make sure we have latest lookahead translation.  See comments at
+     user semantic actions for why this is necessary.  */
+  yytoken = yychar == YYEMPTY ? YYEMPTY : YYTRANSLATE (yychar);
+
   /* If not already recovering from an error, report this error.  */
   if (!yyerrstatus)
     {
@@ -2351,37 +2388,36 @@ yyerrlab:
 #if ! YYERROR_VERBOSE
       yyerror (YY_("syntax error"));
 #else
+# define YYSYNTAX_ERROR yysyntax_error (&yymsg_alloc, &yymsg, \
+                                        yyssp, yytoken)
       {
-	YYSIZE_T yysize = yysyntax_error (0, yystate, yychar);
-	if (yymsg_alloc < yysize && yymsg_alloc < YYSTACK_ALLOC_MAXIMUM)
-	  {
-	    YYSIZE_T yyalloc = 2 * yysize;
-	    if (! (yysize <= yyalloc && yyalloc <= YYSTACK_ALLOC_MAXIMUM))
-	      yyalloc = YYSTACK_ALLOC_MAXIMUM;
-	    if (yymsg != yymsgbuf)
-	      YYSTACK_FREE (yymsg);
-	    yymsg = (char *) YYSTACK_ALLOC (yyalloc);
-	    if (yymsg)
-	      yymsg_alloc = yyalloc;
-	    else
-	      {
-		yymsg = yymsgbuf;
-		yymsg_alloc = sizeof yymsgbuf;
-	      }
-	  }
-
-	if (0 < yysize && yysize <= yymsg_alloc)
-	  {
-	    (void) yysyntax_error (yymsg, yystate, yychar);
-	    yyerror (yymsg);
-	  }
-	else
-	  {
-	    yyerror (YY_("syntax error"));
-	    if (yysize != 0)
-	      goto yyexhaustedlab;
-	  }
+        char const *yymsgp = YY_("syntax error");
+        int yysyntax_error_status;
+        yysyntax_error_status = YYSYNTAX_ERROR;
+        if (yysyntax_error_status == 0)
+          yymsgp = yymsg;
+        else if (yysyntax_error_status == 1)
+          {
+            if (yymsg != yymsgbuf)
+              YYSTACK_FREE (yymsg);
+            yymsg = (char *) YYSTACK_ALLOC (yymsg_alloc);
+            if (!yymsg)
+              {
+                yymsg = yymsgbuf;
+                yymsg_alloc = sizeof yymsgbuf;
+                yysyntax_error_status = 2;
+              }
+            else
+              {
+                yysyntax_error_status = YYSYNTAX_ERROR;
+                yymsgp = yymsg;
+              }
+          }
+        yyerror (yymsgp);
+        if (yysyntax_error_status == 2)
+          goto yyexhaustedlab;
       }
+# undef YYSYNTAX_ERROR
 #endif
     }
 
@@ -2440,7 +2476,7 @@ yyerrlab1:
   for (;;)
     {
       yyn = yypact[yystate];
-      if (yyn != YYPACT_NINF)
+      if (!yypact_value_is_default (yyn))
 	{
 	  yyn += YYTERROR;
 	  if (0 <= yyn && yyn <= YYLAST && yycheck[yyn] == YYTERROR)
@@ -2499,8 +2535,13 @@ yyexhaustedlab:
 
 yyreturn:
   if (yychar != YYEMPTY)
-     yydestruct ("Cleanup: discarding lookahead",
-		 yytoken, &yylval);
+    {
+      /* Make sure we have latest lookahead translation.  See comments at
+         user semantic actions for why this is necessary.  */
+      yytoken = YYTRANSLATE (yychar);
+      yydestruct ("Cleanup: discarding lookahead",
+                  yytoken, &yylval);
+    }
   /* Do not reclaim the symbols of the rule which action triggered
      this YYABORT or YYACCEPT.  */
   YYPOPSTACK (yylen);
@@ -2525,8 +2566,8 @@ yyreturn:
 
 
 
-/* Line 1684 of yacc.c  */
-#line 767 "ncgen.y"
+/* Line 2067 of yacc.c  */
+#line 767 "/home/wfisher/Desktop/netcdf-cmake/ncgen3/ncgen.y"
 
 
 /* HELPER PROGRAMS */
