@@ -29,7 +29,7 @@
 int
 main(int argc, char **argv)
 {
-    EXTERNL int ncopts;		/* netCDF error options */
+	//EXTERNL int ncopts;		/* netCDF error options */
     char *testfiles[] = {"nonesuch", "nctest_classic.nc", 
 			 "nctest_64bit_offset.nc", "nctest_netcdf4.nc"};
     char *testfile;
