@@ -1,21 +1,21 @@
 /*********************************************************************
  *   Copyright 1993, UCAR/Unidata
  *   See netcdf/COPYRIGHT file for copying and redistribution conditions.
- *   $Header: /upc/share/CVS/netcdf/nctest/vartests.c,v 1.12 1996/01/16 23:38:39 steve Exp $
+ *   $Header: /upc/share/CVS/netcdf-3/nctest/vartests.c,v 1.19 2006/10/31 16:20:49 ed Exp $
  *********************************************************************/
 
+#include <config.h>
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>		/* for free() */
 #include "netcdf.h"
 #include "testcdf.h"		/* defines in-memory test cdf structure */
+#include "emalloc.h"
 #include "add.h"		/* functions to update in-memory netcdf */
 #include "error.h"
 #include "tests.h"
 
 #define LEN_OF(array) ((sizeof array) / (sizeof array[0]))
-#define min(A, B)	((A) < (B) ? (A) : (B))
-#define max(A, B)	((A) > (B) ? (A) : (B))
 
 
 /*
@@ -24,28 +24,28 @@
  *    try with undefined name, check error
  *    try with bad handle, check error
  */
-void
+int
 test_ncvarid(path)
-     char *path;		/* name of writable netcdf file to open */
+     const char *path;		/* name of writable netcdf file to open */
 {
-    int nerrs = 0;
     static char pname[] = "test_ncvarid";
     int cdfid;			/* netcdf id */
     int id;
     int varid;			/* variable id */
     static struct cdfvar xx =	/* variable */
       {"xx", NC_FLOAT, 1, ___, 0};
+    int nerrs = 0;
 
     (void) fprintf(stderr, "*** Testing %s ...\t\t", &pname[5]);
 
     if ((cdfid = ncopen(path, NC_WRITE)) == -1) {
 	error("%s: ncopen failed", pname);
-	return;
+	return ++nerrs;
     }
     /* opened, enter define mode */
     if (ncredef(cdfid) == -1) {
 	error("%s: cdredef failed", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     /* in define mode, add a variable */
     xx.dims = (int *) emalloc(sizeof(int) * xx.ndims);
@@ -54,32 +54,32 @@ test_ncvarid(path)
     if ((varid = ncvardef(cdfid,
 			   xx.name, xx.type, xx.ndims, xx.dims)) == -1) {
 	error("%s: ncvardef failed", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     add_var(&test, &xx);	/* keep in-memory netcdf in sync */
 
     /* check id returned for name matches id returned from definition */
     if (ncvarid(cdfid, xx.name) != varid) {
 	error("%s: ncvarid returned wrong value in define mode", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     if (ncendef (cdfid) == -1) {
 	error("%s: ncendef failed", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     /* in data mode, check returned id for variable just added */
     if (ncvarid(cdfid, xx.name) != varid) {
 	error("%s: ncvarid returned wrong value in data mode", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     /* try with undefined variable, should fail */
     if (ncvarid(cdfid, "santa-claus") != -1) {
 	error("%s: ncvarid with bogus name should have failed ", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     if (ncclose (cdfid) == -1) {
 	error("%s: ncclose failed", pname);
-	return;
+	return ++nerrs;
     }
     /* try on bad handle, should fail */
     if (ncvarid(cdfid, xx.name) != -1) {
@@ -90,6 +90,9 @@ test_ncvarid(path)
       (void) fprintf(stderr,"FAILED! ***\n");
     else
       (void) fprintf(stderr,"ok ***\n");
+
+    free(xx.dims);
+    return nerrs;
 }
 
 
@@ -100,9 +103,9 @@ test_ncvarid(path)
  *    try with bad variable handle, check error
  *    try with bad netCDF handle, check error
  */
-void
+int
 test_ncvarinq(path)
-     char *path;		/* name of writable netcdf file to open */
+     const char *path;		/* name of writable netcdf file to open */
 {
     int nerrs = 0;
     static char pname[] = "test_ncvarinq";
@@ -115,7 +118,7 @@ test_ncvarinq(path)
 
     if ((cdfid = ncopen(path, NC_WRITE)) == -1) {
 	error("%s: ncopen failed", pname);
-	return;
+	return ++nerrs;
     }
     /* opened, in data mode */
     var.dims = (int *) emalloc(sizeof(int) * MAX_VAR_DIMS);
@@ -125,7 +128,7 @@ test_ncvarinq(path)
 		      &var.ndims, var.dims, &var.natts) == -1) {
 	    error("%s: ncvarinq in data mode failed on var id %d",
 		  pname, varid);
-	    ncclose(cdfid); return;
+	    ncclose(cdfid); return ++nerrs;
 	}
 	/* compare returned with expected values */
 	if (strcmp(var.name, test.vars[varid].name) != 0) {
@@ -155,7 +158,7 @@ test_ncvarinq(path)
     }
     if (ncredef(cdfid) == -1) {
 	error("%s: ncredef failed", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     /* in define mode, compare returned with expected values again */
     for (varid = 0 ; varid < test.nvars; varid++) { /* loop on all var ids */
@@ -163,7 +166,7 @@ test_ncvarinq(path)
 		      &var.ndims, var.dims, &var.natts) == -1) {
 	    error("%s: ncvarinq in data mode failed on var id %d",
 		  pname, varid);
-	    ncclose(cdfid); return;
+	    ncclose(cdfid); return ++nerrs;
 	}
 	if (strcmp(var.name, test.vars[varid].name) != 0) {
 	    error("%s: ncvarinq (define mode), name %s, expected %s for id = %d",
@@ -197,15 +200,15 @@ test_ncvarinq(path)
 		      &var.ndims, var.dims, &var.natts) != -1) {
 	error("%s: ncvarinq should have failed on bad variable ids",
 	      pname, varid);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     if (ncendef (cdfid) == -1) {
 	error("%s: ncendef failed", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     if (ncclose (cdfid) == -1) {
 	error("%s: ncclose failed", pname);
-	return;
+	return ++nerrs;
     }
     /* should fail, since bad handle */
     if (test.nvars >= 1) {	/* if any variables have been defined */
@@ -215,12 +218,14 @@ test_ncvarinq(path)
 	    nerrs++;
 	}
     }
-    free((char *) var.dims);
+    free(var.dims);
     free(var.name);
     if (nerrs > 0)
       (void) fprintf(stderr,"FAILED! ***\n");
     else
       (void) fprintf(stderr,"ok ***\n");
+
+    return nerrs;
 }
 
 
@@ -243,7 +248,7 @@ struct cdfelm {			/* coordinates and generic value */
  *    for each existing variable, put values of its type at each point
  *    get values and compare with put values
  */
-int
+static int
 test_varputget1(cdfid)
      int cdfid;			/* handle of netcdf open and in data mode */
 {
@@ -267,7 +272,7 @@ test_varputget1(cdfid)
 	    /* "lower-left" corner */
 	    elm[0].coords[id] = 0;
 	    /* if unlimited dimension, choose record 3 for max, arbitrarily */
-	    dsize = test.dims[test.vars[iv].dims[id]].size;
+	    dsize = (int) test.dims[test.vars[iv].dims[id]].size;
 	    if (dsize == NC_UNLIMITED)
 	      dsize = 3;
 	    /* middle */
@@ -360,6 +365,7 @@ test_varputget1(cdfid)
 	    }
 	}
     }
+   
     return nerrs;
 }
 
@@ -373,9 +379,9 @@ test_varputget1(cdfid)
  *    try in define mode, check error
  *    try with bad netCDF handle, check error
  */
-void
+int
 test_ncvarput1(path)
-     char *path;		/* name of writable netcdf file to open */
+     const char *path;		/* name of writable netcdf file to open */
 {
     int nerrs = 0;
     static char pname[] = "test_ncvarput1";
@@ -387,7 +393,7 @@ test_ncvarput1(path)
 
     if ((cdfid = ncopen(path, NC_WRITE)) == -1) {
 	error("%s: ncopen failed", pname);
-	return;
+	return ++nerrs;
     }
     /* opened in data mode, try putting and getting values of each type */
     nerrs += test_varputget1 (cdfid);	/* tests ncvarput1 and ncvarget1 */
@@ -405,38 +411,38 @@ test_ncvarput1(path)
 	elm.coords[test.vars[iv].ndims/2] = -1;
 	if(ncvarput1 (cdfid, iv, elm.coords, (void *) &elm.val) != -1) {
 	    error("%s: ncvarput1 should fail for negative coordinate", pname);
-	    ncclose(cdfid); return;
+	    ncclose(cdfid); return ++nerrs;
 	}
 	elm.coords[test.vars[iv].ndims/2] =
 	  test.dims[test.vars[iv].dims[test.vars[iv].ndims/2]].size;
 	if(ncvarput1 (cdfid, iv, elm.coords, (void *) &elm.val) != -1) {
 	    error("%s: ncvarput1 should fail for too-high coordinate", pname);
-	    ncclose(cdfid); return;
+	    ncclose(cdfid); return ++nerrs;
 	}
     }
     /* try with bad variable handle, should fail */
     if(ncvarput1 (cdfid, -1, elm.coords, (void *) &elm.val) != -1 ||
        ncvarput1 (cdfid, test.nvars, elm.coords, (void *) &elm.val) != -1) {
 	error("%s: ncvarput1 should fail for bad variable handle", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     if (ncredef(cdfid) == -1) {
 	error("%s: ncredef failed", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     /* try in define mode, should fail */
     if (test.nvars > 0)
       if(ncvarput1 (cdfid, 0, elm.coords, (void *) &elm.val) != -1) {
 	  error("%s: ncvarput1 should fail in define mode", pname);
-	  ncclose(cdfid); return;
+	  ncclose(cdfid); return ++nerrs;
       }
     if (ncendef (cdfid) == -1) {
 	error("%s: ncendef failed", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     if (ncclose (cdfid) == -1) {
 	error("%s: ncclose failed", pname);
-	return;
+	return ++nerrs;
     }
     /* try with bad netCDF handle, should fail */
     if(ncvarput1 (cdfid, 0, elm.coords, (void *) &elm.val) != -1) {
@@ -447,6 +453,8 @@ test_ncvarput1(path)
       (void) fprintf(stderr,"FAILED! ***\n");
     else
       (void) fprintf(stderr,"ok ***\n");
+
+    return nerrs;
 }
 
 
@@ -459,9 +467,9 @@ test_ncvarput1(path)
  *    try in define mode, check error
  *    try with bad netCDF handle, check error
  */
-void
+int
 test_ncvarget1(path)
-     char *path;		/* name of writable netcdf file to open */
+     const char *path;		/* name of writable netcdf file to open */
 {
     int nerrs = 0;
     static char pname[] = "test_ncvarget1";
@@ -473,7 +481,7 @@ test_ncvarget1(path)
 
     if ((cdfid = ncopen(path, NC_WRITE)) == -1) {
 	error("%s: ncopen failed", pname);
-	return;
+	return ++nerrs;
     }
     /* opened in data mode, try putting and getting values of each type */
     nerrs += test_varputget1 (cdfid);	/* tests ncvarput1 and ncvarget1 */
@@ -491,39 +499,39 @@ test_ncvarget1(path)
 	elm.coords[test.vars[iv].ndims/2] = -1;
 	if(ncvarget1 (cdfid, iv, elm.coords, (void *) &elm.val) != -1) {
 	    error("%s: ncvarget1 should fail for negative coordinate", pname);
-	    ncclose(cdfid); return;
+	    ncclose(cdfid); return ++nerrs;
 	}
 	elm.coords[test.vars[iv].ndims/2] =
 	  test.dims[test.vars[iv].dims[test.vars[iv].ndims/2]].size;
 	if(ncvarget1 (cdfid, iv, elm.coords, (void *) &elm.val) != -1) {
 	    error("%s: ncvarget1 should fail for too-high coordinate", pname);
-	    ncclose(cdfid); return;
+	    ncclose(cdfid); return ++nerrs;
 	}
     }
     /* try with bad variable handle, should fail */
     if(ncvarget1 (cdfid, -1, elm.coords, (void *) &elm.val) != -1 ||
        ncvarget1 (cdfid, test.nvars, elm.coords, (void *) &elm.val) != -1) {
 	error("%s: ncvarget1 should fail for bad variable handle", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     if (ncredef(cdfid) == -1) {
 	error("%s: ncredef failed", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     /* try in define mode, should fail */
     if (test.nvars > 0)
       if(ncvarget1 (cdfid, 0, elm.coords, (void *) &elm.val) != -1) {
 	  error("%s: ncvarget1 should fail in define mode",
 		pname);
-	  ncclose(cdfid); return;
+	  ncclose(cdfid); return ++nerrs;
       }
     if (ncendef (cdfid) == -1) {
 	error("%s: ncendef failed", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     if (ncclose (cdfid) == -1) {
 	error("%s: ncclose failed", pname);
-	return;
+	return ++nerrs;
     }
     /* try with bad netCDF handle, should fail */
     if(ncvarget1 (cdfid, 0, elm.coords, (void *) &elm.val) != -1) {
@@ -534,6 +542,8 @@ test_ncvarget1(path)
       (void) fprintf(stderr,"FAILED! ***\n");
     else
       (void) fprintf(stderr,"ok ***\n");
+
+    return nerrs;
 }
 
 
@@ -545,9 +555,9 @@ test_ncvarget1(path)
  *    try with bad variable handle, check error
  *    try renaming to existing variable name, check error
  */
-void
+int
 test_ncvarrename(path)
-     char *path;		/* name of writable netcdf file to open */
+     const char *path;		/* name of writable netcdf file to open */
 {
     int nerrs = 0;
     static char pname[] = "test_ncvarrename";
@@ -566,12 +576,12 @@ test_ncvarrename(path)
 
     if ((cdfid = ncopen(path, NC_WRITE)) == -1) {
 	error("%s: ncopen failed", pname);
-	return;
+	return ++nerrs;
     }
     /* opened */
     if (ncredef(cdfid) == -1) {
 	error("%s: ncredef failed", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     /* in define mode, add two variables */
     yy.dims = (int *) emalloc(sizeof(int) * yy.ndims);
@@ -580,7 +590,7 @@ test_ncvarrename(path)
     if ((yy_id = ncvardef(cdfid,
 			   yy.name, yy.type, yy.ndims, yy.dims)) == -1) {
 	error("%s: ncvardef failed", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     add_var(&test, &yy);	/* keep in-memory netcdf in sync */
     zz.dims = (int *) emalloc(sizeof(int) * zz.ndims);
@@ -588,14 +598,14 @@ test_ncvarrename(path)
       zz.dims[id] = id;
     if (ncvardef(cdfid, zz.name, zz.type, zz.ndims, zz.dims) == -1) {
 	error("%s: ncvardef failed", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     add_var(&test, &zz);	/* keep in-memory netcdf in sync */
 
     /* rename first variable */
     if (ncvarrename(cdfid, yy_id, newname) == -1) {
 	error("%s: ncvarrename failed", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     /* check new name with ncvarid, ncvarinq */
     if (yy_id != ncvarid(cdfid, newname)) {
@@ -606,42 +616,42 @@ test_ncvarrename(path)
     if (ncvarinq(cdfid, yy_id, var.name,
 		  &var.type, &var.ndims, var.dims, &var.natts) == -1) {
 	error("%s: ncvarinq failed", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     if (strcmp(var.name,yy.name) == 0) {
 	error("%s: ncvarrename failed to change name", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     if (strcmp(var.name,newname) != 0) {
 	error("%s: ncvarrename changed name to %s instead of %s",
 	      pname, var.name, newname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     (void) strcpy(test.vars[yy_id].name, newname); /* keep test consistent */
     /* try to rename second variable same as first, should fail */
     if (ncvarrename(cdfid, yy_id, zz.name) != -1) {
 	error("%s: ncvarrename should have failed with used name", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     /* try with bad variable handles, check for failure */
     if (ncvarrename(cdfid, -1, var.name) != -1 ||
 	ncvarrename(cdfid, test.nvars, var.name) != -1) {
 	error("%s: ncvarrename should have failed on bad variable ids",
 	      pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     if (ncendef (cdfid) == -1) {
 	error("%s: ncendef failed", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     /* in data mode */
     if (ncvarrename(cdfid, yy_id, "a_longer_name") != -1) {
 	error("%s: ncvarrename to longer should fail in data mode", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     if (ncvarrename(cdfid, yy_id, shortname) == -1) {
 	error("%s: ncvarrename to shorter should succeed in data mode", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     (void) strcpy(test.vars[yy_id].name, shortname); /* keep test consistent */
     /* check new name with ncvarid, ncvarinq */
@@ -651,17 +661,21 @@ test_ncvarrename(path)
     }
     if (ncclose (cdfid) == -1) {
 	error("%s: ncclose failed", pname);
-	return;
+	return ++nerrs;
     }
     /* should fail, since bad handle */
     if (ncvarrename (cdfid, 0, var.name) != -1) {
 	error("%s: ncvarrename failed to report bad netcdf handle ", pname);
 	nerrs++;
     }
+    free(yy.dims);
+    free(zz.dims);
     free(var.name);
-    free((char *)var.dims);
+    free(var.dims);
     if (nerrs > 0)
       (void) fprintf(stderr,"FAILED! ***\n");
     else
       (void) fprintf(stderr,"ok ***\n");
+
+    return nerrs;
 }
